/*     */ package com.netapp.asup.client.serialize;
/*     */ 
/*     */ import com.netapp.asup.client.arma.ARMAFilter;
/*     */ import com.netapp.asup.client.arma.ARMAObject;
/*     */ import com.netapp.asup.client.arma.ARMAObjectFactory;
/*     */ import com.netapp.asup.client.helper.ObjectDataXmlHelper;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectFactoryImpl;
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import java.util.regex.Matcher;
/*     */ import java.util.regex.Pattern;
/*     */ import javax.xml.stream.FactoryConfigurationError;
/*     */ import javax.xml.stream.XMLStreamException;
/*     */ import javax.xml.stream.XMLStreamWriter;
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ObjectDataService
/*     */ {
/*     */   private static final String COMMA_DELIMITER = ",";
/*     */   private static final String ENCODING_TYPE = "UTF-8";
/*     */   private static final String MULTI_PATH_DELIMITER = "\\|";
/*     */   private static final String MULTI_FILTER_ATTR_DELIMITER = "\\^\\^";
/*     */   private static final String MULTI_FILTER_VALUE_DELIMITER = "\\|\\|";
/*     */   private static final String filterAttrRegex = "(\\S+)\\.(\\w+)=([\\S\\s]+)";
/*  35 */   private static final Pattern filterAttrPattern = Pattern.compile("(\\S+)\\.(\\w+)=([\\S\\s]+)");
/*     */   
/*     */   private static final String DOT = ".";
/*     */   
/*     */   private static final String OBJECTS = "objects";
/*  40 */   public static AsupLogger logger = AsupLogger.getLogger(ObjectDataService.class);
/*     */   
/*     */   public String serializeObjectData(RequestIdentifier id, String path, String filter, String objects)
/*     */     throws ARMAException, UnsupportedEncodingException, FactoryConfigurationError, XMLStreamException
/*     */   {
/*  45 */     return serializeObjectData(id, path, filter, objects, false);
/*     */   }
/*     */   
/*     */   public String serializeObjectData(RequestIdentifier id, String path, String filter, String objects, boolean isSecureFlag)
/*     */     throws ARMAException, FactoryConfigurationError, XMLStreamException, UnsupportedEncodingException
/*     */   {
/*  51 */     ARMAObjectFactory factory = new ARMAObjectFactoryImpl();
/*  52 */     ARMAObject root = null;
/*     */     
/*  54 */     switch (id.getType()) {
/*     */     case CLUSTER_ID: 
/*  56 */       root = factory.getRoot(id.getClusterId(), null, isSecureFlag);
/*  57 */       break;
/*     */     
/*     */     case SYSID_SYS_SER_NO: 
/*  60 */       root = factory.getRoot(id.getSystemId(), id.getSysSerNo(), null, isSecureFlag);
/*  61 */       break;
/*     */     
/*     */     case BIZ_KEY: 
/*  64 */       root = factory.getRoot(id.getBizKey(), isSecureFlag);
/*     */     }
/*     */     
/*     */     
/*     */ 
/*  69 */     ByteArrayOutputStream out = new ByteArrayOutputStream();
/*  70 */     XMLStreamWriter writer = ObjectDataXmlHelper.initializeXmlWriter(out);
/*     */     
/*  72 */     if (objects == null) {
/*  73 */       ObjectDataXmlHelper.writeObjectData(writer, root.getObjectName(), root.getInstances());
/*  74 */       ObjectDataXmlHelper.closeXmlWriter(writer);
/*  75 */       return out.toString("UTF-8");
/*     */     }
/*     */     
/*  78 */     Map<String, Map<String, List<String>>> objectToFilterMap = getObjectTofilterMap(filter);
/*     */     
/*  80 */     Map<String, ARMAObject> nameARMAMap = addFiltersAndPath(root, path, objectToFilterMap);
/*     */     
/*  82 */     String[] objectArray = objects.split(",");
/*  83 */     ARMAObject jumpNode = null;
/*  84 */     for (String obj : objectArray) {
/*  85 */       if (!nameARMAMap.containsKey(obj)) {
/*  86 */         jumpNode = root.jumpToNode(obj);
/*  87 */         nameARMAMap.put(jumpNode.getObjectName(), jumpNode);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*  92 */     root.getData(objectArray);
/*     */     
/*  94 */     writer.writeStartElement("objects");
/*  95 */     for (String obj : objectArray) {
/*  96 */       ARMAObject fetchedObject = (ARMAObject)nameARMAMap.get(obj);
/*  97 */       ObjectDataXmlHelper.writeObjectData(writer, fetchedObject.getObjectName(), fetchedObject.getInstances());
/*     */     }
/*  99 */     writer.writeEndElement();
/*     */     
/* 101 */     ObjectDataXmlHelper.closeXmlWriter(writer);
/*     */     
/* 103 */     return out.toString("UTF-8");
/*     */   }
/*     */   
/*     */ 
/*     */   private Map<String, ARMAObject> addFiltersAndPath(ARMAObject root, String path, Map<String, Map<String, List<String>>> objectToFilterMap)
/*     */     throws ARMAException
/*     */   {
/* 110 */     Map<String, ARMAObject> nameARMAMap = new HashMap();
/*     */     
/* 112 */     ARMAObject jumpNode = null;
/* 113 */     Map<String, List<String>> filterMap = null;
/* 114 */     List<String> filterValues = null;
/* 115 */     String[] paths = null;
/* 116 */     List<String> pathAsList = null;
/* 117 */     List<String> processedFilters = new ArrayList();
/* 118 */     if (path != null) {
/* 119 */       String[] multiPath = path.split("\\|");
/*     */       Iterator i$;
/* 121 */       String filterObjName; ARMAFilter armaFilter; for (String connectedPath : multiPath)
/*     */       {
/* 123 */         paths = connectedPath.split(",");
/*     */         
/* 125 */         jumpNode = root.jumpToNode(paths[0]);
/* 126 */         nameARMAMap.put(jumpNode.getObjectName(), jumpNode);
/* 127 */         root.jumpToPath(connectedPath.substring(connectedPath.indexOf(",") + 1).split(","));
/*     */         
/*     */ 
/* 130 */         pathAsList = Arrays.asList(paths);
/* 131 */         for (i$ = objectToFilterMap.keySet().iterator(); i$.hasNext();) { filterObjName = (String)i$.next();
/*     */           
/* 133 */           if (pathAsList.contains(filterObjName))
/*     */           {
/* 135 */             jumpNode = root.jumpToNode(filterObjName);
/* 136 */             nameARMAMap.put(jumpNode.getObjectName(), jumpNode);
/* 137 */             armaFilter = jumpNode.getFilter();
/* 138 */             filterMap = (Map)objectToFilterMap.get(filterObjName);
/* 139 */             for (String filterAttrName : filterMap.keySet()) {
/* 140 */               if (!processedFilters.contains(filterObjName + "." + filterAttrName)) {
/* 141 */                 filterValues = (List)filterMap.get(filterAttrName);
/* 142 */                 if (filterValues.size() == 1) {
/* 143 */                   armaFilter.setFilter(filterAttrName, (String)filterValues.get(0));
/*     */                 } else {
/* 145 */                   armaFilter.setMultiValueFilter(filterAttrName, (String[])filterValues.toArray(new String[0]));
/*     */                 }
/* 147 */                 processedFilters.add(filterObjName + "." + filterAttrName);
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 160 */     return nameARMAMap;
/*     */   }
/*     */   
/*     */   private Map<String, Map<String, List<String>>> getObjectTofilterMap(String filter) throws ARMAException
/*     */   {
/* 165 */     Map<String, Map<String, List<String>>> objectToFilterMap = new HashMap();
/* 166 */     if (filter != null)
/*     */     {
/* 168 */       Matcher matcher = null;
/* 169 */       int tokenCount = 0;
/* 170 */       String filterObjectName = null;
/* 171 */       List<String> filterAttrValues = null;
/*     */       
/* 173 */       for (String filterAttribute : filter.split("\\^\\^"))
/*     */       {
/*     */ 
/* 176 */         matcher = filterAttrPattern.matcher(filterAttribute);
/* 177 */         if (matcher.matches())
/*     */         {
/* 179 */           tokenCount = matcher.groupCount();
/*     */           
/* 181 */           if (tokenCount != 3) {
/* 182 */             throw new ARMAException(logger, null, "error in filter input in the REST URL :" + filterAttribute + " Usage: objectName.attributeName=value1||value2||value3");
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*     */ 
/* 188 */           filterObjectName = matcher.group(1);
/* 189 */           filterAttrValues = Arrays.asList(matcher.group(3).split("\\|\\|"));
/*     */           
/*     */ 
/* 192 */           Map<String, List<String>> filterMap = (Map)objectToFilterMap.get(filterObjectName);
/*     */           
/* 194 */           if (filterMap == null)
/* 195 */             filterMap = new HashMap();
/* 196 */           filterMap.put(matcher.group(2), filterAttrValues);
/* 197 */           objectToFilterMap.put(filterObjectName, filterMap);
/*     */         }
/*     */       }
/*     */     }
/* 201 */     return objectToFilterMap;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\client\serialize\ObjectDataService.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */