/*      */ package com.thoughtworks.paranamer;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.lang.reflect.AccessibleObject;
/*      */ import java.lang.reflect.Constructor;
/*      */ import java.lang.reflect.Method;
/*      */ import java.lang.reflect.Modifier;
/*      */ import java.util.HashMap;
/*      */ import java.util.Map;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class BytecodeReadingParanamer
/*      */   implements Paranamer
/*      */ {
/*   55 */   private static final Map<String, String> primitives = new HashMap() {};
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public static final String __PARANAMER_DATA = "lookupParameterNames java.lang.reflect.AccessibleObject methodOrConstructor \nlookupParameterNames java.lang.reflect.AccessibleObject,boolean methodOrCtor,throwExceptionIfMissing \n";
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String[] lookupParameterNames(AccessibleObject methodOrConstructor)
/*      */   {
/*   68 */     return lookupParameterNames(methodOrConstructor, true);
/*      */   }
/*      */   
/*      */   public String[] lookupParameterNames(AccessibleObject methodOrCtor, boolean throwExceptionIfMissing)
/*      */   {
/*   73 */     Class<?>[] types = null;
/*   74 */     Class<?> declaringClass = null;
/*   75 */     String name = null;
/*   76 */     if ((methodOrCtor instanceof Method)) {
/*   77 */       Method method = (Method)methodOrCtor;
/*   78 */       types = method.getParameterTypes();
/*   79 */       name = method.getName();
/*   80 */       declaringClass = method.getDeclaringClass();
/*      */     } else {
/*   82 */       Constructor<?> constructor = (Constructor)methodOrCtor;
/*   83 */       types = constructor.getParameterTypes();
/*   84 */       declaringClass = constructor.getDeclaringClass();
/*   85 */       name = "<init>";
/*      */     }
/*      */     
/*   88 */     if (types.length == 0) {
/*   89 */       return EMPTY_NAMES;
/*      */     }
/*   91 */     InputStream byteCodeStream = getClassAsStream(declaringClass);
/*   92 */     if (byteCodeStream == null) {
/*   93 */       if (throwExceptionIfMissing) {
/*   94 */         throw new ParameterNamesNotFoundException("Unable to get class bytes");
/*      */       }
/*   96 */       return Paranamer.EMPTY_NAMES;
/*      */     }
/*      */     try
/*      */     {
/*  100 */       ClassReader reader = new ClassReader(byteCodeStream, null);
/*  101 */       TypeCollector visitor = new TypeCollector(name, types, throwExceptionIfMissing, null);
/*  102 */       reader.accept(visitor);
/*  103 */       String[] parameterNamesForMethod = visitor.getParameterNamesForMethod();
/*      */       try {
/*  105 */         byteCodeStream.close();
/*      */       }
/*      */       catch (IOException e) {}
/*  108 */       return parameterNamesForMethod;
/*      */     } catch (IOException e) {
/*  110 */       if (throwExceptionIfMissing)
/*  111 */         throw new ParameterNamesNotFoundException("IoException while reading class bytes", e);
/*      */     }
/*  113 */     return Paranamer.EMPTY_NAMES;
/*      */   }
/*      */   
/*      */ 
/*      */   private InputStream getClassAsStream(Class<?> clazz)
/*      */   {
/*  119 */     ClassLoader classLoader = clazz.getClassLoader();
/*  120 */     if (classLoader == null) {
/*  121 */       classLoader = ClassLoader.getSystemClassLoader();
/*      */     }
/*  123 */     return getClassAsStream(classLoader, clazz.getName());
/*      */   }
/*      */   
/*      */   private InputStream getClassAsStream(ClassLoader classLoader, String className) {
/*  127 */     String name = className.replace('.', '/') + ".class";
/*      */     
/*      */ 
/*  130 */     InputStream asStream = classLoader.getResourceAsStream(name);
/*  131 */     if (asStream == null) {
/*  132 */       asStream = BytecodeReadingParanamer.class.getResourceAsStream(name);
/*      */     }
/*  134 */     return asStream;
/*      */   }
/*      */   
/*      */ 
/*      */   private static class TypeCollector
/*      */   {
/*      */     private static final String COMMA = ",";
/*      */     
/*      */     private final String methodName;
/*      */     
/*      */     private final Class<?>[] parameterTypes;
/*      */     
/*      */     private final boolean throwExceptionIfMissing;
/*      */     
/*      */     private BytecodeReadingParanamer.MethodCollector collector;
/*      */     
/*      */     public static final String __PARANAMER_DATA = "visitMethod int,java.lang.String,java.lang.String access,name,desc \n";
/*      */     
/*      */ 
/*      */     private TypeCollector(String methodName, Class<?>[] parameterTypes, boolean throwExceptionIfMissing)
/*      */     {
/*  155 */       this.methodName = methodName;
/*  156 */       this.parameterTypes = parameterTypes;
/*  157 */       this.throwExceptionIfMissing = throwExceptionIfMissing;
/*  158 */       this.collector = null;
/*      */     }
/*      */     
/*      */     public BytecodeReadingParanamer.MethodCollector visitMethod(int access, String name, String desc)
/*      */     {
/*  163 */       if (this.collector != null) {
/*  164 */         return null;
/*      */       }
/*      */       
/*  167 */       if (!name.equals(this.methodName)) {
/*  168 */         return null;
/*      */       }
/*  170 */       BytecodeReadingParanamer.Type[] argumentTypes = BytecodeReadingParanamer.Type.getArgumentTypes(desc);
/*  171 */       int longOrDoubleQuantity = 0;
/*  172 */       for (BytecodeReadingParanamer.Type t : argumentTypes) {
/*  173 */         if ((t.getClassName().equals("long")) || (t.getClassName().equals("double")))
/*      */         {
/*  175 */           longOrDoubleQuantity++;
/*      */         }
/*      */       }
/*  178 */       int paramCount = argumentTypes.length;
/*      */       
/*  180 */       if (paramCount != this.parameterTypes.length) {
/*  181 */         return null;
/*      */       }
/*  183 */       for (int i = 0; i < argumentTypes.length; i++) {
/*  184 */         if (!correctTypeName(argumentTypes, i).equals(this.parameterTypes[i].getName()))
/*      */         {
/*  186 */           return null;
/*      */         }
/*      */       }
/*  189 */       this.collector = new BytecodeReadingParanamer.MethodCollector(Modifier.isStatic(access) ? 0 : 1, argumentTypes.length + longOrDoubleQuantity, null);
/*      */       
/*  191 */       return this.collector;
/*      */     }
/*      */     
/*      */     private String correctTypeName(BytecodeReadingParanamer.Type[] argumentTypes, int i) {
/*  195 */       String s = argumentTypes[i].getClassName();
/*      */       
/*  197 */       String braces = "";
/*  198 */       while (s.endsWith("[]")) {
/*  199 */         braces = braces + "[";
/*  200 */         s = s.substring(0, s.length() - 2);
/*      */       }
/*  202 */       if (!braces.equals("")) {
/*  203 */         if (BytecodeReadingParanamer.primitives.containsKey(s)) {
/*  204 */           s = braces + (String)BytecodeReadingParanamer.primitives.get(s);
/*      */         } else {
/*  206 */           s = braces + "L" + s + ";";
/*      */         }
/*      */       }
/*  209 */       return s;
/*      */     }
/*      */     
/*      */     private String[] getParameterNamesForMethod() {
/*  213 */       if (this.collector == null) {
/*  214 */         return Paranamer.EMPTY_NAMES;
/*      */       }
/*  216 */       if (!this.collector.isDebugInfoPresent()) {
/*  217 */         if (this.throwExceptionIfMissing) {
/*  218 */           throw new ParameterNamesNotFoundException("Parameter names not found for " + this.methodName);
/*      */         }
/*  220 */         return Paranamer.EMPTY_NAMES;
/*      */       }
/*      */       
/*  223 */       return this.collector.getResult().split(",");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static class MethodCollector
/*      */   {
/*      */     private final int paramCount;
/*      */     
/*      */     private final int ignoreCount;
/*      */     
/*      */     private int currentParameter;
/*      */     
/*      */     private final StringBuffer result;
/*      */     
/*      */     private boolean debugInfoPresent;
/*      */     
/*      */     public static final String __PARANAMER_DATA = "visitLocalVariable java.lang.String,int name,index \n";
/*      */     
/*      */ 
/*      */     private MethodCollector(int ignoreCount, int paramCount)
/*      */     {
/*  246 */       this.ignoreCount = ignoreCount;
/*  247 */       this.paramCount = paramCount;
/*  248 */       this.result = new StringBuffer();
/*  249 */       this.currentParameter = 0;
/*      */       
/*  251 */       this.debugInfoPresent = (paramCount == 0);
/*      */     }
/*      */     
/*      */     public void visitLocalVariable(String name, int index) {
/*  255 */       if ((index >= this.ignoreCount) && (index < this.ignoreCount + this.paramCount)) {
/*  256 */         if (!name.equals("arg" + this.currentParameter)) {
/*  257 */           this.debugInfoPresent = true;
/*      */         }
/*  259 */         this.result.append(',');
/*  260 */         this.result.append(name);
/*  261 */         this.currentParameter += 1;
/*      */       }
/*      */     }
/*      */     
/*      */     private String getResult() {
/*  266 */       return this.result.length() != 0 ? this.result.substring(1) : "";
/*      */     }
/*      */     
/*      */     private boolean isDebugInfoPresent() {
/*  270 */       return this.debugInfoPresent;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static class ClassReader
/*      */   {
/*      */     public final byte[] b;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private final int[] items;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private final String[] strings;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private final int maxStringLength;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public final int header;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int FIELD = 9;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int METH = 10;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int IMETH = 11;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int INT = 3;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int FLOAT = 4;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int LONG = 5;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int DOUBLE = 6;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int NAME_TYPE = 12;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     static final int UTF8 = 1;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public static final String __PARANAMER_DATA = "";
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private ClassReader(byte[] b)
/*      */     {
/*  409 */       this(b, 0);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private ClassReader(byte[] b, int off)
/*      */     {
/*  419 */       this.b = b;
/*      */       
/*  421 */       this.items = new int[readUnsignedShort(off + 8)];
/*  422 */       int n = this.items.length;
/*  423 */       this.strings = new String[n];
/*  424 */       int max = 0;
/*  425 */       int index = off + 10;
/*  426 */       for (int i = 1; i < n; i++) {
/*  427 */         this.items[i] = (index + 1);
/*      */         int size;
/*  429 */         switch (b[index]) {
/*      */         case 3: 
/*      */         case 4: 
/*      */         case 9: 
/*      */         case 10: 
/*      */         case 11: 
/*      */         case 12: 
/*  436 */           size = 5;
/*  437 */           break;
/*      */         case 5: 
/*      */         case 6: 
/*  440 */           size = 9;
/*  441 */           i++;
/*  442 */           break;
/*      */         case 1: 
/*  444 */           size = 3 + readUnsignedShort(index + 1);
/*  445 */           if (size > max) {
/*  446 */             max = size;
/*      */           }
/*      */           break;
/*      */         case 2: case 7: 
/*      */         case 8: 
/*      */         default: 
/*  452 */           size = 3;
/*      */         }
/*      */         
/*  455 */         index += size;
/*      */       }
/*  457 */       this.maxStringLength = max;
/*      */       
/*  459 */       this.header = index;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private ClassReader(InputStream is)
/*      */       throws IOException
/*      */     {
/*  470 */       this(readClass(is));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static byte[] readClass(InputStream is)
/*      */       throws IOException
/*      */     {
/*  481 */       if (is == null) {
/*  482 */         throw new IOException("Class not found");
/*      */       }
/*  484 */       byte[] b = new byte[is.available()];
/*  485 */       int len = 0;
/*      */       for (;;) {
/*  487 */         int n = is.read(b, len, b.length - len);
/*  488 */         if (n == -1) {
/*  489 */           if (len < b.length) {
/*  490 */             byte[] c = new byte[len];
/*  491 */             System.arraycopy(b, 0, c, 0, len);
/*  492 */             b = c;
/*      */           }
/*  494 */           return b;
/*      */         }
/*  496 */         len += n;
/*  497 */         if (len == b.length) {
/*  498 */           int last = is.read();
/*  499 */           if (last < 0) {
/*  500 */             return b;
/*      */           }
/*  502 */           byte[] c = new byte[b.length + 1000];
/*  503 */           System.arraycopy(b, 0, c, 0, len);
/*  504 */           c[(len++)] = ((byte)last);
/*  505 */           b = c;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private void accept(BytecodeReadingParanamer.TypeCollector classVisitor)
/*      */     {
/*  522 */       char[] c = new char[this.maxStringLength];
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  527 */       int anns = 0;
/*  528 */       int ianns = 0;
/*      */       
/*      */ 
/*  531 */       int u = this.header;
/*  532 */       int v = this.items[readUnsignedShort(u + 4)];
/*  533 */       int len = readUnsignedShort(u + 6);
/*  534 */       int w = 0;
/*  535 */       u += 8;
/*  536 */       for (int i = 0; i < len; i++) {
/*  537 */         u += 2;
/*      */       }
/*  539 */       v = u;
/*  540 */       i = readUnsignedShort(v);
/*  541 */       v += 2;
/*  542 */       for (; i > 0; i--) {
/*  543 */         int j = readUnsignedShort(v + 6);
/*  544 */         v += 8;
/*  545 */         for (; j > 0; j--) {
/*  546 */           v += 6 + readInt(v + 2);
/*      */         }
/*      */       }
/*  549 */       i = readUnsignedShort(v);
/*  550 */       v += 2;
/*  551 */       for (; i > 0; i--) {
/*  552 */         int j = readUnsignedShort(v + 6);
/*  553 */         v += 8;
/*  554 */         for (; j > 0; j--) {
/*  555 */           v += 6 + readInt(v + 2);
/*      */         }
/*      */       }
/*      */       
/*  559 */       i = readUnsignedShort(v);
/*  560 */       v += 2;
/*  561 */       for (; i > 0; i--) {
/*  562 */         v += 6 + readInt(v + 2);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  568 */       i = readUnsignedShort(u);
/*  569 */       u += 2;
/*  570 */       for (; i > 0; i--) {
/*  571 */         int j = readUnsignedShort(u + 6);
/*  572 */         u += 8;
/*  573 */         for (; j > 0; j--) {
/*  574 */           u += 6 + readInt(u + 2);
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*  579 */       i = readUnsignedShort(u);
/*  580 */       u += 2;
/*  581 */       for (; i > 0; i--)
/*      */       {
/*  583 */         u = readMethod(classVisitor, c, u);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private int readMethod(BytecodeReadingParanamer.TypeCollector classVisitor, char[] c, int u)
/*      */     {
/*  594 */       int access = readUnsignedShort(u);
/*  595 */       String name = readUTF8(u + 2, c);
/*  596 */       String desc = readUTF8(u + 4, c);
/*  597 */       int v = 0;
/*  598 */       int w = 0;
/*      */       
/*      */ 
/*  601 */       int j = readUnsignedShort(u + 6);
/*  602 */       u += 8;
/*  603 */       for (; j > 0; j--) {
/*  604 */         String attrName = readUTF8(u, c);
/*  605 */         int attrSize = readInt(u + 2);
/*  606 */         u += 6;
/*      */         
/*      */ 
/*  609 */         if (attrName.equals("Code")) {
/*  610 */           v = u;
/*      */         }
/*  612 */         u += attrSize;
/*      */       }
/*      */       
/*  615 */       if (w != 0)
/*      */       {
/*  617 */         w += 2;
/*  618 */         for (j = 0; j < readUnsignedShort(w); j++) {
/*  619 */           w += 2;
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*  624 */       BytecodeReadingParanamer.MethodCollector mv = classVisitor.visitMethod(access, name, desc);
/*      */       
/*  626 */       if ((mv != null) && (v != 0)) {
/*  627 */         int codeLength = readInt(v + 4);
/*  628 */         v += 8;
/*      */         
/*  630 */         int codeStart = v;
/*  631 */         int codeEnd = v + codeLength;
/*  632 */         v = codeEnd;
/*      */         
/*  634 */         j = readUnsignedShort(v);
/*  635 */         v += 2;
/*  636 */         for (; j > 0; j--) {
/*  637 */           v += 8;
/*      */         }
/*      */         
/*      */ 
/*  641 */         int varTable = 0;
/*  642 */         int varTypeTable = 0;
/*  643 */         j = readUnsignedShort(v);
/*  644 */         v += 2;
/*  645 */         for (; j > 0; j--) {
/*  646 */           String attrName = readUTF8(v, c);
/*  647 */           if (attrName.equals("LocalVariableTable")) {
/*  648 */             varTable = v + 6;
/*  649 */           } else if (attrName.equals("LocalVariableTypeTable")) {
/*  650 */             varTypeTable = v + 6;
/*      */           }
/*  652 */           v += 6 + readInt(v + 2);
/*      */         }
/*      */         
/*  655 */         v = codeStart;
/*      */         
/*  657 */         if (varTable != 0) {
/*  658 */           if (varTypeTable != 0) {
/*  659 */             int k = readUnsignedShort(varTypeTable) * 3;
/*  660 */             w = varTypeTable + 2;
/*  661 */             int[] typeTable = new int[k];
/*  662 */             while (k > 0) {
/*  663 */               typeTable[(--k)] = (w + 6);
/*  664 */               typeTable[(--k)] = readUnsignedShort(w + 8);
/*  665 */               typeTable[(--k)] = readUnsignedShort(w);
/*  666 */               w += 10;
/*      */             }
/*      */           }
/*  669 */           int k = readUnsignedShort(varTable);
/*  670 */           w = varTable + 2;
/*  671 */           for (; k > 0; k--) {
/*  672 */             int index = readUnsignedShort(w + 8);
/*  673 */             mv.visitLocalVariable(readUTF8(w + 4, c), index);
/*  674 */             w += 10;
/*      */           }
/*      */         }
/*      */       }
/*  678 */       return u;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private int readUnsignedShort(int index)
/*      */     {
/*  690 */       byte[] b = this.b;
/*  691 */       return (b[index] & 0xFF) << 8 | b[(index + 1)] & 0xFF;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private int readInt(int index)
/*      */     {
/*  703 */       byte[] b = this.b;
/*  704 */       return (b[index] & 0xFF) << 24 | (b[(index + 1)] & 0xFF) << 16 | (b[(index + 2)] & 0xFF) << 8 | b[(index + 3)] & 0xFF;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private String readUTF8(int index, char[] buf)
/*      */     {
/*  720 */       int item = readUnsignedShort(index);
/*  721 */       String s = this.strings[item];
/*  722 */       if (s != null) {
/*  723 */         return s;
/*      */       }
/*  725 */       index = this.items[item];
/*  726 */       return this.strings[item] = readUTF(index + 2, readUnsignedShort(index), buf);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private String readUTF(int index, int utfLen, char[] buf)
/*      */     {
/*  739 */       int endIndex = index + utfLen;
/*  740 */       byte[] b = this.b;
/*  741 */       int strLen = 0;
/*      */       
/*  743 */       int st = 0;
/*  744 */       char cc = '\000';
/*  745 */       while (index < endIndex) {
/*  746 */         int c = b[(index++)];
/*  747 */         switch (st) {
/*      */         case 0: 
/*  749 */           c &= 0xFF;
/*  750 */           if (c < 128) {
/*  751 */             buf[(strLen++)] = ((char)c);
/*  752 */           } else if ((c < 224) && (c > 191)) {
/*  753 */             cc = (char)(c & 0x1F);
/*  754 */             st = 1;
/*      */           } else {
/*  756 */             cc = (char)(c & 0xF);
/*  757 */             st = 2;
/*      */           }
/*  759 */           break;
/*      */         
/*      */         case 1: 
/*  762 */           buf[(strLen++)] = ((char)(cc << '\006' | c & 0x3F));
/*  763 */           st = 0;
/*  764 */           break;
/*      */         
/*      */         case 2: 
/*  767 */           cc = (char)(cc << '\006' | c & 0x3F);
/*  768 */           st = 1;
/*      */         }
/*      */         
/*      */       }
/*  772 */       return new String(buf, 0, strLen);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static class Type
/*      */   {
/*      */     private static final int VOID = 0;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int BOOLEAN = 1;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int CHAR = 2;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int BYTE = 3;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int SHORT = 4;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int INT = 5;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int FLOAT = 6;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int LONG = 7;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int DOUBLE = 8;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int ARRAY = 9;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static final int OBJECT = 10;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  874 */     private static final Type VOID_TYPE = new Type(0, null, 1443168256, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  880 */     private static final Type BOOLEAN_TYPE = new Type(1, null, 1509950721, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  886 */     private static final Type CHAR_TYPE = new Type(2, null, 1124075009, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  892 */     private static final Type BYTE_TYPE = new Type(3, null, 1107297537, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  898 */     private static final Type SHORT_TYPE = new Type(4, null, 1392510721, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  904 */     private static final Type INT_TYPE = new Type(5, null, 1224736769, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  910 */     private static final Type FLOAT_TYPE = new Type(6, null, 1174536705, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  916 */     private static final Type LONG_TYPE = new Type(7, null, 1241579778, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  922 */     private static final Type DOUBLE_TYPE = new Type(8, null, 1141048066, 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private final int sort;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private char[] buf;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private int off;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private final int len;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public static final String __PARANAMER_DATA = "";
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private Type(int sort)
/*      */     {
/*  963 */       this.sort = sort;
/*  964 */       this.len = 1;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private Type(int sort, char[] buf, int off, int len)
/*      */     {
/*  976 */       this.sort = sort;
/*  977 */       this.buf = buf;
/*  978 */       this.off = off;
/*  979 */       this.len = len;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static Type[] getArgumentTypes(String methodDescriptor)
/*      */     {
/*  992 */       char[] buf = methodDescriptor.toCharArray();
/*  993 */       int off = 1;
/*  994 */       int size = 0;
/*      */       for (;;) {
/*  996 */         char car = buf[(off++)];
/*  997 */         if (car == ')')
/*      */           break;
/*  999 */         if (car == 'L') {
/* 1000 */           while (buf[(off++)] != ';') {}
/*      */           
/* 1002 */           size++;
/* 1003 */         } else if (car != '[') {
/* 1004 */           size++;
/*      */         }
/*      */       }
/*      */       
/* 1008 */       Type[] args = new Type[size];
/* 1009 */       off = 1;
/* 1010 */       size = 0;
/* 1011 */       while (buf[off] != ')') {
/* 1012 */         args[size] = getType(buf, off);
/* 1013 */         off += args[size].len + (args[size].sort == 10 ? 2 : 0);
/* 1014 */         size++;
/*      */       }
/* 1016 */       return args;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static Type getType(char[] buf, int off)
/*      */     {
/* 1029 */       switch (buf[off]) {
/*      */       case 'V': 
/* 1031 */         return VOID_TYPE;
/*      */       case 'Z': 
/* 1033 */         return BOOLEAN_TYPE;
/*      */       case 'C': 
/* 1035 */         return CHAR_TYPE;
/*      */       case 'B': 
/* 1037 */         return BYTE_TYPE;
/*      */       case 'S': 
/* 1039 */         return SHORT_TYPE;
/*      */       case 'I': 
/* 1041 */         return INT_TYPE;
/*      */       case 'F': 
/* 1043 */         return FLOAT_TYPE;
/*      */       case 'J': 
/* 1045 */         return LONG_TYPE;
/*      */       case 'D': 
/* 1047 */         return DOUBLE_TYPE;
/*      */       case '[': 
/* 1049 */         len = 1;
/* 1050 */         while (buf[(off + len)] == '[') {
/* 1051 */           len++;
/*      */         }
/* 1053 */         if (buf[(off + len)] == 'L') {
/* 1054 */           len++;
/* 1055 */           while (buf[(off + len)] != ';') {
/* 1056 */             len++;
/*      */           }
/*      */         }
/* 1059 */         return new Type(9, buf, off, len + 1);
/*      */       }
/*      */       
/* 1062 */       int len = 1;
/* 1063 */       while (buf[(off + len)] != ';') {
/* 1064 */         len++;
/*      */       }
/* 1066 */       return new Type(10, buf, off + 1, len - 1);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private int getDimensions()
/*      */     {
/* 1081 */       int i = 1;
/* 1082 */       while (this.buf[(this.off + i)] == '[') {
/* 1083 */         i++;
/*      */       }
/* 1085 */       return i;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private Type getElementType()
/*      */     {
/* 1095 */       return getType(this.buf, this.off + getDimensions());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private String getClassName()
/*      */     {
/* 1104 */       switch (this.sort) {
/*      */       case 0: 
/* 1106 */         return "void";
/*      */       case 1: 
/* 1108 */         return "boolean";
/*      */       case 2: 
/* 1110 */         return "char";
/*      */       case 3: 
/* 1112 */         return "byte";
/*      */       case 4: 
/* 1114 */         return "short";
/*      */       case 5: 
/* 1116 */         return "int";
/*      */       case 6: 
/* 1118 */         return "float";
/*      */       case 7: 
/* 1120 */         return "long";
/*      */       case 8: 
/* 1122 */         return "double";
/*      */       case 9: 
/* 1124 */         StringBuffer b = new StringBuffer(getElementType().getClassName());
/* 1125 */         for (int i = getDimensions(); i > 0; i--) {
/* 1126 */           b.append("[]");
/*      */         }
/* 1128 */         return b.toString();
/*      */       }
/*      */       
/* 1131 */       return new String(this.buf, this.off, this.len).replace('/', '.');
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\thoughtworks\paranamer\BytecodeReadingParanamer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */