/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.EOFException;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.nio.ByteBuffer;
/*     */ import org.apache.avro.util.ByteBufferInputStream;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ class DirectBinaryDecoder
/*     */   extends BinaryDecoder
/*     */ {
/*     */   private InputStream in;
/*     */   private ByteReader byteReader;
/*     */   
/*     */   private class ByteReader
/*     */   {
/*     */     private ByteReader() {}
/*     */     
/*     */     public ByteBuffer read(ByteBuffer old, int length)
/*     */       throws IOException
/*     */     {
/*     */       ByteBuffer result;
/*  43 */       if ((old != null) && (length <= old.capacity())) {
/*  44 */         ByteBuffer result = old;
/*  45 */         result.clear();
/*     */       } else {
/*  47 */         result = ByteBuffer.allocate(length);
/*     */       }
/*  49 */       DirectBinaryDecoder.this.doReadBytes(result.array(), result.position(), length);
/*  50 */       result.limit(length);
/*  51 */       return result;
/*     */     }
/*     */   }
/*     */   
/*     */   private class ReuseByteReader extends DirectBinaryDecoder.ByteReader {
/*     */     private final ByteBufferInputStream bbi;
/*     */     
/*  58 */     public ReuseByteReader(ByteBufferInputStream bbi) { super(null);
/*  59 */       this.bbi = bbi;
/*     */     }
/*     */     
/*     */     public ByteBuffer read(ByteBuffer old, int length) throws IOException
/*     */     {
/*  64 */       if (old != null) {
/*  65 */         return super.read(old, length);
/*     */       }
/*  67 */       return this.bbi.readBuffer(length);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   DirectBinaryDecoder(InputStream in)
/*     */   {
/*  77 */     configure(in);
/*     */   }
/*     */   
/*     */   DirectBinaryDecoder configure(InputStream in) {
/*  81 */     this.in = in;
/*  82 */     this.byteReader = ((in instanceof ByteBufferInputStream) ? new ReuseByteReader((ByteBufferInputStream)in) : new ByteReader(null));
/*     */     
/*  84 */     return this;
/*     */   }
/*     */   
/*     */   public boolean readBoolean() throws IOException
/*     */   {
/*  89 */     int n = this.in.read();
/*  90 */     if (n < 0) {
/*  91 */       throw new EOFException();
/*     */     }
/*  93 */     return n == 1;
/*     */   }
/*     */   
/*     */   public int readInt() throws IOException
/*     */   {
/*  98 */     int n = 0;
/*     */     
/* 100 */     int shift = 0;
/*     */     do {
/* 102 */       int b = this.in.read();
/* 103 */       if (b >= 0) {
/* 104 */         n |= (b & 0x7F) << shift;
/* 105 */         if ((b & 0x80) == 0) {
/* 106 */           return n >>> 1 ^ -(n & 0x1);
/*     */         }
/*     */       } else {
/* 109 */         throw new EOFException();
/*     */       }
/* 111 */       shift += 7;
/* 112 */     } while (shift < 32);
/* 113 */     throw new IOException("Invalid int encoding");
/*     */   }
/*     */   
/*     */   public long readLong()
/*     */     throws IOException
/*     */   {
/* 119 */     long n = 0L;
/*     */     
/* 121 */     int shift = 0;
/*     */     do {
/* 123 */       int b = this.in.read();
/* 124 */       if (b >= 0) {
/* 125 */         n |= (b & 0x7F) << shift;
/* 126 */         if ((b & 0x80) == 0) {
/* 127 */           return n >>> 1 ^ -(n & 1L);
/*     */         }
/*     */       } else {
/* 130 */         throw new EOFException();
/*     */       }
/* 132 */       shift += 7;
/* 133 */     } while (shift < 64);
/* 134 */     throw new IOException("Invalid long encoding");
/*     */   }
/*     */   
/* 137 */   private final byte[] buf = new byte[8];
/*     */   
/*     */   public float readFloat() throws IOException
/*     */   {
/* 141 */     doReadBytes(this.buf, 0, 4);
/* 142 */     int n = this.buf[0] & 0xFF | (this.buf[1] & 0xFF) << 8 | (this.buf[2] & 0xFF) << 16 | (this.buf[3] & 0xFF) << 24;
/*     */     
/*     */ 
/*     */ 
/* 146 */     return Float.intBitsToFloat(n);
/*     */   }
/*     */   
/*     */   public double readDouble() throws IOException
/*     */   {
/* 151 */     doReadBytes(this.buf, 0, 8);
/* 152 */     long n = this.buf[0] & 0xFF | (this.buf[1] & 0xFF) << 8 | (this.buf[2] & 0xFF) << 16 | (this.buf[3] & 0xFF) << 24 | (this.buf[4] & 0xFF) << 32 | (this.buf[5] & 0xFF) << 40 | (this.buf[6] & 0xFF) << 48 | (this.buf[7] & 0xFF) << 56;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 160 */     return Double.longBitsToDouble(n);
/*     */   }
/*     */   
/*     */   public ByteBuffer readBytes(ByteBuffer old) throws IOException
/*     */   {
/* 165 */     int length = readInt();
/* 166 */     return this.byteReader.read(old, length);
/*     */   }
/*     */   
/*     */   protected void doSkipBytes(long length) throws IOException
/*     */   {
/* 171 */     while (length > 0L) {
/* 172 */       long n = this.in.skip(length);
/* 173 */       if (n <= 0L) {
/* 174 */         throw new EOFException();
/*     */       }
/* 176 */       length -= n;
/*     */     }
/*     */   }
/*     */   
/*     */   protected void doReadBytes(byte[] bytes, int start, int length) throws IOException
/*     */   {
/*     */     for (;;)
/*     */     {
/* 184 */       int n = this.in.read(bytes, start, length);
/* 185 */       if ((n == length) || (length == 0))
/* 186 */         return;
/* 187 */       if (n < 0) {
/* 188 */         throw new EOFException();
/*     */       }
/* 190 */       start += n;
/* 191 */       length -= n;
/*     */     }
/*     */   }
/*     */   
/*     */   public InputStream inputStream()
/*     */   {
/* 197 */     return this.in;
/*     */   }
/*     */   
/*     */   public boolean isEnd() throws IOException
/*     */   {
/* 202 */     throw new UnsupportedOperationException();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\DirectBinaryDecoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */