/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.BeanProperty.Std;
/*     */ import org.codehaus.jackson.map.DeserializationConfig;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.DeserializerProvider;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.ResolvableDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.annotate.JsonCachable;
/*     */ import org.codehaus.jackson.map.deser.impl.BeanPropertyMap;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedClass;
/*     */ import org.codehaus.jackson.map.type.ClassKey;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.util.TokenBuffer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ @JsonCachable
/*     */ public class BeanDeserializer
/*     */   extends StdDeserializer<Object>
/*     */   implements ResolvableDeserializer
/*     */ {
/*     */   protected final AnnotatedClass _forClass;
/*     */   protected final JavaType _beanType;
/*     */   protected final BeanProperty _property;
/*     */   protected final Constructor<?> _defaultConstructor;
/*     */   protected final Creator.StringBased _stringCreator;
/*     */   protected final Creator.NumberBased _numberCreator;
/*     */   protected final Creator.Delegating _delegatingCreator;
/*     */   protected final Creator.PropertyBased _propertyBasedCreator;
/*     */   protected final BeanPropertyMap _beanProperties;
/*     */   protected final SettableAnyProperty _anySetter;
/*     */   protected final HashSet<String> _ignorableProps;
/*     */   protected final boolean _ignoreAllUnknown;
/*     */   protected final Map<String, SettableBeanProperty> _backRefs;
/*     */   protected HashMap<ClassKey, JsonDeserializer<Object>> _subDeserializers;
/*     */   
/*     */   public BeanDeserializer(AnnotatedClass forClass, JavaType type, BeanProperty property, CreatorContainer creators, BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs, HashSet<String> ignorableProps, boolean ignoreAllUnknown, SettableAnyProperty anySetter)
/*     */   {
/* 171 */     super(type);
/* 172 */     this._forClass = forClass;
/* 173 */     this._beanType = type;
/* 174 */     this._property = property;
/* 175 */     this._beanProperties = properties;
/* 176 */     this._backRefs = backRefs;
/* 177 */     this._ignorableProps = ignorableProps;
/* 178 */     this._ignoreAllUnknown = ignoreAllUnknown;
/* 179 */     this._anySetter = anySetter;
/*     */     
/*     */ 
/* 182 */     this._stringCreator = creators.stringCreator();
/* 183 */     this._numberCreator = creators.numberCreator();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 190 */     this._delegatingCreator = creators.delegatingCreator();
/* 191 */     this._propertyBasedCreator = creators.propertyBasedCreator();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 196 */     if ((this._delegatingCreator != null) || (this._propertyBasedCreator != null)) {
/* 197 */       this._defaultConstructor = null;
/*     */     } else {
/* 199 */       this._defaultConstructor = creators.getDefaultConstructor();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BeanDeserializer(BeanDeserializer src)
/*     */   {
/* 212 */     super(src._beanType);
/* 213 */     this._forClass = src._forClass;
/* 214 */     this._beanType = src._beanType;
/* 215 */     this._property = src._property;
/* 216 */     this._beanProperties = src._beanProperties;
/* 217 */     this._backRefs = src._backRefs;
/* 218 */     this._ignorableProps = src._ignorableProps;
/* 219 */     this._ignoreAllUnknown = src._ignoreAllUnknown;
/* 220 */     this._anySetter = src._anySetter;
/*     */     
/*     */ 
/* 223 */     this._defaultConstructor = src._defaultConstructor;
/* 224 */     this._stringCreator = src._stringCreator;
/* 225 */     this._numberCreator = src._numberCreator;
/* 226 */     this._delegatingCreator = src._delegatingCreator;
/* 227 */     this._propertyBasedCreator = src._propertyBasedCreator;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean hasProperty(String propertyName)
/*     */   {
/* 237 */     return this._beanProperties.find(propertyName) != null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getPropertyCount()
/*     */   {
/* 246 */     return this._beanProperties.size();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolve(DeserializationConfig config, DeserializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/* 263 */     Iterator<SettableBeanProperty> it = this._beanProperties.allProperties();
/* 264 */     while (it.hasNext()) {
/* 265 */       SettableBeanProperty prop = (SettableBeanProperty)it.next();
/*     */       
/* 267 */       if (!prop.hasValueDeserializer()) {
/* 268 */         prop.setValueDeserializer(findDeserializer(config, provider, prop.getType(), prop));
/*     */       }
/*     */       
/* 271 */       String refName = prop.getManagedReferenceName();
/* 272 */       if (refName != null) {
/* 273 */         JsonDeserializer<?> valueDeser = prop._valueDeserializer;
/* 274 */         SettableBeanProperty backProp = null;
/* 275 */         boolean isContainer = false;
/* 276 */         if ((valueDeser instanceof BeanDeserializer)) {
/* 277 */           backProp = ((BeanDeserializer)valueDeser).findBackReference(refName);
/* 278 */         } else if ((valueDeser instanceof ContainerDeserializer)) {
/* 279 */           JsonDeserializer<?> contentDeser = ((ContainerDeserializer)valueDeser).getContentDeserializer();
/* 280 */           if (!(contentDeser instanceof BeanDeserializer)) {
/* 281 */             throw new IllegalArgumentException("Can not handle managed/back reference '" + refName + "': value deserializer is of type ContainerDeserializer, but content type is not handled by a BeanDeserializer " + " (instead it's of type " + contentDeser.getClass().getName() + ")");
/*     */           }
/*     */           
/*     */ 
/* 285 */           backProp = ((BeanDeserializer)contentDeser).findBackReference(refName);
/* 286 */           isContainer = true;
/* 287 */         } else { if ((valueDeser instanceof AbstractDeserializer)) {
/* 288 */             throw new IllegalArgumentException("Can not handle managed/back reference for abstract types (property " + this._beanType.getRawClass().getName() + "." + prop.getName() + ")");
/*     */           }
/* 290 */           throw new IllegalArgumentException("Can not handle managed/back reference '" + refName + "': type for value deserializer is not BeanDeserializer or ContainerDeserializer, but " + valueDeser.getClass().getName());
/*     */         }
/*     */         
/*     */ 
/* 294 */         if (backProp == null) {
/* 295 */           throw new IllegalArgumentException("Can not handle managed/back reference '" + refName + "': no back reference property found from type " + prop.getType());
/*     */         }
/*     */         
/*     */ 
/* 299 */         JavaType referredType = this._beanType;
/* 300 */         JavaType backRefType = backProp.getType();
/* 301 */         if (!backRefType.getRawClass().isAssignableFrom(referredType.getRawClass())) {
/* 302 */           throw new IllegalArgumentException("Can not handle managed/back reference '" + refName + "': back reference type (" + backRefType.getRawClass().getName() + ") not compatible with managed type (" + referredType.getRawClass().getName() + ")");
/*     */         }
/*     */         
/*     */ 
/* 306 */         this._beanProperties.replace(new SettableBeanProperty.ManagedReferenceProperty(refName, prop, backProp, this._forClass.getAnnotations(), isContainer));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 312 */     if ((this._anySetter != null) && (!this._anySetter.hasValueDeserializer())) {
/* 313 */       this._anySetter.setValueDeserializer(findDeserializer(config, provider, this._anySetter.getType(), this._anySetter.getProperty()));
/*     */     }
/*     */     
/*     */ 
/* 317 */     if (this._delegatingCreator != null)
/*     */     {
/* 319 */       BeanProperty.Std property = new BeanProperty.Std(null, this._delegatingCreator.getValueType(), this._forClass.getAnnotations(), this._delegatingCreator.getCreator());
/*     */       
/* 321 */       JsonDeserializer<Object> deser = findDeserializer(config, provider, this._delegatingCreator.getValueType(), property);
/* 322 */       this._delegatingCreator.setDeserializer(deser);
/*     */     }
/*     */     
/* 325 */     if (this._propertyBasedCreator != null) {
/* 326 */       for (SettableBeanProperty prop : this._propertyBasedCreator.properties()) {
/* 327 */         if (!prop.hasValueDeserializer()) {
/* 328 */           prop.setValueDeserializer(findDeserializer(config, provider, prop.getType(), prop));
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Object deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 346 */     JsonToken t = jp.getCurrentToken();
/*     */     
/* 348 */     if (t == JsonToken.START_OBJECT) {
/* 349 */       jp.nextToken();
/* 350 */       return deserializeFromObject(jp, ctxt);
/*     */     }
/*     */     
/* 353 */     switch (t) {
/*     */     case VALUE_STRING: 
/* 355 */       return deserializeFromString(jp, ctxt);
/*     */     case VALUE_NUMBER_INT: 
/*     */     case VALUE_NUMBER_FLOAT: 
/* 358 */       return deserializeFromNumber(jp, ctxt);
/*     */     case VALUE_EMBEDDED_OBJECT: 
/* 360 */       return jp.getEmbeddedObject();
/*     */     
/*     */     case VALUE_TRUE: 
/*     */     case VALUE_FALSE: 
/*     */     case START_ARRAY: 
/* 365 */       return deserializeUsingCreator(jp, ctxt);
/*     */     case FIELD_NAME: 
/*     */     case END_OBJECT: 
/* 368 */       return deserializeFromObject(jp, ctxt);
/*     */     }
/* 370 */     throw ctxt.mappingException(getBeanClass());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object deserialize(JsonParser jp, DeserializationContext ctxt, Object bean)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 382 */     JsonToken t = jp.getCurrentToken();
/*     */     
/* 384 */     if (t == JsonToken.START_OBJECT) {}
/* 385 */     for (t = jp.nextToken(); 
/*     */         
/* 387 */         t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
/* 388 */       String propName = jp.getCurrentName();
/* 389 */       SettableBeanProperty prop = this._beanProperties.find(propName);
/* 390 */       jp.nextToken();
/*     */       
/* 392 */       if (prop != null) {
/*     */         try {
/* 394 */           prop.deserializeAndSet(jp, ctxt, bean);
/*     */         } catch (Exception e) {
/* 396 */           wrapAndThrow(e, bean, propName, ctxt);
/*     */ 
/*     */ 
/*     */         }
/*     */         
/*     */ 
/*     */       }
/* 403 */       else if ((this._ignorableProps != null) && (this._ignorableProps.contains(propName))) {
/* 404 */         jp.skipChildren();
/*     */ 
/*     */       }
/* 407 */       else if (this._anySetter != null) {
/* 408 */         this._anySetter.deserializeAndSet(jp, ctxt, bean, propName);
/*     */       }
/*     */       else
/*     */       {
/* 412 */         handleUnknownProperty(jp, ctxt, bean, propName); }
/*     */     }
/* 414 */     return bean;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 423 */     return typeDeserializer.deserializeTypedFromObject(jp, ctxt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 432 */   public final Class<?> getBeanClass() { return this._beanType.getRawClass(); }
/*     */   
/* 434 */   public JavaType getValueType() { return this._beanType; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Iterator<SettableBeanProperty> properties()
/*     */   {
/* 442 */     if (this._beanProperties == null) {
/* 443 */       throw new IllegalStateException("Can only call before BeanDeserializer has been resolved");
/*     */     }
/* 445 */     return this._beanProperties.allProperties();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SettableBeanProperty findBackReference(String logicalName)
/*     */   {
/* 454 */     if (this._backRefs == null) {
/* 455 */       return null;
/*     */     }
/* 457 */     return (SettableBeanProperty)this._backRefs.get(logicalName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object deserializeFromObject(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 469 */     if (this._defaultConstructor == null)
/*     */     {
/* 471 */       if (this._propertyBasedCreator != null) {
/* 472 */         return _deserializeUsingPropertyBased(jp, ctxt);
/*     */       }
/*     */       
/* 475 */       if (this._delegatingCreator != null) {
/* 476 */         return this._delegatingCreator.deserialize(jp, ctxt);
/*     */       }
/*     */       
/* 479 */       if (this._beanType.isAbstract()) {
/* 480 */         throw JsonMappingException.from(jp, "Can not instantiate abstract type " + this._beanType + " (need to add/enable type information?)");
/*     */       }
/*     */       
/* 483 */       throw JsonMappingException.from(jp, "No suitable constructor found for type " + this._beanType + ": can not instantiate from JSON object (need to add/enable type information?)");
/*     */     }
/*     */     
/* 486 */     Object bean = constructDefaultInstance();
/* 487 */     for (; jp.getCurrentToken() != JsonToken.END_OBJECT; jp.nextToken()) {
/* 488 */       String propName = jp.getCurrentName();
/*     */       
/* 490 */       jp.nextToken();
/* 491 */       SettableBeanProperty prop = this._beanProperties.find(propName);
/* 492 */       if (prop != null) {
/*     */         try {
/* 494 */           prop.deserializeAndSet(jp, ctxt, bean);
/*     */         } catch (Exception e) {
/* 496 */           wrapAndThrow(e, bean, propName, ctxt);
/*     */ 
/*     */ 
/*     */         }
/*     */         
/*     */ 
/*     */       }
/* 503 */       else if ((this._ignorableProps != null) && (this._ignorableProps.contains(propName))) {
/* 504 */         jp.skipChildren();
/* 505 */       } else if (this._anySetter != null) {
/*     */         try {
/* 507 */           this._anySetter.deserializeAndSet(jp, ctxt, bean, propName);
/*     */         } catch (Exception e) {
/* 509 */           wrapAndThrow(e, bean, propName, ctxt);
/*     */         }
/*     */         
/*     */       }
/*     */       else {
/* 514 */         handleUnknownProperty(jp, ctxt, bean, propName);
/*     */       }
/*     */     }
/* 517 */     return bean;
/*     */   }
/*     */   
/*     */   public Object deserializeFromString(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 523 */     if (this._stringCreator != null) {
/* 524 */       return this._stringCreator.construct(jp.getText());
/*     */     }
/* 526 */     if (this._delegatingCreator != null) {
/* 527 */       return this._delegatingCreator.deserialize(jp, ctxt);
/*     */     }
/*     */     
/* 530 */     if ((ctxt.isEnabled(DeserializationConfig.Feature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT)) && 
/* 531 */       (jp.getTextLength() == 0)) {
/* 532 */       return null;
/*     */     }
/*     */     
/* 535 */     throw ctxt.instantiationException(getBeanClass(), "no suitable creator method found to deserialize from JSON String");
/*     */   }
/*     */   
/*     */   public Object deserializeFromNumber(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 541 */     if (this._numberCreator != null) {
/* 542 */       switch (jp.getNumberType()) {
/*     */       case INT: 
/* 544 */         return this._numberCreator.construct(jp.getIntValue());
/*     */       case LONG: 
/* 546 */         return this._numberCreator.construct(jp.getLongValue());
/*     */       }
/*     */     }
/* 549 */     if (this._delegatingCreator != null) {
/* 550 */       return this._delegatingCreator.deserialize(jp, ctxt);
/*     */     }
/* 552 */     throw ctxt.instantiationException(getBeanClass(), "no suitable creator method found to deserialize from JSON Number");
/*     */   }
/*     */   
/*     */   public Object deserializeUsingCreator(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 558 */     if (this._delegatingCreator != null) {
/*     */       try {
/* 560 */         return this._delegatingCreator.deserialize(jp, ctxt);
/*     */       } catch (Exception e) {
/* 562 */         wrapInstantiationProblem(e, ctxt);
/*     */       }
/*     */     }
/* 565 */     throw ctxt.mappingException(getBeanClass());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Object _deserializeUsingPropertyBased(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 581 */     Creator.PropertyBased creator = this._propertyBasedCreator;
/* 582 */     PropertyValueBuffer buffer = creator.startBuilding(jp, ctxt);
/*     */     
/*     */ 
/* 585 */     TokenBuffer unknown = null;
/*     */     
/* 587 */     for (JsonToken t = jp.getCurrentToken(); 
/* 588 */         t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
/* 589 */       String propName = jp.getCurrentName();
/* 590 */       jp.nextToken();
/*     */       
/* 592 */       SettableBeanProperty prop = creator.findCreatorProperty(propName);
/* 593 */       if (prop != null)
/*     */       {
/* 595 */         Object value = prop.deserialize(jp, ctxt);
/* 596 */         if (buffer.assignParameter(prop.getCreatorIndex(), value)) {
/* 597 */           jp.nextToken();
/*     */           Object bean;
/*     */           try {
/* 600 */             bean = creator.build(buffer);
/*     */           } catch (Exception e) {
/* 602 */             wrapAndThrow(e, this._beanType.getRawClass(), propName, ctxt);
/* 603 */             continue;
/*     */           }
/*     */           
/* 606 */           if (bean.getClass() != this._beanType.getRawClass()) {
/* 607 */             return handlePolymorphic(jp, ctxt, bean, unknown);
/*     */           }
/* 609 */           if (unknown != null) {
/* 610 */             bean = handleUnknownProperties(ctxt, bean, unknown);
/*     */           }
/*     */           
/* 613 */           return deserialize(jp, ctxt, bean);
/*     */         }
/*     */       }
/*     */       else
/*     */       {
/* 618 */         prop = this._beanProperties.find(propName);
/* 619 */         if (prop != null) {
/* 620 */           buffer.bufferProperty(prop, prop.deserialize(jp, ctxt));
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         }
/* 626 */         else if ((this._ignorableProps != null) && (this._ignorableProps.contains(propName))) {
/* 627 */           jp.skipChildren();
/*     */ 
/*     */ 
/*     */         }
/* 631 */         else if (this._anySetter != null) {
/* 632 */           buffer.bufferAnyProperty(this._anySetter, propName, this._anySetter.deserialize(jp, ctxt));
/*     */         }
/*     */         else
/*     */         {
/* 636 */           if (unknown == null) {
/* 637 */             unknown = new TokenBuffer(jp.getCodec());
/*     */           }
/* 639 */           unknown.writeFieldName(propName);
/* 640 */           unknown.copyCurrentStructure(jp);
/*     */         }
/*     */       }
/*     */     }
/*     */     Object bean;
/*     */     try {
/* 646 */       bean = creator.build(buffer);
/*     */     } catch (Exception e) {
/* 648 */       wrapInstantiationProblem(e, ctxt);
/* 649 */       return null;
/*     */     }
/* 651 */     if (unknown != null)
/*     */     {
/* 653 */       if (bean.getClass() != this._beanType.getRawClass()) {
/* 654 */         return handlePolymorphic(null, ctxt, bean, unknown);
/*     */       }
/*     */       
/* 657 */       return handleUnknownProperties(ctxt, bean, unknown);
/*     */     }
/* 659 */     return bean;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void handleUnknownProperty(JsonParser jp, DeserializationContext ctxt, Object beanOrClass, String propName)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 681 */     if ((this._ignoreAllUnknown) || ((this._ignorableProps != null) && (this._ignorableProps.contains(propName))))
/*     */     {
/* 683 */       jp.skipChildren();
/* 684 */       return;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 689 */     super.handleUnknownProperty(jp, ctxt, beanOrClass, propName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object handleUnknownProperties(DeserializationContext ctxt, Object bean, TokenBuffer unknownTokens)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 701 */     unknownTokens.writeEndObject();
/*     */     
/*     */ 
/* 704 */     JsonParser bufferParser = unknownTokens.asParser();
/* 705 */     while (bufferParser.nextToken() != JsonToken.END_OBJECT) {
/* 706 */       String propName = bufferParser.getCurrentName();
/*     */       
/* 708 */       bufferParser.nextToken();
/* 709 */       handleUnknownProperty(bufferParser, ctxt, bean, propName);
/*     */     }
/* 711 */     return bean;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object handlePolymorphic(JsonParser jp, DeserializationContext ctxt, Object bean, TokenBuffer unknownTokens)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 730 */     JsonDeserializer<Object> subDeser = _findSubclassDeserializer(ctxt, bean, unknownTokens);
/* 731 */     if (subDeser != null) {
/* 732 */       if (unknownTokens != null)
/*     */       {
/* 734 */         unknownTokens.writeEndObject();
/* 735 */         JsonParser p2 = unknownTokens.asParser();
/* 736 */         p2.nextToken();
/* 737 */         bean = subDeser.deserialize(p2, ctxt, bean);
/*     */       }
/*     */       
/* 740 */       if (jp != null) {
/* 741 */         bean = subDeser.deserialize(jp, ctxt, bean);
/*     */       }
/* 743 */       return bean;
/*     */     }
/*     */     
/* 746 */     if (unknownTokens != null) {
/* 747 */       bean = handleUnknownProperties(ctxt, bean, unknownTokens);
/*     */     }
/*     */     
/* 750 */     if (jp != null) {
/* 751 */       bean = deserialize(jp, ctxt, bean);
/*     */     }
/* 753 */     return bean;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _findSubclassDeserializer(DeserializationContext ctxt, Object bean, TokenBuffer unknownTokens)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*     */     JsonDeserializer<Object> subDeser;
/*     */     
/*     */ 
/*     */ 
/* 766 */     synchronized (this) {
/* 767 */       subDeser = this._subDeserializers == null ? null : (JsonDeserializer)this._subDeserializers.get(new ClassKey(bean.getClass()));
/*     */     }
/* 769 */     if (subDeser != null) {
/* 770 */       return subDeser;
/*     */     }
/*     */     
/* 773 */     DeserializerProvider deserProv = ctxt.getDeserializerProvider();
/* 774 */     if (deserProv != null) {
/* 775 */       JavaType type = ctxt.constructType(bean.getClass());
/*     */       
/*     */ 
/*     */ 
/* 779 */       subDeser = deserProv.findValueDeserializer(ctxt.getConfig(), type, this._property);
/*     */       
/* 781 */       if (subDeser != null) {
/* 782 */         synchronized (this) {
/* 783 */           if (this._subDeserializers == null) {
/* 784 */             this._subDeserializers = new HashMap();
/*     */           }
/* 786 */           this._subDeserializers.put(new ClassKey(bean.getClass()), subDeser);
/*     */         }
/*     */       }
/*     */     }
/* 790 */     return subDeser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object constructDefaultInstance()
/*     */   {
/*     */     try
/*     */     {
/* 804 */       return this._defaultConstructor.newInstance(new Object[0]);
/*     */     } catch (Exception e) {
/* 806 */       ClassUtil.unwrapAndThrowAsIAE(e); }
/* 807 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void wrapAndThrow(Throwable t, Object bean, String fieldName, DeserializationContext ctxt)
/*     */     throws IOException
/*     */   {
/* 837 */     while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 838 */       t = t.getCause();
/*     */     }
/*     */     
/* 841 */     if ((t instanceof Error)) {
/* 842 */       throw ((Error)t);
/*     */     }
/* 844 */     boolean wrap = (ctxt == null) || (ctxt.isEnabled(DeserializationConfig.Feature.WRAP_EXCEPTIONS));
/*     */     
/* 846 */     if ((t instanceof IOException)) {
/* 847 */       if ((!wrap) || (!(t instanceof JsonMappingException))) {
/* 848 */         throw ((IOException)t);
/*     */       }
/* 850 */     } else if ((!wrap) && 
/* 851 */       ((t instanceof RuntimeException))) {
/* 852 */       throw ((RuntimeException)t);
/*     */     }
/*     */     
/*     */ 
/* 856 */     throw JsonMappingException.wrapWithPath(t, bean, fieldName);
/*     */   }
/*     */   
/*     */   public void wrapAndThrow(Throwable t, Object bean, int index, DeserializationContext ctxt)
/*     */     throws IOException
/*     */   {
/* 862 */     while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 863 */       t = t.getCause();
/*     */     }
/*     */     
/* 866 */     if ((t instanceof Error)) {
/* 867 */       throw ((Error)t);
/*     */     }
/* 869 */     boolean wrap = (ctxt == null) || (ctxt.isEnabled(DeserializationConfig.Feature.WRAP_EXCEPTIONS));
/*     */     
/* 871 */     if ((t instanceof IOException)) {
/* 872 */       if ((!wrap) || (!(t instanceof JsonMappingException))) {
/* 873 */         throw ((IOException)t);
/*     */       }
/* 875 */     } else if ((!wrap) && 
/* 876 */       ((t instanceof RuntimeException))) {
/* 877 */       throw ((RuntimeException)t);
/*     */     }
/*     */     
/*     */ 
/* 881 */     throw JsonMappingException.wrapWithPath(t, bean, index);
/*     */   }
/*     */   
/*     */   protected void wrapInstantiationProblem(Throwable t, DeserializationContext ctxt)
/*     */     throws IOException
/*     */   {
/* 887 */     while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 888 */       t = t.getCause();
/*     */     }
/*     */     
/* 891 */     if ((t instanceof Error)) {
/* 892 */       throw ((Error)t);
/*     */     }
/* 894 */     boolean wrap = (ctxt == null) || (ctxt.isEnabled(DeserializationConfig.Feature.WRAP_EXCEPTIONS));
/* 895 */     if ((t instanceof IOException))
/*     */     {
/* 897 */       throw ((IOException)t); }
/* 898 */     if ((!wrap) && 
/* 899 */       ((t instanceof RuntimeException))) {
/* 900 */       throw ((RuntimeException)t);
/*     */     }
/*     */     
/* 903 */     throw ctxt.instantiationException(this._beanType.getRawClass(), t);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public void wrapAndThrow(Throwable t, Object bean, String fieldName)
/*     */     throws IOException
/*     */   {
/* 913 */     wrapAndThrow(t, bean, fieldName, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public void wrapAndThrow(Throwable t, Object bean, int index)
/*     */     throws IOException
/*     */   {
/* 923 */     wrapAndThrow(t, bean, index, null);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\BeanDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */