/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedConstructor;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.type.TypeBindings;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ abstract class Creator
/*     */ {
/*     */   static final class StringBased
/*     */   {
/*     */     protected final Class<?> _valueClass;
/*     */     protected final Method _factoryMethod;
/*     */     protected final Constructor<?> _ctor;
/*     */     
/*     */     public StringBased(Class<?> valueClass, AnnotatedConstructor ctor, AnnotatedMethod factoryMethod)
/*     */     {
/*  39 */       this._valueClass = valueClass;
/*  40 */       this._ctor = (ctor == null ? null : ctor.getAnnotated());
/*  41 */       this._factoryMethod = (factoryMethod == null ? null : factoryMethod.getAnnotated());
/*     */     }
/*     */     
/*     */     public Object construct(String value)
/*     */     {
/*     */       try {
/*  47 */         if (this._ctor != null) {
/*  48 */           return this._ctor.newInstance(new Object[] { value });
/*     */         }
/*  50 */         if (this._factoryMethod != null) {
/*  51 */           return this._factoryMethod.invoke(this._valueClass, new Object[] { value });
/*     */         }
/*     */       } catch (Exception e) {
/*  54 */         ClassUtil.unwrapAndThrowAsIAE(e);
/*     */       }
/*  56 */       return null;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static final class NumberBased
/*     */   {
/*     */     protected final Class<?> _valueClass;
/*     */     
/*     */ 
/*     */     protected final Constructor<?> _intCtor;
/*     */     
/*     */     protected final Constructor<?> _longCtor;
/*     */     
/*     */     protected final Method _intFactoryMethod;
/*     */     
/*     */     protected final Method _longFactoryMethod;
/*     */     
/*     */ 
/*     */     public NumberBased(Class<?> valueClass, AnnotatedConstructor intCtor, AnnotatedMethod ifm, AnnotatedConstructor longCtor, AnnotatedMethod lfm)
/*     */     {
/*  78 */       this._valueClass = valueClass;
/*  79 */       this._intCtor = (intCtor == null ? null : intCtor.getAnnotated());
/*  80 */       this._longCtor = (longCtor == null ? null : longCtor.getAnnotated());
/*  81 */       this._intFactoryMethod = (ifm == null ? null : ifm.getAnnotated());
/*  82 */       this._longFactoryMethod = (lfm == null ? null : lfm.getAnnotated());
/*     */     }
/*     */     
/*     */     public Object construct(int value)
/*     */     {
/*     */       try
/*     */       {
/*  89 */         if (this._intCtor != null) {
/*  90 */           return this._intCtor.newInstance(new Object[] { Integer.valueOf(value) });
/*     */         }
/*  92 */         if (this._intFactoryMethod != null) {
/*  93 */           return this._intFactoryMethod.invoke(this._valueClass, new Object[] { Integer.valueOf(value) });
/*     */         }
/*     */       } catch (Exception e) {
/*  96 */         ClassUtil.unwrapAndThrowAsIAE(e);
/*     */       }
/*     */       
/*  99 */       return construct(value);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public Object construct(long value)
/*     */     {
/*     */       try
/*     */       {
/* 109 */         if (this._longCtor != null) {
/* 110 */           return this._longCtor.newInstance(new Object[] { Long.valueOf(value) });
/*     */         }
/* 112 */         if (this._longFactoryMethod != null) {
/* 113 */           return this._longFactoryMethod.invoke(this._valueClass, new Object[] { Long.valueOf(value) });
/*     */         }
/*     */       } catch (Exception e) {
/* 116 */         ClassUtil.unwrapAndThrowAsIAE(e);
/*     */       }
/* 118 */       return null;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static final class Delegating
/*     */   {
/*     */     protected final AnnotatedMember _creator;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final JavaType _valueType;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final Constructor<?> _ctor;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final Method _factoryMethod;
/*     */     
/*     */ 
/*     */ 
/*     */     protected JsonDeserializer<Object> _deserializer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public Delegating(BasicBeanDescription beanDesc, AnnotatedConstructor ctor, AnnotatedMethod factory)
/*     */     {
/* 154 */       TypeBindings bindings = beanDesc.bindingsForBeanType();
/* 155 */       if (ctor != null) {
/* 156 */         this._creator = ctor;
/* 157 */         this._ctor = ctor.getAnnotated();
/* 158 */         this._factoryMethod = null;
/* 159 */         this._valueType = bindings.resolveType(ctor.getParameterType(0));
/* 160 */       } else if (factory != null) {
/* 161 */         this._creator = factory;
/* 162 */         this._ctor = null;
/* 163 */         this._factoryMethod = factory.getAnnotated();
/* 164 */         this._valueType = bindings.resolveType(factory.getParameterType(0));
/*     */       } else {
/* 166 */         throw new IllegalArgumentException("Internal error: neither delegating constructor nor factory method passed");
/*     */       }
/*     */     }
/*     */     
/* 170 */     public JavaType getValueType() { return this._valueType; }
/*     */     
/* 172 */     public AnnotatedMember getCreator() { return this._creator; }
/*     */     
/*     */     public void setDeserializer(JsonDeserializer<Object> deser)
/*     */     {
/* 176 */       this._deserializer = deser;
/*     */     }
/*     */     
/*     */     public Object deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 182 */       Object value = this._deserializer.deserialize(jp, ctxt);
/*     */       try {
/* 184 */         if (this._ctor != null) {
/* 185 */           return this._ctor.newInstance(new Object[] { value });
/*     */         }
/*     */         
/* 188 */         return this._factoryMethod.invoke(null, new Object[] { value });
/*     */       } catch (Exception e) {
/* 190 */         ClassUtil.unwrapAndThrowAsIAE(e); }
/* 191 */       return null;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static final class PropertyBased
/*     */   {
/*     */     protected final Constructor<?> _ctor;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final Method _factoryMethod;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final HashMap<String, SettableBeanProperty> _properties;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final Object[] _defaultValues;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public PropertyBased(AnnotatedConstructor ctor, SettableBeanProperty[] ctorProps, AnnotatedMethod factory, SettableBeanProperty[] factoryProps)
/*     */     {
/*     */       SettableBeanProperty[] props;
/*     */       
/*     */ 
/*     */ 
/* 224 */       if (ctor != null) {
/* 225 */         this._ctor = ctor.getAnnotated();
/* 226 */         this._factoryMethod = null;
/* 227 */         props = ctorProps; } else { SettableBeanProperty[] props;
/* 228 */         if (factory != null) {
/* 229 */           this._ctor = null;
/* 230 */           this._factoryMethod = factory.getAnnotated();
/* 231 */           props = factoryProps;
/*     */         } else {
/* 233 */           throw new IllegalArgumentException("Internal error: neither delegating constructor nor factory method passed"); } }
/*     */       SettableBeanProperty[] props;
/* 235 */       this._properties = new HashMap();
/*     */       
/* 237 */       Object[] defValues = null;
/* 238 */       int i = 0; for (int len = props.length; i < len; i++) {
/* 239 */         SettableBeanProperty prop = props[i];
/* 240 */         this._properties.put(prop.getName(), prop);
/* 241 */         if (prop.getType().isPrimitive()) {
/* 242 */           if (defValues == null) {
/* 243 */             defValues = new Object[len];
/*     */           }
/* 245 */           defValues[i] = ClassUtil.defaultValue(prop.getType().getRawClass());
/*     */         }
/*     */       }
/* 248 */       this._defaultValues = defValues;
/*     */     }
/*     */     
/*     */     public Collection<SettableBeanProperty> properties() {
/* 252 */       return this._properties.values();
/*     */     }
/*     */     
/*     */     public SettableBeanProperty findCreatorProperty(String name) {
/* 256 */       return (SettableBeanProperty)this._properties.get(name);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public PropertyValueBuffer startBuilding(JsonParser jp, DeserializationContext ctxt)
/*     */     {
/* 264 */       return new PropertyValueBuffer(jp, ctxt, this._properties.size());
/*     */     }
/*     */     
/*     */     public Object build(PropertyValueBuffer buffer) throws Exception
/*     */     {
/*     */       Object bean;
/*     */       try {
/*     */         Object bean;
/* 272 */         if (this._ctor != null) {
/* 273 */           bean = this._ctor.newInstance(buffer.getParameters(this._defaultValues));
/*     */         } else {
/* 275 */           bean = this._factoryMethod.invoke(null, buffer.getParameters(this._defaultValues));
/*     */         }
/*     */       } catch (Exception e) {
/* 278 */         ClassUtil.throwRootCause(e);
/* 279 */         return null;
/*     */       }
/*     */       
/* 282 */       for (PropertyValue pv = buffer.buffered(); pv != null; pv = pv.next) {
/* 283 */         pv.assign(bean);
/*     */       }
/* 285 */       return bean;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\Creator.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */