/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Type;
/*     */ import java.util.HashMap;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.io.SerializedString;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.SerializerProvider;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.ser.impl.PropertySerializerMap;
/*     */ import org.codehaus.jackson.map.ser.impl.PropertySerializerMap.SerializerAndMapResult;
/*     */ import org.codehaus.jackson.map.util.Annotations;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BeanPropertyWriter
/*     */   implements BeanProperty
/*     */ {
/*     */   protected final AnnotatedMember _member;
/*     */   protected final Annotations _contextAnnotations;
/*     */   protected final JavaType _declaredType;
/*     */   protected final Method _accessorMethod;
/*     */   protected final Field _field;
/*     */   protected HashMap<Object, Object> _internalSettings;
/*     */   protected final SerializedString _name;
/*     */   protected final JavaType _cfgSerializationType;
/*     */   protected final JsonSerializer<Object> _serializer;
/*     */   protected PropertySerializerMap _dynamicSerializers;
/*     */   protected final boolean _suppressNulls;
/*     */   protected final Object _suppressableValue;
/*     */   protected Class<?>[] _includeInViews;
/*     */   protected TypeSerializer _typeSerializer;
/*     */   protected JavaType _nonTrivialBaseType;
/*     */   
/*     */   public BeanPropertyWriter(AnnotatedMember member, Annotations contextAnnotations, String name, JavaType declaredType, JsonSerializer<Object> ser, TypeSerializer typeSer, JavaType serType, Method m, Field f, boolean suppressNulls, Object suppressableValue)
/*     */   {
/* 166 */     this(member, contextAnnotations, new SerializedString(name), declaredType, ser, typeSer, serType, m, f, suppressNulls, suppressableValue);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public BeanPropertyWriter(AnnotatedMember member, Annotations contextAnnotations, SerializedString name, JavaType declaredType, JsonSerializer<Object> ser, TypeSerializer typeSer, JavaType serType, Method m, Field f, boolean suppressNulls, Object suppressableValue)
/*     */   {
/* 176 */     this._member = member;
/* 177 */     this._contextAnnotations = contextAnnotations;
/* 178 */     this._name = name;
/* 179 */     this._declaredType = declaredType;
/* 180 */     this._serializer = ser;
/* 181 */     this._dynamicSerializers = (ser == null ? PropertySerializerMap.emptyMap() : null);
/* 182 */     this._typeSerializer = typeSer;
/* 183 */     this._cfgSerializationType = serType;
/* 184 */     this._accessorMethod = m;
/* 185 */     this._field = f;
/* 186 */     this._suppressNulls = suppressNulls;
/* 187 */     this._suppressableValue = suppressableValue;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BeanPropertyWriter(BeanPropertyWriter base)
/*     */   {
/* 195 */     this(base, base._serializer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BeanPropertyWriter(BeanPropertyWriter base, JsonSerializer<Object> ser)
/*     */   {
/* 203 */     this._serializer = ser;
/*     */     
/* 205 */     this._member = base._member;
/* 206 */     this._contextAnnotations = base._contextAnnotations;
/* 207 */     this._declaredType = base._declaredType;
/* 208 */     this._accessorMethod = base._accessorMethod;
/* 209 */     this._field = base._field;
/*     */     
/* 211 */     if (base._internalSettings != null) {
/* 212 */       this._internalSettings = new HashMap(base._internalSettings);
/*     */     }
/* 214 */     this._name = base._name;
/* 215 */     this._cfgSerializationType = base._cfgSerializationType;
/* 216 */     this._dynamicSerializers = base._dynamicSerializers;
/* 217 */     this._suppressNulls = base._suppressNulls;
/* 218 */     this._suppressableValue = base._suppressableValue;
/* 219 */     this._includeInViews = base._includeInViews;
/* 220 */     this._typeSerializer = base._typeSerializer;
/* 221 */     this._nonTrivialBaseType = base._nonTrivialBaseType;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public BeanPropertyWriter withSerializer(JsonSerializer<Object> ser)
/*     */   {
/* 232 */     if (getClass() != BeanPropertyWriter.class) {
/* 233 */       throw new IllegalStateException("BeanPropertyWriter sub-class does not override 'withSerializer()'; needs to!");
/*     */     }
/* 235 */     return new BeanPropertyWriter(this, ser);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setViews(Class<?>[] views)
/*     */   {
/* 246 */     this._includeInViews = views;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setNonTrivialBaseType(JavaType t)
/*     */   {
/* 256 */     this._nonTrivialBaseType = t;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getName()
/*     */   {
/* 266 */     return this._name.getValue();
/*     */   }
/*     */   
/*     */   public JavaType getType() {
/* 270 */     return this._declaredType;
/*     */   }
/*     */   
/*     */   public <A extends Annotation> A getAnnotation(Class<A> acls) {
/* 274 */     return this._member.getAnnotation(acls);
/*     */   }
/*     */   
/*     */   public <A extends Annotation> A getContextAnnotation(Class<A> acls) {
/* 278 */     return this._contextAnnotations.get(acls);
/*     */   }
/*     */   
/*     */   public AnnotatedMember getMember() {
/* 282 */     return this._member;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object getInternalSetting(Object key)
/*     */   {
/* 301 */     if (this._internalSettings == null) {
/* 302 */       return null;
/*     */     }
/* 304 */     return this._internalSettings.get(key);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object setInternalSetting(Object key, Object value)
/*     */   {
/* 316 */     if (this._internalSettings == null) {
/* 317 */       this._internalSettings = new HashMap();
/*     */     }
/* 319 */     return this._internalSettings.put(key, value);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object removeInternalSetting(Object key)
/*     */   {
/* 331 */     Object removed = null;
/* 332 */     if (this._internalSettings != null) {
/* 333 */       removed = this._internalSettings.remove(key);
/*     */       
/* 335 */       if (this._internalSettings.size() == 0) {
/* 336 */         this._internalSettings = null;
/*     */       }
/*     */     }
/* 339 */     return removed;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 348 */   public SerializedString getSerializedName() { return this._name; }
/*     */   
/* 350 */   public boolean hasSerializer() { return this._serializer != null; }
/*     */   
/*     */   protected JsonSerializer<Object> getSerializer()
/*     */   {
/* 354 */     return this._serializer;
/*     */   }
/*     */   
/*     */   public JavaType getSerializationType() {
/* 358 */     return this._cfgSerializationType;
/*     */   }
/*     */   
/*     */   public Class<?> getRawSerializationType() {
/* 362 */     return this._cfgSerializationType == null ? null : this._cfgSerializationType.getRawClass();
/*     */   }
/*     */   
/*     */   public Class<?> getPropertyType()
/*     */   {
/* 367 */     if (this._accessorMethod != null) {
/* 368 */       return this._accessorMethod.getReturnType();
/*     */     }
/* 370 */     return this._field.getType();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Type getGenericPropertyType()
/*     */   {
/* 380 */     if (this._accessorMethod != null) {
/* 381 */       return this._accessorMethod.getGenericReturnType();
/*     */     }
/* 383 */     return this._field.getGenericType();
/*     */   }
/*     */   
/* 386 */   public Class<?>[] getViews() { return this._includeInViews; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void serializeAsField(Object bean, JsonGenerator jgen, SerializerProvider prov)
/*     */     throws Exception
/*     */   {
/* 402 */     Object value = get(bean);
/*     */     
/* 404 */     if (value == null) {
/* 405 */       if (!this._suppressNulls) {
/* 406 */         jgen.writeFieldName(this._name);
/* 407 */         prov.defaultSerializeNull(jgen);
/*     */       }
/* 409 */       return;
/*     */     }
/*     */     
/* 412 */     if (value == bean) {
/* 413 */       _reportSelfReference(bean);
/*     */     }
/* 415 */     if ((this._suppressableValue != null) && (this._suppressableValue.equals(value))) {
/* 416 */       return;
/*     */     }
/*     */     
/* 419 */     JsonSerializer<Object> ser = this._serializer;
/* 420 */     if (ser == null) {
/* 421 */       Class<?> cls = value.getClass();
/* 422 */       PropertySerializerMap map = this._dynamicSerializers;
/* 423 */       ser = map.serializerFor(cls);
/* 424 */       if (ser == null) {
/* 425 */         ser = _findAndAddDynamic(map, cls, prov);
/*     */       }
/*     */     }
/* 428 */     jgen.writeFieldName(this._name);
/* 429 */     if (this._typeSerializer == null) {
/* 430 */       ser.serialize(value, jgen, prov);
/*     */     } else {
/* 432 */       ser.serializeWithType(value, jgen, prov, this._typeSerializer);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map, Class<?> type, SerializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/*     */     PropertySerializerMap.SerializerAndMapResult result;
/*     */     
/*     */     PropertySerializerMap.SerializerAndMapResult result;
/* 443 */     if (this._nonTrivialBaseType != null) {
/* 444 */       JavaType t = this._nonTrivialBaseType.forcedNarrowBy(type);
/* 445 */       result = map.findAndAddSerializer(t, provider, this);
/*     */     } else {
/* 447 */       result = map.findAndAddSerializer(type, provider, this);
/*     */     }
/*     */     
/* 450 */     if (map != result.map) {
/* 451 */       this._dynamicSerializers = result.map;
/*     */     }
/* 453 */     return result.serializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Object get(Object bean)
/*     */     throws Exception
/*     */   {
/* 466 */     if (this._accessorMethod != null) {
/* 467 */       return this._accessorMethod.invoke(bean, new Object[0]);
/*     */     }
/* 469 */     return this._field.get(bean);
/*     */   }
/*     */   
/*     */   protected void _reportSelfReference(Object bean)
/*     */     throws JsonMappingException
/*     */   {
/* 475 */     throw new JsonMappingException("Direct self-reference leading to cycle");
/*     */   }
/*     */   
/*     */ 
/*     */   public String toString()
/*     */   {
/* 481 */     StringBuilder sb = new StringBuilder(40);
/* 482 */     sb.append("property '").append(getName()).append("' (");
/* 483 */     if (this._accessorMethod != null) {
/* 484 */       sb.append("via method ").append(this._accessorMethod.getDeclaringClass().getName()).append("#").append(this._accessorMethod.getName());
/*     */     } else {
/* 486 */       sb.append("field \"").append(this._field.getDeclaringClass().getName()).append("#").append(this._field.getName());
/*     */     }
/* 488 */     if (this._serializer == null) {
/* 489 */       sb.append(", no static serializer");
/*     */     } else {
/* 491 */       sb.append(", static serializer of type " + this._serializer.getClass().getName());
/*     */     }
/* 493 */     sb.append(')');
/* 494 */     return sb.toString();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\BeanPropertyWriter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */