/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Modifier;
/*     */ import java.lang.reflect.Type;
/*     */ import org.codehaus.jackson.JsonGenerationException;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.ResolvableSerializer;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.SerializerProvider;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.schema.JsonSchema;
/*     */ import org.codehaus.jackson.schema.SchemaAware;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ @JacksonStdImpl
/*     */ public final class JsonValueSerializer
/*     */   extends SerializerBase<Object>
/*     */   implements ResolvableSerializer, SchemaAware
/*     */ {
/*     */   protected final Method _accessorMethod;
/*     */   protected JsonSerializer<Object> _valueSerializer;
/*     */   protected final BeanProperty _property;
/*     */   protected boolean _forceTypeInformation;
/*     */   
/*     */   public JsonValueSerializer(Method valueMethod, JsonSerializer<Object> ser, BeanProperty property)
/*     */   {
/*  60 */     super(Object.class);
/*  61 */     this._accessorMethod = valueMethod;
/*  62 */     this._valueSerializer = ser;
/*  63 */     this._property = property;
/*     */   }
/*     */   
/*     */   public void serialize(Object bean, JsonGenerator jgen, SerializerProvider prov)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/*     */     try
/*     */     {
/*  71 */       Object value = this._accessorMethod.invoke(bean, new Object[0]);
/*     */       
/*  73 */       if (value == null) {
/*  74 */         prov.defaultSerializeNull(jgen);
/*  75 */         return;
/*     */       }
/*  77 */       JsonSerializer<Object> ser = this._valueSerializer;
/*  78 */       if (ser == null) {
/*  79 */         Class<?> c = value.getClass();
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  85 */         ser = prov.findTypedValueSerializer(c, true, this._property);
/*     */       }
/*  87 */       ser.serialize(value, jgen, prov);
/*     */     } catch (IOException ioe) {
/*  89 */       throw ioe;
/*     */     } catch (Exception e) {
/*  91 */       Throwable t = e;
/*     */       
/*  93 */       while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/*  94 */         t = t.getCause();
/*     */       }
/*     */       
/*  97 */       if ((t instanceof Error)) {
/*  98 */         throw ((Error)t);
/*     */       }
/*     */       
/* 101 */       throw JsonMappingException.wrapWithPath(t, bean, this._accessorMethod.getName() + "()");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void serializeWithType(Object bean, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 111 */     Object value = null;
/*     */     try {
/* 113 */       value = this._accessorMethod.invoke(bean, new Object[0]);
/*     */       
/*     */ 
/* 116 */       if (value == null) {
/* 117 */         provider.defaultSerializeNull(jgen);
/* 118 */         return;
/*     */       }
/* 120 */       JsonSerializer<Object> ser = this._valueSerializer;
/* 121 */       if (ser != null)
/*     */       {
/*     */ 
/*     */ 
/* 125 */         if (this._forceTypeInformation) {
/* 126 */           typeSer.writeTypePrefixForScalar(bean, jgen);
/*     */         }
/* 128 */         ser.serializeWithType(value, jgen, provider, typeSer);
/* 129 */         if (this._forceTypeInformation) {
/* 130 */           typeSer.writeTypeSuffixForScalar(bean, jgen);
/*     */         }
/* 132 */         return;
/*     */       }
/*     */       
/* 135 */       Class<?> c = value.getClass();
/* 136 */       ser = provider.findTypedValueSerializer(c, true, this._property);
/*     */       
/* 138 */       ser.serialize(value, jgen, provider);
/*     */     } catch (IOException ioe) {
/* 140 */       throw ioe;
/*     */     } catch (Exception e) {
/* 142 */       Throwable t = e;
/*     */       
/* 144 */       while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 145 */         t = t.getCause();
/*     */       }
/*     */       
/* 148 */       if ((t instanceof Error)) {
/* 149 */         throw ((Error)t);
/*     */       }
/*     */       
/* 152 */       throw JsonMappingException.wrapWithPath(t, bean, this._accessorMethod.getName() + "()");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     throws JsonMappingException
/*     */   {
/* 160 */     return (this._valueSerializer instanceof SchemaAware) ? ((SchemaAware)this._valueSerializer).getSchema(provider, null) : JsonSchema.getDefaultSchemaNode();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolve(SerializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/* 178 */     if (this._valueSerializer == null)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 184 */       if ((provider.isEnabled(SerializationConfig.Feature.USE_STATIC_TYPING)) || (Modifier.isFinal(this._accessorMethod.getReturnType().getModifiers())))
/*     */       {
/* 186 */         JavaType t = provider.constructType(this._accessorMethod.getGenericReturnType());
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 192 */         this._valueSerializer = provider.findTypedValueSerializer(t, false, this._property);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 197 */         this._forceTypeInformation = isNaturalTypeWithStdHandling(t, this._valueSerializer);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   protected boolean isNaturalTypeWithStdHandling(JavaType type, JsonSerializer<?> ser)
/*     */   {
/* 204 */     Class<?> cls = type.getRawClass();
/*     */     
/* 206 */     if (type.isPrimitive()) {
/* 207 */       if ((cls != Integer.TYPE) && (cls != Boolean.TYPE) && (cls != Double.TYPE)) {
/* 208 */         return false;
/*     */       }
/*     */     }
/* 211 */     else if ((cls != String.class) && (cls != Integer.class) && (cls != Boolean.class) && (cls != Double.class))
/*     */     {
/* 213 */       return false;
/*     */     }
/*     */     
/*     */ 
/* 217 */     return ser.getClass().getAnnotation(JacksonStdImpl.class) != null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String toString()
/*     */   {
/* 229 */     return "(@JsonValue serializer for method " + this._accessorMethod.getDeclaringClass() + "#" + this._accessorMethod.getName() + ")";
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\JsonValueSerializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */