/*      */ package org.codehaus.janino.tools;
/*      */ 
/*      */ import java.io.BufferedReader;
/*      */ import java.io.ByteArrayInputStream;
/*      */ import java.io.ByteArrayOutputStream;
/*      */ import java.io.DataInputStream;
/*      */ import java.io.EOFException;
/*      */ import java.io.File;
/*      */ import java.io.FileInputStream;
/*      */ import java.io.FileNotFoundException;
/*      */ import java.io.FileOutputStream;
/*      */ import java.io.FileReader;
/*      */ import java.io.FilterInputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.LineNumberReader;
/*      */ import java.io.OutputStream;
/*      */ import java.io.PrintStream;
/*      */ import java.io.PrintWriter;
/*      */ import java.util.ArrayList;
/*      */ import java.util.HashMap;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.StringTokenizer;
/*      */ import org.codehaus.janino.JaninoRuntimeException;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class Disassembler
/*      */ {
/*   60 */   private IndentPrintWriter ipw = new IndentPrintWriter(System.out);
/*   61 */   private boolean verbose = false;
/*   62 */   private File sourceDirectory = new File(".");
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static void main(String[] args)
/*      */     throws IOException
/*      */   {
/*   73 */     Disassembler d = new Disassembler();
/*      */     
/*   75 */     for (int i = 0; i < args.length; i++) {
/*   76 */       String arg = args[i];
/*   77 */       if (arg.charAt(0) != '-') break;
/*   78 */       if (arg.equals("-o")) {
/*   79 */         d.setOut(new FileOutputStream(args[(++i)]));
/*      */       }
/*   81 */       else if (arg.equals("-verbose")) {
/*   82 */         d.setVerbose(true);
/*      */       }
/*   84 */       else if (arg.equals("-src")) {
/*   85 */         d.setSourceDirectory(new File(args[(++i)]));
/*      */       }
/*   87 */       else if (arg.equals("-help"))
/*      */       {
/*   89 */         System.out.println("Usage:  java jexpr.Jdisasm [ -o <output-file> ] [ -verbose ] [ -src <source-dir> ] <class-file> ...");
/*      */         
/*   91 */         System.exit(0);
/*      */       }
/*      */       else {
/*   94 */         System.err.println("Unrecognized command line option \"" + arg + "\"; try \"-help\".");
/*      */       }
/*      */     }
/*   97 */     if (i == args.length) {
/*   98 */       System.err.println("Class file name missing, try \"-help\".");
/*   99 */       System.exit(1);
/*      */     }
/*  101 */     for (; i < args.length; i++) {
/*  102 */       d.disasm(new File(args[i]));
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public void setOut(OutputStream os)
/*      */   {
/*  109 */     this.ipw = new IndentPrintWriter(os);
/*      */   }
/*      */   
/*  112 */   public void setVerbose(boolean verbose) { this.verbose = verbose; }
/*      */   
/*      */   public void setSourceDirectory(File sourceDirectory) {
/*  115 */     this.sourceDirectory = sourceDirectory;
/*      */   }
/*      */   
/*      */   void print(int i)
/*      */   {
/*  120 */     this.ipw.print(i);
/*      */   }
/*      */   
/*      */ 
/*  124 */   void print(String s) { this.ipw.print(s); }
/*  125 */   void println() { this.ipw.println(); }
/*  126 */   void println(String s) { this.ipw.println(s); }
/*      */   
/*  128 */   void indent(String s) { this.ipw.print(s);
/*  129 */     this.ipw.indent();
/*      */   }
/*      */   
/*  132 */   void indentln(String s) { this.ipw.println(s);
/*  133 */     this.ipw.indent();
/*      */   }
/*      */   
/*  136 */   void unindent(String s) { this.ipw.unindent();
/*  137 */     this.ipw.print(s);
/*      */   }
/*      */   
/*  140 */   void unindentln(String s) { this.ipw.unindent();
/*  141 */     this.ipw.println(s);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void disasm(File file)
/*      */     throws IOException
/*      */   {
/*  151 */     InputStream is = new FileInputStream(file);
/*      */     try {
/*  153 */       disasm(is); return;
/*      */     } finally {
/*  155 */       try { is.close();
/*      */       } catch (IOException ex) {}
/*      */     }
/*      */   }
/*      */   
/*  160 */   public void disasm(InputStream is) throws IOException { disasmClassFile(new DataInputStream(is));
/*  161 */     println();
/*  162 */     this.ipw.flush();
/*      */   }
/*      */   
/*      */   private void disasmClassFile(DataInputStream dis) throws IOException {
/*  166 */     indentln("ClassFile {");
/*  167 */     println("magic = 0x" + Integer.toHexString(dis.readInt()));
/*  168 */     println("minor_version = " + dis.readShort());
/*  169 */     println("major_version = " + dis.readShort());
/*      */     
/*  171 */     this.constantPool = new ConstantPoolInfo[dis.readShort()];
/*  172 */     for (short i = 1; i < this.constantPool.length;) {
/*  173 */       ConstantPoolInfo cpi = readConstantPoolInfo(dis);
/*  174 */       i = (short)(i + 1);this.constantPool[i] = cpi;
/*  175 */       for (int j = cpi.getSizeInConstantPool(); j > 1; j--) {
/*  176 */         i = (short)(i + 1);this.constantPool[i] = null;
/*      */       }
/*      */     }
/*  179 */     indentln("constant_pool[] = {");
/*  180 */     for (short i = 1; i < this.constantPool.length;) {
/*  181 */       print(i + ": ");
/*  182 */       ConstantPoolInfo cpi = this.constantPool[i];
/*  183 */       cpi.print();
/*  184 */       println();
/*  185 */       i = (short)(i + cpi.getSizeInConstantPool());
/*      */     }
/*  187 */     unindentln("}");
/*      */     
/*  189 */     println("access_flags = " + decodeAccess(dis.readShort()));
/*      */     
/*  191 */     short thisClass = dis.readShort();
/*  192 */     indent("this_class = " + thisClass + " (");
/*  193 */     printConstantPoolEntry(thisClass);
/*  194 */     unindentln(")");
/*      */     
/*  196 */     short superClass = dis.readShort();
/*  197 */     indent("super_class = " + superClass + " (");
/*  198 */     printConstantPoolEntry(superClass);
/*  199 */     unindentln(")");
/*      */     
/*  201 */     indentln("interfaces[] = {");
/*  202 */     short interfacesCount = dis.readShort();
/*  203 */     for (short i = 0; i < interfacesCount; i = (short)(i + 1)) {
/*  204 */       printConstantPoolEntry(null, dis.readShort());
/*  205 */       println();
/*      */     }
/*  207 */     unindentln("}");
/*      */     
/*  209 */     indentln("fields[] = {");
/*  210 */     short fieldsCount = dis.readShort();
/*  211 */     for (short i = 0; i < fieldsCount; i = (short)(i + 1)) {
/*  212 */       disasmFieldInfo(dis);
/*  213 */       println();
/*      */     }
/*  215 */     unindentln("}");
/*      */     
/*      */ 
/*      */ 
/*  219 */     short methodsCount = dis.readShort();
/*  220 */     MethodInfo[] methodInfos = new MethodInfo[methodsCount];
/*  221 */     for (int i = 0; i < methodsCount; i++) { methodInfos[i] = readMethodInfo(dis);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  226 */     short attributesCount = dis.readShort();
/*  227 */     AttributeInfo[] attributes = new AttributeInfo[attributesCount];
/*  228 */     for (short i = 0; i < attributesCount; i = (short)(i + 1)) { attributes[i] = readAttributeInfo(dis);
/*      */     }
/*      */     
/*  231 */     Map sourceLines = new HashMap();
/*      */     
/*  233 */     for (int i = 0; i < attributes.length; i++) {
/*  234 */       if ((attributes[i] instanceof SourceFileAttribute)) {
/*  235 */         ConstantPoolInfo cpi = getConstantPoolEntry(((SourceFileAttribute)attributes[i]).sourceFileIndex);
/*      */         
/*      */ 
/*  238 */         if ((cpi instanceof ConstantUtf8Info)) {
/*  239 */           String sourceFile = ((ConstantUtf8Info)cpi).getValue();
/*      */           LineNumberReader lnr;
/*      */           try {
/*  242 */             lnr = new LineNumberReader(new FileReader(new File(this.sourceDirectory, sourceFile)));
/*      */           }
/*      */           catch (FileNotFoundException ex) {
/*      */             break;
/*      */           }
/*      */           try {
/*      */             for (;;) {
/*  249 */               String sl = lnr.readLine();
/*  250 */               if (sl == null) break;
/*  251 */               sourceLines.put(new Integer(lnr.getLineNumber()), sl);
/*      */             }
/*      */           } finally {
/*  254 */             lnr.close();
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  261 */     indentln("methods[] = {");
/*  262 */     for (short i = 0; i < methodInfos.length; i = (short)(i + 1)) {
/*  263 */       methodInfos[i].print(sourceLines);
/*  264 */       println();
/*      */     }
/*  266 */     unindentln("}");
/*      */     
/*  268 */     indentln("attributes[] = {");
/*  269 */     for (short i = 0; i < attributes.length; i = (short)(i + 1)) {
/*  270 */       attributes[i].print();
/*  271 */       println();
/*      */     }
/*  273 */     unindentln("}");
/*  274 */     unindent("}");
/*      */   }
/*      */   
/*      */   private ConstantPoolInfo readConstantPoolInfo(DataInputStream dis) throws IOException {
/*  278 */     byte tag = dis.readByte();
/*  279 */     switch (tag)
/*      */     {
/*      */     case 7: 
/*  282 */       short nameIndex = dis.readShort();
/*  283 */       new ConstantPoolInfo(nameIndex) { private final short val$nameIndex;
/*      */         
/*  285 */         public void print() { Disassembler.this.print("CONSTANT_Class_info { ");
/*  286 */           Disassembler.this.printConstantPoolEntry("name_index", this.val$nameIndex);
/*  287 */           Disassembler.this.print(" }");
/*      */         }
/*      */       };
/*      */     
/*      */ 
/*      */     case 9: 
/*  293 */       short classIndex = dis.readShort();
/*  294 */       short nameAndTypeIndex = dis.readShort();
/*  295 */       new ConstantPoolInfo(classIndex) { private final short val$classIndex;
/*      */         
/*  297 */         public void print() { Disassembler.this.indentln("CONSTANT_Fieldref_info {");
/*  298 */           Disassembler.this.printConstantPoolEntry("class_index", this.val$classIndex);
/*  299 */           Disassembler.this.println();
/*  300 */           Disassembler.this.printConstantPoolEntry("name_and_type_index", this.val$nameAndTypeIndex);
/*  301 */           Disassembler.this.println();
/*  302 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     
/*      */ 
/*      */     case 10: 
/*  308 */       short classIndex = dis.readShort();
/*  309 */       short nameAndTypeIndex = dis.readShort();
/*  310 */       new ConstantPoolInfo(classIndex) { private final short val$nameAndTypeIndex;
/*      */         
/*  312 */         public void print() { Disassembler.this.indentln("CONSTANT_Methodref_info {");
/*  313 */           Disassembler.this.printConstantPoolEntry("class_index", this.val$classIndex);
/*  314 */           Disassembler.this.println();
/*  315 */           Disassembler.this.printConstantPoolEntry("name_and_type_index", this.val$nameAndTypeIndex);
/*  316 */           Disassembler.this.println();
/*  317 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     
/*      */ 
/*      */     case 11: 
/*  323 */       short classIndex = dis.readShort();
/*  324 */       short nameAndTypeIndex = dis.readShort();
/*  325 */       new ConstantPoolInfo(classIndex) { private final short val$classIndex;
/*      */         
/*  327 */         public void print() { Disassembler.this.indentln("CONSTANT_InterfaceMethodref_info {");
/*  328 */           Disassembler.this.printConstantPoolEntry("class_index", this.val$classIndex);
/*  329 */           Disassembler.this.println();
/*  330 */           Disassembler.this.printConstantPoolEntry("name_and_type_index", this.val$nameAndTypeIndex);
/*  331 */           Disassembler.this.println();
/*  332 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     
/*      */ 
/*      */     case 8: 
/*  338 */       short stringIndex = dis.readShort();
/*  339 */       new ConstantPoolInfo(stringIndex) { private final short val$nameAndTypeIndex;
/*      */         
/*  341 */         public void print() { Disassembler.ConstantPoolInfo cpi = Disassembler.this.getConstantPoolEntry(this.val$stringIndex);
/*      */           String s;
/*  343 */           String s; if (cpi == null) {
/*  344 */             s = "INVALID CONSTANT POOL INDEX";
/*      */           } else { String s;
/*  346 */             if (!(cpi instanceof Disassembler.ConstantUtf8Info)) {
/*  347 */               s = "STRING CONSTANT VALUE IS NOT UTF8";
/*      */             } else {
/*  349 */               s = ((Disassembler.ConstantUtf8Info)cpi).getValue();
/*  350 */               if (s.length() > 80) {
/*  351 */                 s = Disassembler.stringToJavaLiteral(s.substring(0, 80)) + "... (" + s.length() + " chars)";
/*      */ 
/*      */ 
/*      */               }
/*      */               else
/*      */               {
/*      */ 
/*  358 */                 s = Disassembler.stringToJavaLiteral(s); }
/*      */             }
/*      */           }
/*  361 */           if (Disassembler.this.verbose) {
/*  362 */             Disassembler.this.print("CONSTANT_String_info { string_index = " + this.val$stringIndex + " (" + s + ") }");
/*      */ 
/*      */ 
/*      */           }
/*      */           else
/*      */           {
/*      */ 
/*      */ 
/*  370 */             Disassembler.this.print(s);
/*      */           }
/*      */         }
/*      */       };
/*      */     
/*      */ 
/*      */     case 3: 
/*  377 */       int bytes = dis.readInt();
/*  378 */       new ConstantPoolInfo(bytes) { private final short val$classIndex;
/*      */         private final short val$nameAndTypeIndex;
/*  380 */         public void print() { Disassembler.this.print("CONSTANT_Integer_info { bytes = " + this.val$bytes + " }"); }
/*      */       };
/*      */     
/*      */ 
/*      */ 
/*      */     case 4: 
/*  386 */       float bytes = dis.readFloat();
/*  387 */       new ConstantPoolInfo(bytes) { private final short val$stringIndex;
/*      */         
/*  389 */         public void print() { Disassembler.this.print("CONSTANT_Float_info { bytes = " + this.val$bytes + " }"); }
/*      */       };
/*      */     
/*      */ 
/*      */ 
/*      */     case 5: 
/*  395 */       long bytes = dis.readLong();
/*  396 */       new ConstantPoolInfo(bytes) { private final int val$bytes;
/*      */         private final float val$bytes;
/*  398 */         public void print() { Disassembler.this.print("CONSTANT_Long_info { high_bytes/low_bytes = " + this.val$bytes + " }"); }
/*      */         
/*  400 */         private final long val$bytes; public int getSizeInConstantPool() { return 2; }
/*      */       };
/*      */     
/*      */ 
/*      */     case 6: 
/*  405 */       double bytes = dis.readDouble();
/*  406 */       new ConstantPoolInfo(bytes) { private final double val$bytes;
/*      */         
/*  408 */         public void print() { Disassembler.this.print("CONSTANT_Double_info { high_bytes/low_bytes = " + this.val$bytes + " }"); }
/*      */         
/*  410 */         public int getSizeInConstantPool() { return 2; }
/*      */       };
/*      */     
/*      */ 
/*      */     case 12: 
/*  415 */       short nameIndex = dis.readShort();
/*  416 */       short descriptorIndex = dis.readShort();
/*  417 */       new ConstantPoolInfo(nameIndex) { private final short val$nameIndex;
/*      */         private final short val$descriptorIndex;
/*  419 */         public void print() { Disassembler.this.indentln("CONSTANT_NameAndType_info {");
/*  420 */           Disassembler.this.printConstantPoolEntry("name_index", this.val$nameIndex);
/*  421 */           Disassembler.this.println();
/*  422 */           Disassembler.this.printConstantPoolEntry("descriptor_index", this.val$descriptorIndex);
/*  423 */           Disassembler.this.println();
/*  424 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     
/*      */     case 1: 
/*  429 */       return new ConstantUtf8Info(dis.readUTF());
/*      */     }
/*  431 */     throw new JaninoRuntimeException("Invalid cp_info tag \"" + tag + "\"");
/*      */   }
/*      */   
/*      */   private static String stringToJavaLiteral(String s)
/*      */   {
/*  436 */     for (int i = 0; i < s.length();) {
/*  437 */       char c = s.charAt(i);
/*  438 */       int idx = "\r\n\"\t\b".indexOf(c);
/*  439 */       if (idx == -1) {
/*  440 */         i++;
/*      */       } else {
/*  442 */         s = s.substring(0, i) + '\\' + "rn\"tb".charAt(idx) + s.substring(i + 1);
/*  443 */         i += 2;
/*      */       }
/*  445 */       if (i >= 80) break;
/*      */     }
/*  447 */     return '"' + s + '"';
/*      */   }
/*      */   
/*      */   private void disasmFieldInfo(DataInputStream dis) throws IOException {
/*  451 */     indentln("field_info {");
/*  452 */     println("access_flags = " + decodeAccess(dis.readShort()));
/*      */     
/*  454 */     printConstantPoolEntry("name_index", dis.readShort());
/*  455 */     println();
/*      */     
/*  457 */     printConstantPoolEntry("descriptor_index", dis.readShort());
/*  458 */     println();
/*      */     
/*  460 */     indentln("attributes[] = {");
/*  461 */     short attributesCount = dis.readShort();
/*  462 */     for (short i = 0; i < attributesCount; i = (short)(i + 1)) {
/*  463 */       disasmAttributeInfo(dis);
/*  464 */       println();
/*      */     }
/*  466 */     unindentln("}");
/*  467 */     unindent("}");
/*      */   }
/*      */   
/*      */   private MethodInfo readMethodInfo(DataInputStream dis) throws IOException {
/*  471 */     short accessFlags = dis.readShort();
/*  472 */     short nameIndex = dis.readShort();
/*  473 */     short descriptorIndex = dis.readShort();
/*      */     
/*      */ 
/*  476 */     short attributesCount = dis.readShort();
/*  477 */     AttributeInfo[] attributeInfos = new AttributeInfo[attributesCount];
/*  478 */     for (short i = 0; i < attributesCount; i = (short)(i + 1)) { attributeInfos[i] = readAttributeInfo(dis);
/*      */     }
/*  480 */     new MethodInfo() { private final short val$accessFlags;
/*      */       private final short val$nameIndex;
/*  482 */       public void print(Map sourceLines) { Disassembler.this.indentln("method_info {");
/*  483 */         Disassembler.this.println("access_flags = " + Disassembler.decodeAccess(this.val$accessFlags));
/*      */         
/*  485 */         Disassembler.this.printConstantPoolEntry("name_index", this.val$nameIndex);
/*  486 */         Disassembler.this.println();
/*      */         
/*  488 */         Disassembler.this.printConstantPoolEntry("descriptor_index", this.val$descriptorIndex);
/*  489 */         Disassembler.this.println();
/*      */         
/*  491 */         Disassembler.this.indentln("attributes[] = {");
/*  492 */         for (short i = 0; i < this.val$attributeInfos.length; i = (short)(i + 1)) {
/*  493 */           Disassembler.AttributeInfo ai = this.val$attributeInfos[i];
/*  494 */           if ((ai instanceof Disassembler.SourceRelatedAttributeInfo)) {
/*  495 */             ((Disassembler.SourceRelatedAttributeInfo)ai).print(sourceLines);
/*      */           } else {
/*  497 */             ai.print();
/*      */           }
/*  499 */           Disassembler.this.println();
/*      */         }
/*  501 */         Disassembler.this.unindentln("}");
/*  502 */         Disassembler.this.unindent("}");
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */   private final short val$descriptorIndex;
/*      */   
/*      */   private final Disassembler.AttributeInfo[] val$attributeInfos;
/*      */   
/*      */   private void disasmAttributeInfo(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  515 */     readAttributeInfo(dis).print();
/*      */   }
/*      */   
/*      */   private AttributeInfo readAttributeInfo(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  521 */     short attributeNameIndex = dis.readShort();
/*      */     
/*      */ 
/*  524 */     ConstantPoolInfo cpi = getConstantPoolEntry(attributeNameIndex);
/*  525 */     String attributeName; String attributeName; if (cpi == null) {
/*  526 */       attributeName = "INVALID CONSTANT POOL INDEX";
/*      */     } else { String attributeName;
/*  528 */       if (!(cpi instanceof ConstantUtf8Info)) {
/*  529 */         attributeName = "ATTRIBUTE NAME IS NOT UTF8";
/*      */       } else {
/*  531 */         attributeName = ((ConstantUtf8Info)cpi).getValue();
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  536 */     int attributeLength = dis.readInt();
/*  537 */     byte[] ba = new byte[attributeLength];
/*  538 */     if (dis.read(ba) != ba.length) throw new EOFException();
/*  539 */     ByteArrayInputStream bais = new ByteArrayInputStream(ba);
/*  540 */     DataInputStream dis2 = new DataInputStream(bais);
/*      */     
/*      */ 
/*  543 */     AttributeInfo res = readAttributeBody(attributeName, dis2);
/*      */     
/*      */ 
/*  546 */     int av = bais.available();
/*  547 */     if (av > 0) { throw new JaninoRuntimeException(av + " extraneous bytes in attribute \"" + attributeName + "\"");
/*      */     }
/*  549 */     return res;
/*      */   }
/*      */   
/*      */   private AttributeInfo readAttributeBody(String attributeName, DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  555 */     if (attributeName.equals("ConstantValue")) {
/*  556 */       short constantValueIndex = dis.readShort();
/*  557 */       new AttributeInfo() { private final short val$constantValueIndex;
/*      */         
/*  559 */         public void print() { Disassembler.this.indent("ConstantValue " + this.val$constantValueIndex + " (");
/*  560 */           Disassembler.this.printConstantPoolEntry(this.val$constantValueIndex);
/*  561 */           Disassembler.this.unindent(")");
/*      */         }
/*      */       };
/*      */     }
/*  565 */     if (attributeName.equals("Code")) {
/*  566 */       short maxStack = dis.readShort();
/*  567 */       short maxLocals = dis.readShort();
/*  568 */       byte[] code = readByteArray(dis, dis.readInt());
/*  569 */       ExceptionTableEntry[] exceptionTable = readExceptionTable(dis);
/*  570 */       AttributeInfo[] attributes = readAttributes(dis);
/*  571 */       new SourceRelatedAttributeInfo(maxStack) { private final short val$maxStack;
/*      */         private final short val$maxLocals;
/*  573 */         public void print(Map sourceLines) { Disassembler.this.indentln("Code {");
/*  574 */           Disassembler.this.println("max_stack = " + this.val$maxStack);
/*  575 */           Disassembler.this.println("max_locals = " + this.val$maxLocals);
/*      */           
/*  577 */           Disassembler.this.indentln("code = { ");
/*  578 */           Disassembler.LocalVariableTableAttribute localVariableTableAttribute = null;
/*  579 */           Disassembler.LineNumberTableAttribute lineNumberTableAttribute = null;
/*  580 */           for (int i = 0; i < this.val$attributes.length; i++) {
/*  581 */             Disassembler.AttributeInfo a = this.val$attributes[i];
/*  582 */             if ((a instanceof Disassembler.LocalVariableTableAttribute)) {
/*  583 */               if (localVariableTableAttribute != null) {
/*  584 */                 throw new JaninoRuntimeException("Duplicate LocalVariableTable attribute");
/*      */               }
/*  586 */               localVariableTableAttribute = (Disassembler.LocalVariableTableAttribute)a;
/*      */             }
/*  588 */             if ((a instanceof Disassembler.LineNumberTableAttribute)) {
/*  589 */               if (lineNumberTableAttribute != null) {
/*  590 */                 throw new JaninoRuntimeException("Duplicate LineNumberTable attribute");
/*      */               }
/*  592 */               lineNumberTableAttribute = (Disassembler.LineNumberTableAttribute)a;
/*      */             }
/*      */           }
/*      */           try {
/*  596 */             Disassembler.this.disasmBytecode(new ByteArrayInputStream(this.val$code), localVariableTableAttribute, lineNumberTableAttribute, sourceLines);
/*      */           }
/*      */           catch (IOException ignored) {}
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  605 */           Disassembler.this.unindentln("}");
/*      */           
/*  607 */           Disassembler.this.indentln("exception_table = {");
/*  608 */           for (int i = 0; i < this.val$exceptionTable.length; i++) {
/*  609 */             this.val$exceptionTable[i].print();
/*  610 */             Disassembler.this.println();
/*      */           }
/*  612 */           Disassembler.this.unindentln("}");
/*      */           
/*  614 */           Disassembler.this.indentln("attributes[] = {");
/*  615 */           for (int i = 0; i < this.val$attributes.length; i++) {
/*  616 */             this.val$attributes[i].print();
/*  617 */             Disassembler.this.println();
/*      */           }
/*  619 */           Disassembler.this.unindentln("}");
/*  620 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     }
/*  624 */     if (attributeName.equals("Exceptions")) {
/*  625 */       short[] exceptionIndexTable = readShortArray(dis, dis.readShort());
/*  626 */       new AttributeInfo() { private final Disassembler.AttributeInfo[] val$attributes;
/*      */         private final byte[] val$code;
/*  628 */         public void print() { Disassembler.this.indentln("Exceptions {");
/*  629 */           Disassembler.this.indentln("exception_index_table = {");
/*  630 */           for (short i = 0; i < this.val$exceptionIndexTable.length; i = (short)(i + 1)) {
/*  631 */             short exceptionIndex = this.val$exceptionIndexTable[i];
/*  632 */             Disassembler.this.print(exceptionIndex + " (");
/*  633 */             Disassembler.this.printConstantPoolEntry(exceptionIndex);
/*  634 */             Disassembler.this.println("),");
/*      */           }
/*  636 */           Disassembler.this.unindentln("}");
/*  637 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     }
/*  641 */     if (attributeName.equals("InnerClasses")) {
/*  642 */       short[] data = readShortArray(dis, 4 * dis.readShort());
/*  643 */       new AttributeInfo() { private final Disassembler.ExceptionTableEntry[] val$exceptionTable;
/*      */         
/*  645 */         public void print() { Disassembler.this.indentln("InnerClasses {");
/*  646 */           Disassembler.this.indentln("classes = {");
/*  647 */           for (int i = 0; i < this.val$data.length; i += 4) {
/*  648 */             Disassembler.this.indentln("{");
/*  649 */             Disassembler.this.printConstantPoolEntry("inner_class_info_index", this.val$data[i]);
/*  650 */             Disassembler.this.println();
/*      */             
/*  652 */             short outerClassInfoIndex = this.val$data[(i + 1)];
/*  653 */             if (outerClassInfoIndex == 0) {
/*  654 */               Disassembler.this.print("(not a member)");
/*      */             } else {
/*  656 */               Disassembler.this.printConstantPoolEntry("outer_class_info_index", outerClassInfoIndex);
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*  661 */             Disassembler.this.println();
/*      */             
/*  663 */             short innerNameIndex = this.val$data[(i + 2)];
/*  664 */             if (innerNameIndex == 0) {
/*  665 */               Disassembler.this.print("(anonymous)");
/*      */             } else {
/*  667 */               Disassembler.this.printConstantPoolEntry("inner_name_index", innerNameIndex);
/*      */             }
/*  669 */             Disassembler.this.println();
/*      */             
/*  671 */             Disassembler.this.println("inner_class_access_flags = " + Disassembler.decodeAccess(this.val$data[(i + 3)]));
/*      */             
/*      */ 
/*      */ 
/*  675 */             Disassembler.this.unindentln(i == this.val$data.length - 1 ? "}" : "},");
/*      */           }
/*  677 */           Disassembler.this.unindentln("}");
/*  678 */           Disassembler.this.unindent("}");
/*      */         }
/*      */       };
/*      */     }
/*  682 */     if (attributeName.equals("Synthetic")) {
/*  683 */       new AttributeInfo() { private final short[] val$exceptionIndexTable;
/*      */         private final short[] val$data;
/*  685 */         public void print() { Disassembler.this.print("Synthetic"); }
/*      */       };
/*      */     }
/*      */     
/*  689 */     if (attributeName.equals("SourceFile")) {
/*  690 */       return new SourceFileAttribute(dis.readShort());
/*      */     }
/*  692 */     if (attributeName.equals("LineNumberTable")) {
/*  693 */       return new LineNumberTableAttribute(readShortArray(dis, 2 * dis.readShort()));
/*      */     }
/*  695 */     if (attributeName.equals("LocalVariableTable")) {
/*  696 */       return new LocalVariableTableAttribute(readShortArray(dis, 5 * dis.readShort()));
/*      */     }
/*  698 */     if (attributeName.equals("Deprecated")) {
/*  699 */       new AttributeInfo() {
/*      */         public void print() {
/*  701 */           Disassembler.this.print("Deprecated");
/*      */         }
/*      */       };
/*      */     }
/*      */     
/*  706 */     ByteArrayOutputStream baos = new ByteArrayOutputStream();
/*      */     for (;;) {
/*  708 */       int c = dis.read();
/*  709 */       if (c == -1) break;
/*  710 */       baos.write(c);
/*      */     }
/*  712 */     new AttributeInfo() { private byte[] info;
/*      */       
/*  714 */       public void print() { Disassembler.this.indentln(this.val$attributeName + " {");
/*  715 */         Disassembler.this.print("info = { ");
/*  716 */         for (int i = 0; i < this.info.length; i++) {
/*  717 */           Disassembler.this.print("0x" + Integer.toHexString(0xFF & this.info[i]) + ", ");
/*      */         }
/*  719 */         Disassembler.this.println("}");
/*  720 */         Disassembler.this.unindent("}");
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */   private static abstract interface MethodInfo { public abstract void print(Map paramMap); }
/*      */   
/*      */   private static abstract interface AttributeInfo { public abstract void print(); }
/*      */   
/*  729 */   private static abstract class SourceRelatedAttributeInfo implements Disassembler.AttributeInfo { SourceRelatedAttributeInfo(Disassembler.1 x0) { this(); }
/*      */     public abstract void print(Map paramMap);
/*  731 */     public void print() { throw new UnsupportedOperationException("print"); }
/*      */     
/*      */     private SourceRelatedAttributeInfo() {}
/*      */   }
/*      */   
/*      */   private class SourceFileAttribute implements Disassembler.AttributeInfo { final short sourceFileIndex;
/*      */     
/*  738 */     SourceFileAttribute(short sourceFileIndex) { this.sourceFileIndex = sourceFileIndex; }
/*      */     
/*      */     public void print() {
/*  741 */       Disassembler.this.indentln("SourceFile {");
/*  742 */       Disassembler.this.printConstantPoolEntry("sourcefile_index", this.sourceFileIndex);
/*  743 */       Disassembler.this.println();
/*  744 */       Disassembler.this.unindent("}");
/*      */     }
/*      */   }
/*      */   
/*      */   public class LineNumberTableAttribute implements Disassembler.AttributeInfo { private final short[] data;
/*      */     
/*  750 */     public LineNumberTableAttribute(short[] data) { this.data = data; }
/*      */     
/*      */     public void print() {
/*  753 */       Disassembler.this.indentln("LineNumberTable {");
/*  754 */       Disassembler.this.indentln("line_number_table = {");
/*  755 */       for (short i = 0; i < this.data.length; i = (short)(i + 2)) {
/*  756 */         Disassembler.this.print("start_pc = " + this.data[i]);
/*  757 */         Disassembler.this.print(", ");
/*  758 */         Disassembler.this.println("line_number = " + this.data[(i + 1)]);
/*      */       }
/*  760 */       Disassembler.this.unindentln("}");
/*  761 */       Disassembler.this.unindent("}");
/*      */     }
/*      */     
/*  764 */     public short findLineNumber(short offset) { for (short i = 0; i < this.data.length; i = (short)(i + 2)) {
/*  765 */         if (this.data[i] == offset) return this.data[(i + 1)];
/*      */       }
/*  767 */       return -1; } }
/*      */   
/*      */   private class LocalVariableTableAttribute implements Disassembler.AttributeInfo { private short[] data;
/*      */     
/*  771 */     public LocalVariableTableAttribute(short[] data) { this.data = data; }
/*      */     
/*  773 */     public void print() { Disassembler.this.indentln("LocalVariableTable {");
/*  774 */       Disassembler.this.indentln("local_variable_table = {");
/*  775 */       for (int i = 0; i < this.data.length; i += 5) {
/*  776 */         Disassembler.this.print("start_pc = " + this.data[i]);
/*  777 */         Disassembler.this.print(", ");
/*  778 */         Disassembler.this.print("length = " + this.data[(i + 1)]);
/*  779 */         Disassembler.this.print(", ");
/*  780 */         Disassembler.this.printConstantPoolEntry("name_index", this.data[(i + 2)]);
/*  781 */         Disassembler.this.print(", ");
/*  782 */         Disassembler.this.printConstantPoolEntry("descriptor_index", this.data[(i + 3)]);
/*  783 */         Disassembler.this.print(", ");
/*  784 */         Disassembler.this.println("index = " + this.data[(i + 4)]);
/*      */       }
/*  786 */       Disassembler.this.unindentln("}");
/*  787 */       Disassembler.this.unindent("}");
/*      */     }
/*      */     
/*  790 */     public String find(short index, int instructionOffset) { for (int i = 0; i < this.data.length; i += 5) {
/*  791 */         short startPC = this.data[i];
/*  792 */         short length = this.data[(i + 1)];
/*  793 */         if ((instructionOffset >= startPC) && (instructionOffset <= startPC + length) && (index == this.data[(i + 4)]))
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  798 */           String name = ((Disassembler.ConstantUtf8Info)Disassembler.this.getConstantPoolEntry(this.data[(i + 2)])).getValue();
/*      */           
/*      */ 
/*  801 */           String descriptor = ((Disassembler.ConstantUtf8Info)Disassembler.this.getConstantPoolEntry(this.data[(i + 3)])).getValue();
/*      */           
/*      */ 
/*  804 */           return Disassembler.decodeDescriptor(descriptor) + " " + name;
/*      */         }
/*      */       }
/*  807 */       return "anonymous";
/*      */     }
/*      */   }
/*      */   
/*      */   private static byte[] readByteArray(InputStream is, int size) throws IOException {
/*  812 */     byte[] res = new byte[size];
/*  813 */     if (is.read(res) != size) throw new EOFException("EOF in byte array");
/*  814 */     return res;
/*      */   }
/*      */   
/*  817 */   private ExceptionTableEntry[] readExceptionTable(DataInputStream dis) throws IOException { ExceptionTableEntry[] res = new ExceptionTableEntry[dis.readShort()];
/*  818 */     for (short i = 0; i < res.length; i = (short)(i + 1)) {
/*  819 */       res[i] = new ExceptionTableEntry(dis.readShort(), dis.readShort(), dis.readShort(), dis.readShort());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  826 */     return res; }
/*      */   
/*      */   private class ExceptionTableEntry { private short startPC;
/*      */     private short endPC;
/*  830 */     ExceptionTableEntry(short startPC, short endPC, short handlerPC, short catchType) { this.startPC = startPC;
/*  831 */       this.endPC = endPC;
/*  832 */       this.handlerPC = handlerPC;
/*  833 */       this.catchType = catchType; }
/*      */     
/*      */     private short handlerPC;
/*  836 */     private short catchType; public void print() { Disassembler.this.print("start_pc = " + this.startPC + ", end_pc = " + this.endPC + ", handler_pc = " + this.handlerPC + ", catch_type = " + this.catchType + " (");
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  847 */       if (this.catchType == 0) {
/*  848 */         Disassembler.this.print("finally");
/*      */       } else {
/*  850 */         Disassembler.this.printConstantPoolEntry(this.catchType);
/*      */       }
/*  852 */       Disassembler.this.print(")");
/*      */     }
/*      */   }
/*      */   
/*      */   private AttributeInfo[] readAttributes(DataInputStream dis) throws IOException
/*      */   {
/*  858 */     AttributeInfo[] res = new AttributeInfo[dis.readShort()];
/*  859 */     for (int i = 0; i < res.length; i++) {
/*  860 */       res[i] = readAttributeInfo(dis);
/*      */     }
/*  862 */     return res;
/*      */   }
/*      */   
/*      */   private static short[] readShortArray(DataInputStream dis, int size) throws IOException {
/*  866 */     short[] res = new short[size];
/*  867 */     for (int i = 0; i < res.length; i++) res[i] = dis.readShort();
/*  868 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */   private final String val$attributeName;
/*      */   private final ByteArrayOutputStream val$baos;
/*      */   private void disasmBytecode(InputStream is, LocalVariableTableAttribute localVariableTableAttribute, LineNumberTableAttribute lineNumberTableAttribute, Map sourceLines)
/*      */     throws IOException
/*      */   {
/*  877 */     CountingInputStream cis = new CountingInputStream(is);
/*  878 */     DataInputStream dis = new DataInputStream(cis);
/*      */     for (;;)
/*      */     {
/*  881 */       short instructionOffset = (short)(int)cis.getCount();
/*      */       
/*  883 */       int opcode = dis.read();
/*  884 */       if (opcode == -1) { return;
/*      */       }
/*  886 */       if (lineNumberTableAttribute != null) {
/*  887 */         short lineNumber = lineNumberTableAttribute.findLineNumber(instructionOffset);
/*  888 */         if (lineNumber != -1) {
/*  889 */           String sourceLine = (String)sourceLines.get(new Integer(lineNumber));
/*  890 */           if (sourceLine == null) sourceLine = "(Source line not available)";
/*  891 */           println("            Line " + lineNumber + ": " + sourceLine);
/*      */         }
/*      */       }
/*  894 */       print(instructionOffset + ": ");
/*      */       
/*  896 */       Instruction instruction = opcodeToInstruction[opcode];
/*  897 */       if (instruction == null) {
/*  898 */         println("??? (invalid opcode \"" + opcode + "\")");
/*      */       }
/*      */       else {
/*  901 */         disasmInstruction(instruction, dis, instructionOffset, localVariableTableAttribute);
/*  902 */         println();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private void disasmInstruction(Instruction instruction, DataInputStream dis, int instructionOffset, LocalVariableTableAttribute localVariableTableAttribute)
/*      */     throws IOException
/*      */   {
/*  911 */     print(instruction.getMnemonic());
/*  912 */     Operand[] operands = instruction.getOperands();
/*  913 */     if (operands != null) {
/*  914 */       for (int i = 0; i < operands.length; i++) {
/*  915 */         Operand operand = operands[i];
/*  916 */         print(" ");
/*  917 */         operand.disasm(dis, instructionOffset, localVariableTableAttribute, this);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*  922 */   private static final String[] instructions = { "50  aaload", "83  aastore", "1   aconst_null", "25  aload         localvariablearrayindex1", "42  aload_0       localvariablearrayindex_0", "43  aload_1       localvariablearrayindex_1", "44  aload_2       localvariablearrayindex_2", "45  aload_3       localvariablearrayindex_3", "189 anewarray     constantpoolindex2", "176 areturn", "190 arraylength", "58  astore        localvariablearrayindex1", "75  astore_0      localvariablearrayindex_0", "76  astore_1      localvariablearrayindex_1", "77  astore_2      localvariablearrayindex_2", "78  astore_3      localvariablearrayindex_3", "191 athrow", "51  baload", "84  bastore", "16  bipush        signedbyte", "52  caload", "85  castore", "192 checkcast     constantpoolindex2", "144 d2f", "142 d2i", "143 d2l", "99  dadd", "49  daload", "82  dastore", "152 dcmpg", "151 dcmpl", "14  dconst_0", "15  dconst_1", "111 ddiv", "24  dload         localvariablearrayindex1", "38  dload_0       localvariablearrayindex_0", "39  dload_1       localvariablearrayindex_1", "40  dload_2       localvariablearrayindex_2", "41  dload_3       localvariablearrayindex_3", "107 dmul", "119 dneg", "115 drem", "175 dreturn", "57  dstore        localvariablearrayindex1", "71  dstore_0      localvariablearrayindex_0", "72  dstore_1      localvariablearrayindex_1", "73  dstore_2      localvariablearrayindex_2", "74  dstore_3      localvariablearrayindex_3", "103 dsub", "89  dup", "90  dup_x1", "91  dup_x2", "92  dup2", "93  dup2_x1", "94  dup2_x2", "141 f2d", "139 f2i", "140 f2l", "98  fadd", "48  faload", "81  fastore", "150 fcmpg", "149 fcmpl", "11  fconst_0", "12  fconst_1", "13  fconst_2", "110 fdiv", "23  fload         localvariablearrayindex1", "34  fload_0       localvariablearrayindex_0", "35  fload_1       localvariablearrayindex_1", "36  fload_2       localvariablearrayindex_2", "37  fload_3       localvariablearrayindex_3", "106 fmul", "118 fneg", "114 frem", "174 freturn", "56  fstore        localvariablearrayindex1", "67  fstore_0      localvariablearrayindex_0", "68  fstore_1      localvariablearrayindex_1", "69  fstore_2      localvariablearrayindex_2", "70  fstore_3      localvariablearrayindex_3", "102 fsub", "180 getfield      constantpoolindex2", "178 getstatic     constantpoolindex2", "167 goto          branchoffset2", "200 goto_w        branchoffset4", "145 i2b", "146 i2c", "135 i2d", "134 i2f", "133 i2l", "147 i2s", "96  iadd", "46  iaload", "126 iand", "79  iastore", "2   iconst_m1", "3   iconst_0", "4   iconst_1", "5   iconst_2", "6   iconst_3", "7   iconst_4", "8   iconst_5", "108 idiv", "165 if_acmpeq", "166 if_acmpne", "159 if_icmpeq     branchoffset2", "160 if_icmpne     branchoffset2", "161 if_icmplt     branchoffset2", "162 if_icmpge     branchoffset2", "163 if_icmpgt     branchoffset2", "164 if_icmple     branchoffset2", "153 ifeq          branchoffset2", "154 ifne          branchoffset2", "155 iflt          branchoffset2", "156 ifge          branchoffset2", "157 ifgt          branchoffset2", "158 ifle          branchoffset2", "199 ifnonnull     branchoffset2", "198 ifnull        branchoffset2", "132 iinc          localvariablearrayindex1 signedbyte", "21  iload         localvariablearrayindex1", "26  iload_0       localvariablearrayindex_0", "27  iload_1       localvariablearrayindex_1", "28  iload_2       localvariablearrayindex_2", "29  iload_3       localvariablearrayindex_3", "104 imul", "116 ineg", "193 instanceof      constantpoolindex2", "185 invokeinterface constantpoolindex2 signedbyte signedbyte", "183 invokespecial   constantpoolindex2", "184 invokestatic    constantpoolindex2", "182 invokevirtual   constantpoolindex2", "128 ior", "112 irem", "172 ireturn", "120 ishl", "122 ishr", "54  istore          localvariablearrayindex1", "59  istore_0        localvariablearrayindex_0", "60  istore_1        localvariablearrayindex_1", "61  istore_2        localvariablearrayindex_2", "62  istore_3        localvariablearrayindex_3", "100 isub", "124 iushr", "130 ixor", "168 jsr             branchoffset2", "201 jsr_w           branchoffset4", "138 l2d", "137 l2f", "136 l2i", "97  ladd", "47  laload", "127 land", "80  lastore", "148 lcmp", "9   lconst_0", "10  lconst_1", "18  ldc           constantpoolindex1", "19  ldc_w         constantpoolindex2", "20  ldc2_w        constantpoolindex2", "109 ldiv", "22  lload         localvariablearrayindex1", "30  lload_0       localvariablearrayindex_0", "31  lload_1       localvariablearrayindex_1", "32  lload_2       localvariablearrayindex_2", "33  lload_3       localvariablearrayindex_3", "105 lmul", "117 lneg", "171 lookupswitch  lookupswitch", "129 lor", "113 lrem", "173 lreturn", "121 lshl", "123 lshr", "55  lstore        localvariablearrayindex1", "63  lstore_0      localvariablearrayindex_0", "64  lstore_1      localvariablearrayindex_1", "65  lstore_2      localvariablearrayindex_2", "66  lstore_3      localvariablearrayindex_3", "101 lsub", "125 lushr", "131 lxor", "194 monitorenter", "195 monitorexit", "197 multianewarray constantpoolindex2 unsignedbyte", "187 new           constantpoolindex2", "188 newarray      atype", "0   nop", "87  pop", "88  pop2", "181 putfield      constantpoolindex2", "179 putstatic     constantpoolindex2", "169 ret           localvariablearrayindex1", "177 return", "53  saload", "86  sastore", "17  sipush        signedshort", "95  swap", "170 tableswitch   tableswitch", "196 wide          wide" };
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1125 */   private static final String[] wideInstructions = { "21  iload         localvariablearrayindex2", "23  fload         localvariablearrayindex2", "25  aload         localvariablearrayindex2", "22  lload         localvariablearrayindex2", "24  dload         localvariablearrayindex2", "54  istore        localvariablearrayindex2", "56  fstore        localvariablearrayindex2", "58  astore        localvariablearrayindex2", "55  lstore        localvariablearrayindex2", "57  dstore        localvariablearrayindex2", "169 ret           localvariablearrayindex2", "132 iinc          localvariablearrayindex2 signedshort" };
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1139 */   private static final Instruction[] opcodeToInstruction = new Instruction['Ā'];
/* 1140 */   private static final Instruction[] opcodeToWideInstruction = new Instruction['Ā'];
/*      */   
/* 1142 */   static { compileInstructions(instructions, opcodeToInstruction);
/* 1143 */     compileInstructions(wideInstructions, opcodeToWideInstruction);
/*      */   }
/*      */   
/* 1146 */   private static void compileInstructions(String[] instructions, Instruction[] opcodeToInstruction) { for (int j = 0; j < instructions.length; j++) {
/* 1147 */       StringTokenizer st = new StringTokenizer(instructions[j]);
/* 1148 */       String os = st.nextToken();
/* 1149 */       int opcode = Integer.parseInt(os);
/* 1150 */       String mnemonic = st.nextToken();
/* 1151 */       Operand[] operands = null;
/* 1152 */       if (st.hasMoreTokens()) {
/* 1153 */         List l = new ArrayList();
/* 1154 */         while (st.hasMoreTokens()) {
/* 1155 */           String s = st.nextToken();
/*      */           Operand operand;
/* 1157 */           if (s.equals("constantpoolindex1")) {
/* 1158 */             operand = new Operand()
/*      */             {
/*      */ 
/*      */               public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                 throws IOException
/*      */               {
/*      */ 
/* 1165 */                 short index = (short)(0xFF & dis.readByte());
/* 1166 */                 d.printConstantPoolEntry(index);
/*      */               }
/*      */             };
/*      */           } else { Operand operand;
/* 1170 */             if (s.equals("constantpoolindex2")) {
/* 1171 */               operand = new Operand()
/*      */               {
/*      */ 
/*      */ 
/*      */ 
/*      */                 public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                   throws IOException {
/* 1178 */                   d.printConstantPoolEntry(dis.readShort()); }
/*      */               };
/*      */             } else {
/*      */               Operand operand;
/* 1182 */               if (s.equals("localvariablearrayindex1")) {
/* 1183 */                 operand = new Operand()
/*      */                 {
/*      */ 
/*      */                   public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                     throws IOException
/*      */                   {
/*      */ 
/* 1190 */                     short index = (short)dis.readByte();
/* 1191 */                     d.print(index);
/* 1192 */                     if (localVariableTableAttribute != null) {
/* 1193 */                       d.print(" (" + localVariableTableAttribute.find(index, instructionOffset + 2) + ")");
/*      */                     }
/*      */                   }
/*      */                 };
/*      */               }
/*      */               else
/*      */               {
/*      */                 Operand operand;
/*      */                 
/* 1202 */                 if (s.equals("localvariablearrayindex2")) {
/* 1203 */                   operand = new Operand()
/*      */                   {
/*      */ 
/*      */                     public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                       throws IOException
/*      */                     {
/*      */ 
/* 1210 */                       short index = dis.readShort();
/* 1211 */                       d.print(index);
/* 1212 */                       if (localVariableTableAttribute != null) {
/* 1213 */                         d.print(" (" + localVariableTableAttribute.find(index, instructionOffset + 3) + ")");
/*      */                       }
/*      */                     }
/*      */                   };
/*      */                 }
/*      */                 else
/*      */                 {
/*      */                   Operand operand;
/*      */                   
/* 1222 */                   if (s.startsWith("localvariablearrayindex_")) {
/* 1223 */                     short index = Short.parseShort(s.substring(s.length() - 1));
/* 1224 */                     operand = new Operand()
/*      */                     {
/*      */                       private final short val$index;
/*      */                       
/*      */                       public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                         throws IOException
/*      */                       {
/* 1231 */                         if (localVariableTableAttribute != null)
/* 1232 */                           d.print("(" + localVariableTableAttribute.find(this.val$index, instructionOffset + 1) + ")");
/*      */                       }
/*      */                     };
/*      */                   } else {
/*      */                     Operand operand;
/* 1237 */                     if (s.equals("branchoffset2")) {
/* 1238 */                       operand = new Operand()
/*      */                       {
/*      */ 
/*      */ 
/*      */ 
/*      */                         public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                           throws IOException {
/* 1245 */                           d.print(instructionOffset + dis.readShort()); }
/*      */                       };
/*      */                     } else {
/*      */                       Operand operand;
/* 1249 */                       if (s.equals("branchoffset4")) {
/* 1250 */                         operand = new Operand()
/*      */                         {
/*      */ 
/*      */ 
/*      */ 
/*      */                           public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                             throws IOException {
/* 1257 */                             d.print(instructionOffset + dis.readInt()); }
/*      */                         };
/*      */                       } else {
/*      */                         Operand operand;
/* 1261 */                         if (s.equals("signedbyte")) {
/* 1262 */                           operand = new Operand()
/*      */                           {
/*      */ 
/*      */ 
/*      */ 
/*      */                             public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                               throws IOException {
/* 1269 */                               d.print(dis.readByte()); }
/*      */                           };
/*      */                         } else {
/*      */                           Operand operand;
/* 1273 */                           if (s.equals("unsignedbyte")) {
/* 1274 */                             operand = new Operand()
/*      */                             {
/*      */ 
/*      */ 
/*      */ 
/*      */                               public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                 throws IOException {
/* 1281 */                                 d.print(0xFF & dis.readByte()); }
/*      */                             };
/*      */                           } else {
/*      */                             Operand operand;
/* 1285 */                             if (s.equals("atype")) {
/* 1286 */                               operand = new Operand()
/*      */                               {
/*      */ 
/*      */                                 public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                   throws IOException
/*      */                                 {
/*      */ 
/* 1293 */                                   byte b = dis.readByte();
/* 1294 */                                   d.print(b == 11 ? "LONG" : b == 10 ? "INT" : b == 9 ? "SHORT" : b == 8 ? "BYTE" : b == 7 ? "DOUBLE" : b == 6 ? "FLOAT" : b == 5 ? "CHAR" : b == 4 ? "BOOLEAN" : new Integer(0xFF & b).toString());
/*      */                                 }
/*      */                               };
/*      */                             }
/*      */                             else
/*      */                             {
/*      */                               Operand operand;
/*      */                               
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1308 */                               if (s.equals("signedshort")) {
/* 1309 */                                 operand = new Operand()
/*      */                                 {
/*      */ 
/*      */ 
/*      */ 
/*      */                                   public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                     throws IOException {
/* 1316 */                                     d.print(dis.readShort()); }
/*      */                                 };
/*      */                               } else {
/*      */                                 Operand operand;
/* 1320 */                                 if (s.equals("tableswitch")) {
/* 1321 */                                   operand = new Operand()
/*      */                                   {
/*      */ 
/*      */                                     public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                       throws IOException
/*      */                                     {
/*      */ 
/* 1328 */                                       int npads = 3 - instructionOffset % 4;
/* 1329 */                                       for (int i = 0; i < npads; i++) {
/* 1330 */                                         if (dis.readByte() != 0) {
/* 1331 */                                           throw new JaninoRuntimeException("Non-zero pad byte in \"tableswitch\"");
/*      */                                         }
/*      */                                       }
/* 1334 */                                       d.print("default => " + (instructionOffset + dis.readInt()));
/* 1335 */                                       int low = dis.readInt();
/* 1336 */                                       int high = dis.readInt();
/* 1337 */                                       for (int i = low; i <= high; i++) {
/* 1338 */                                         int offset = dis.readInt();
/* 1339 */                                         d.print(", " + i + " => " + (instructionOffset + offset));
/*      */                                       }
/*      */                                     }
/*      */                                   };
/*      */                                 } else { Operand operand;
/* 1344 */                                   if (s.equals("lookupswitch")) {
/* 1345 */                                     operand = new Operand()
/*      */                                     {
/*      */ 
/*      */                                       public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                         throws IOException
/*      */                                       {
/*      */ 
/* 1352 */                                         int npads = 3 - instructionOffset % 4;
/* 1353 */                                         for (int i = 0; i < npads; i++) {
/* 1354 */                                           byte b = dis.readByte();
/* 1355 */                                           if (b != 0) d.print("Padding byte #" + i + " is " + (b & 0xFF));
/*      */                                         }
/* 1357 */                                         d.print("default => " + (instructionOffset + dis.readInt()));
/* 1358 */                                         int npairs = dis.readInt();
/* 1359 */                                         for (int i = 0; i < npairs; i++) {
/* 1360 */                                           int match = dis.readInt();
/* 1361 */                                           int offset = dis.readInt();
/* 1362 */                                           d.print(", " + match + " => " + (instructionOffset + offset));
/*      */                                         }
/*      */                                       }
/*      */                                     };
/*      */                                   } else { Operand operand;
/* 1367 */                                     if (s.equals("wide")) {
/* 1368 */                                       operand = new Operand()
/*      */                                       {
/*      */ 
/*      */                                         public void disasm(DataInputStream dis, int instructionOffset, Disassembler.LocalVariableTableAttribute localVariableTableAttribute, Disassembler d)
/*      */                                           throws IOException
/*      */                                         {
/*      */ 
/* 1375 */                                           int subopcode = 0xFF & dis.readByte();
/* 1376 */                                           Disassembler.Instruction wideInstruction = Disassembler.opcodeToWideInstruction[subopcode];
/* 1377 */                                           if (wideInstruction == null) {
/* 1378 */                                             throw new JaninoRuntimeException("Invalid opcode " + subopcode + " after opcode WIDE");
/*      */                                           }
/*      */                                           
/*      */ 
/*      */ 
/*      */ 
/* 1384 */                                           d.disasmInstruction(wideInstruction, dis, instructionOffset, localVariableTableAttribute);
/*      */ 
/*      */                                         }
/*      */                                         
/*      */ 
/*      */ 
/*      */                                       };
/*      */                                     }
/*      */                                     else
/*      */                                     {
/* 1394 */                                       throw new JaninoRuntimeException("Unknown operand \"" + s + "\""); } } } } } } } } } } } } } }
/*      */           Operand operand;
/* 1396 */           l.add(operand);
/*      */         }
/* 1398 */         operands = (Operand[])l.toArray(new Operand[l.size()]);
/*      */       }
/* 1400 */       opcodeToInstruction[opcode] = new Instruction(mnemonic, operands);
/*      */     }
/*      */   }
/*      */   
/*      */   private ConstantPoolInfo getConstantPoolEntry(short index) {
/* 1405 */     if ((index & 0xFFFF) < this.constantPool.length) {
/* 1406 */       return this.constantPool[(index & 0xFFFF)];
/*      */     }
/* 1408 */     return null;
/*      */   }
/*      */   
/*      */   private void printConstantPoolEntry(short index)
/*      */   {
/* 1413 */     if (this.verbose) print(index + " (");
/* 1414 */     if ((index & 0xFFFF) < this.constantPool.length) {
/* 1415 */       ConstantPoolInfo cpi = this.constantPool[index];
/* 1416 */       if (cpi == null) {
/* 1417 */         print("NULL CONSTANT POOL ENTRY");
/*      */       } else {
/* 1419 */         cpi.print();
/*      */       }
/*      */     } else {
/* 1422 */       print("CONSTANT POOL INDEX OUT OF RANGE");
/*      */     }
/* 1424 */     if (this.verbose) print(")");
/*      */   }
/*      */   
/*      */   private void printConstantPoolEntry(String label, short index) {
/* 1428 */     if (this.verbose) {
/* 1429 */       if (label != null) print(label + " = ");
/* 1430 */       print(index + " (");
/* 1431 */       printConstantPoolEntry(index);
/* 1432 */       print(")");
/*      */     } else {
/* 1434 */       if (label != null) {
/* 1435 */         if (label.endsWith("_index")) label = label.substring(0, label.length() - 6);
/* 1436 */         print(label + " = ");
/*      */       }
/* 1438 */       printConstantPoolEntry(index);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private static class Instruction
/*      */   {
/*      */     private final String mnemonic;
/*      */     private final Disassembler.Operand[] operands;
/*      */     
/*      */     public Instruction(String mnemonic, Disassembler.Operand[] operands)
/*      */     {
/* 1450 */       this.mnemonic = mnemonic;
/* 1451 */       this.operands = operands; }
/*      */     
/* 1453 */     public String getMnemonic() { return this.mnemonic; }
/* 1454 */     public Disassembler.Operand[] getOperands() { return this.operands; }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static abstract interface Operand
/*      */   {
/*      */     public abstract void disasm(DataInputStream paramDataInputStream, int paramInt, Disassembler.LocalVariableTableAttribute paramLocalVariableTableAttribute, Disassembler paramDisassembler)
/*      */       throws IOException;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static class IndentPrintWriter
/*      */     extends PrintWriter
/*      */   {
/*      */     public IndentPrintWriter(OutputStream os)
/*      */     {
/* 1472 */       super(); }
/*      */     
/* 1474 */     public void write(char[] cbuf, int off, int len) { handleIndentation();
/* 1475 */       super.write(cbuf, off, len);
/*      */     }
/*      */     
/* 1478 */     public void write(String str, int off, int len) { handleIndentation();
/* 1479 */       super.write(str, off, len); }
/*      */     
/* 1481 */     public void println() { super.println();this.atBOL = true; }
/* 1482 */     public void indent() { this.indentation += 1; }
/* 1483 */     public void unindent() { this.indentation -= 1; }
/*      */     
/*      */     private void handleIndentation() {
/* 1486 */       if (this.atBOL) {
/* 1487 */         for (int i = 0; i < this.indentation; i++) {
/* 1488 */           super.write(INDENTATION_CHARS, 0, INDENTATION_CHARS.length);
/*      */         }
/* 1490 */         this.atBOL = false;
/*      */       } }
/*      */     
/* 1493 */     private static final char[] INDENTATION_CHARS = { ' ', ' ' };
/*      */     
/* 1495 */     private boolean atBOL = true;
/* 1496 */     private int indentation = 0;
/*      */   }
/*      */   
/* 1499 */   private static abstract class ConstantPoolInfo { ConstantPoolInfo(Disassembler.1 x0) { this(); }
/*      */     
/* 1501 */     public int getSizeInConstantPool() { return 1; }
/*      */     public abstract void print();
/*      */     private ConstantPoolInfo() {} }
/* 1504 */   private class ConstantUtf8Info extends Disassembler.ConstantPoolInfo { public ConstantUtf8Info(String value) { super();
/* 1505 */       this.value = value;
/*      */     }
/*      */     
/* 1508 */     public void print() { Disassembler.this.print(this.value); }
/*      */     
/* 1510 */     public String getValue() { return this.value; }
/*      */     
/*      */     private String value; }
/*      */   
/* 1514 */   private ConstantPoolInfo[] constantPool = null;
/*      */   
/*      */   private static class CountingInputStream extends InputStream { private InputStream is;
/* 1517 */     public CountingInputStream(InputStream is) { this.is = is; }
/*      */     
/* 1519 */     public int read() throws IOException { int res = this.is.read();
/* 1520 */       if (res != -1) this.count += 1L;
/* 1521 */       return res;
/*      */     }
/*      */     
/* 1524 */     public int read(byte[] b, int off, int len) throws IOException { int res = super.read(b, off, len);
/* 1525 */       if (res != -1) this.count += res;
/* 1526 */       return res; }
/*      */     
/* 1528 */     public long getCount() { return this.count; }
/*      */     
/*      */ 
/* 1531 */     private long count = 0L;
/*      */   }
/*      */   
/*      */   private static String decodeAccess(short n) {
/* 1535 */     StringBuffer sb = new StringBuffer();
/* 1536 */     if ((n & 0x7) == 0) sb.append("package ");
/* 1537 */     if ((n & 0x7) == 1) { sb.append("public ");n = (short)(n & 0xFFFFFFF8); }
/* 1538 */     if ((n & 0x7) == 2) { sb.append("private ");n = (short)(n & 0xFFFFFFF8); }
/* 1539 */     if ((n & 0x7) == 4) { sb.append("protected ");n = (short)(n & 0xFFFFFFF8); }
/* 1540 */     if ((n & 0x8) != 0) { sb.append("static ");n = (short)(n & 0xFFFFFFF7); }
/* 1541 */     if ((n & 0x10) != 0) { sb.append("final ");n = (short)(n & 0xFFFFFFEF); }
/* 1542 */     if ((n & 0x20) != 0) { sb.append("super/synchronized ");n = (short)(n & 0xFFFFFFDF); }
/* 1543 */     if ((n & 0x40) != 0) { sb.append("volatile ");n = (short)(n & 0xFFFFFFBF); }
/* 1544 */     if ((n & 0x80) != 0) { sb.append("transient ");n = (short)(n & 0xFF7F); }
/* 1545 */     if ((n & 0x100) != 0) { sb.append("native ");n = (short)(n & 0xFEFF); }
/* 1546 */     if ((n & 0x200) != 0) { sb.append("interface ");n = (short)(n & 0xFDFF); }
/* 1547 */     if ((n & 0x400) != 0) { sb.append("abstract ");n = (short)(n & 0xFBFF); }
/* 1548 */     if ((n & 0x800) != 0) { sb.append("strict ");n = (short)(n & 0xF7FF); }
/* 1549 */     if (n != 0) sb.append("+ " + n + " ");
/* 1550 */     return sb.substring(0, sb.length() - 1);
/*      */   }
/*      */   
/*      */   private static String decodeDescriptor(String d) {
/* 1554 */     int brackets = 0;
/* 1555 */     while (d.charAt(0) == '[') {
/* 1556 */       brackets++;
/* 1557 */       d = d.substring(1);
/*      */     }
/* 1559 */     if (d.length() == 1) {
/* 1560 */       int idx = "BCDFIJSZ".indexOf(d.charAt(0));
/* 1561 */       if (idx != -1) d = PRIMITIVES[idx];
/*      */     }
/* 1563 */     else if ((d.charAt(0) == 'L') && (d.charAt(d.length() - 1) == ';')) {
/* 1564 */       d = d.substring(1, d.length() - 1);
/*      */     }
/* 1566 */     for (; brackets > 0; brackets--) d = d + "[]";
/* 1567 */     return d; }
/*      */   
/* 1569 */   private static final String[] PRIMITIVES = { "byte", "char", "double", "float", "int", "long", "short", "boolean" };
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\tools\Disassembler.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */