/*     */ package org.codehaus.janino.util;
/*     */ 
/*     */ import java.util.Arrays;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.NoSuchElementException;
/*     */ import org.codehaus.janino.JaninoRuntimeException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class MultiIterator
/*     */   implements Iterator
/*     */ {
/*  41 */   private static final Iterator AT_END = new Iterator() {
/*  42 */     public boolean hasNext() { return false; }
/*  43 */     public Object next() { throw new NoSuchElementException(); }
/*  44 */     public void remove() { throw new UnsupportedOperationException(); }
/*     */   };
/*     */   
/*     */   private final Iterator outer;
/*  48 */   private Iterator inner = AT_END;
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Iterator[] iterators)
/*     */   {
/*  54 */     this.outer = Arrays.asList(iterators).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Collection[] collections)
/*     */   {
/*  61 */     this.outer = Arrays.asList(collections).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Object[][] arrays)
/*     */   {
/*  68 */     this.outer = Arrays.asList(arrays).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Collection collection)
/*     */   {
/*  75 */     this.outer = collection.iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Iterator iterator)
/*     */   {
/*  82 */     this.outer = iterator;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Object[] array)
/*     */   {
/*  89 */     this.outer = Arrays.asList(array).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MultiIterator(Object object, Collection collection)
/*     */   {
/*  96 */     this.outer = Arrays.asList(new Object[] { { object }, collection }).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public MultiIterator(Collection collection, Object object)
/*     */   {
/* 106 */     this.outer = Arrays.asList(new Object[] { collection, { object } }).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public MultiIterator(Object prefix, Iterator iterator)
/*     */   {
/* 116 */     this.outer = Arrays.asList(new Object[] { { prefix }, iterator }).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public MultiIterator(Iterator iterator, Object suffix)
/*     */   {
/* 126 */     this.outer = Arrays.asList(new Object[] { iterator, { suffix } }).iterator();
/*     */   }
/*     */   
/*     */ 
/*     */   public boolean hasNext()
/*     */   {
/*     */     for (;;)
/*     */     {
/* 134 */       if (this.inner.hasNext()) return true;
/* 135 */       if (!this.outer.hasNext()) return false;
/* 136 */       Object o = this.outer.next();
/* 137 */       if ((o instanceof Iterator)) {
/* 138 */         this.inner = ((Iterator)o);
/*     */       }
/* 140 */       else if ((o instanceof Collection)) {
/* 141 */         this.inner = ((Collection)o).iterator();
/*     */       }
/* 143 */       else if ((o instanceof Object[])) {
/* 144 */         this.inner = Arrays.asList((Object[])o).iterator();
/*     */       }
/*     */       else {
/* 147 */         throw new JaninoRuntimeException("Unexpected element type \"" + o.getClass().getName() + "\"");
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public Object next() {
/* 153 */     if (hasNext()) return this.inner.next();
/* 154 */     throw new NoSuchElementException();
/*     */   }
/*     */   
/*     */   public void remove() {
/* 158 */     this.inner.remove();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\util\MultiIterator.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */