/*     */ package org.hibernate.annotations.common.annotationfactory;
/*     */ 
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.reflect.InvocationHandler;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.Comparator;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.SortedSet;
/*     */ import java.util.TreeSet;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AnnotationProxy
/*     */   implements Annotation, InvocationHandler
/*     */ {
/*     */   private final Class<? extends Annotation> annotationType;
/*     */   private final Map<Method, Object> values;
/*     */   
/*     */   public AnnotationProxy(AnnotationDescriptor descriptor)
/*     */   {
/*  45 */     this.annotationType = descriptor.type();
/*  46 */     this.values = getAnnotationValues(descriptor);
/*     */   }
/*     */   
/*     */   private Map<Method, Object> getAnnotationValues(AnnotationDescriptor descriptor) {
/*  50 */     Map<Method, Object> result = new HashMap();
/*  51 */     int processedValuesFromDescriptor = 0;
/*  52 */     for (Method m : this.annotationType.getDeclaredMethods()) {
/*  53 */       if (descriptor.containsElement(m.getName())) {
/*  54 */         result.put(m, descriptor.valueOf(m.getName()));
/*  55 */         processedValuesFromDescriptor++;
/*     */       }
/*  57 */       else if (m.getDefaultValue() != null) {
/*  58 */         result.put(m, m.getDefaultValue());
/*     */       }
/*     */       else {
/*  61 */         throw new IllegalArgumentException("No value provided for " + m.getName());
/*     */       }
/*     */     }
/*  64 */     if (processedValuesFromDescriptor != descriptor.numberOfElements()) {
/*  65 */       throw new RuntimeException("Trying to instanciate " + this.annotationType + " with unknown elements");
/*     */     }
/*  67 */     return result;
/*     */   }
/*     */   
/*     */   public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
/*  71 */     if (this.values.containsKey(method)) {
/*  72 */       return this.values.get(method);
/*     */     }
/*  74 */     return method.invoke(this, args);
/*     */   }
/*     */   
/*     */   public Class<? extends Annotation> annotationType() {
/*  78 */     return this.annotationType;
/*     */   }
/*     */   
/*     */   public String toString() {
/*  82 */     StringBuilder result = new StringBuilder();
/*  83 */     result.append('@').append(annotationType().getName()).append('(');
/*  84 */     for (Method m : getRegisteredMethodsInAlphabeticalOrder()) {
/*  85 */       result.append(m.getName()).append('=').append(this.values.get(m)).append(", ");
/*     */     }
/*     */     
/*  88 */     if (this.values.size() > 0) {
/*  89 */       result.delete(result.length() - 2, result.length());
/*  90 */       result.append(")");
/*     */     }
/*     */     else {
/*  93 */       result.delete(result.length() - 1, result.length());
/*     */     }
/*     */     
/*  96 */     return result.toString();
/*     */   }
/*     */   
/*     */   private SortedSet<Method> getRegisteredMethodsInAlphabeticalOrder() {
/* 100 */     SortedSet<Method> result = new TreeSet(new Comparator()
/*     */     {
/*     */       public int compare(Method o1, Method o2) {
/* 103 */         return o1.getName().compareTo(o2.getName());
/*     */       }
/*     */       
/*     */ 
/* 107 */     });
/* 108 */     result.addAll(this.values.keySet());
/* 109 */     return result;
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/annotationfactory/AnnotationProxy.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */