/*     */ package org.hibernate.annotations.common.reflection.java.generics;
/*     */ 
/*     */ import java.lang.reflect.GenericArrayType;
/*     */ import java.lang.reflect.ParameterizedType;
/*     */ import java.lang.reflect.Type;
/*     */ import java.lang.reflect.TypeVariable;
/*     */ import java.lang.reflect.WildcardType;
/*     */ import java.util.Collection;
/*     */ import java.util.Map;
/*     */ import java.util.SortedSet;
/*     */ 
/*     */ public class TypeUtils
/*     */ {
/*     */   public static boolean isResolved(Type t)
/*     */   {
/*  16 */     ((Boolean)new TypeSwitch()
/*     */     {
/*     */       public Boolean caseClass(Class classType) {
/*  19 */         return Boolean.valueOf(true);
/*     */       }
/*     */       
/*     */       public Boolean caseGenericArrayType(GenericArrayType genericArrayType)
/*     */       {
/*  24 */         return Boolean.valueOf(TypeUtils.isResolved(genericArrayType.getGenericComponentType()));
/*     */       }
/*     */       
/*     */       public Boolean caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/*  29 */         Type[] typeArgs = parameterizedType.getActualTypeArguments();
/*  30 */         for (Type arg : typeArgs) {
/*  31 */           if (!TypeUtils.isResolved(arg)) {
/*  32 */             return Boolean.valueOf(false);
/*     */           }
/*     */         }
/*  35 */         return Boolean.valueOf(TypeUtils.isResolved(parameterizedType.getRawType()));
/*     */       }
/*     */       
/*     */       public Boolean caseTypeVariable(TypeVariable typeVariable)
/*     */       {
/*  40 */         return Boolean.valueOf(false);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*  45 */       public Boolean caseWildcardType(WildcardType wildcardType) { return Boolean.valueOf((TypeUtils.areResolved(wildcardType.getUpperBounds()).booleanValue()) && (TypeUtils.areResolved(wildcardType.getLowerBounds()).booleanValue())); } }.doSwitch(t)).booleanValue();
/*     */   }
/*     */   
/*     */ 
/*     */   private static Boolean areResolved(Type[] types)
/*     */   {
/*  51 */     for (Type t : types) {
/*  52 */       if (!isResolved(t)) {
/*  53 */         return Boolean.valueOf(false);
/*     */       }
/*     */     }
/*  56 */     return Boolean.valueOf(true);
/*     */   }
/*     */   
/*     */   public static Class<? extends Collection> getCollectionClass(Type type) {
/*  60 */     (Class)new TypeSwitch()
/*     */     {
/*     */       public Class<? extends Collection> caseClass(Class clazz) {
/*  63 */         return TypeUtils.isCollectionClass(clazz) ? clazz : null;
/*     */       }
/*     */       
/*     */       public Class<? extends Collection> caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/*  68 */         return TypeUtils.getCollectionClass((Class)parameterizedType.getRawType());
/*     */       }
/*     */       
/*     */       public Class<? extends Collection> caseWildcardType(WildcardType wildcardType)
/*     */       {
/*  73 */         Type[] upperBounds = wildcardType.getUpperBounds();
/*  74 */         if (upperBounds.length == 0) {
/*  75 */           return null;
/*     */         }
/*  77 */         return TypeUtils.getCollectionClass(upperBounds[0]);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*  82 */       public Class<? extends Collection> defaultCase(Type t) { return null; } }.doSwitch(type);
/*     */   }
/*     */   
/*     */ 
/*     */   private static boolean isCollectionClass(Class<?> clazz)
/*     */   {
/*  88 */     return (clazz == Collection.class) || (clazz == java.util.List.class) || (clazz == java.util.Set.class) || (clazz == Map.class) || (clazz == SortedSet.class) || (clazz == java.util.SortedMap.class);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static boolean isSimple(Type type)
/*     */   {
/*  97 */     ((Boolean)new TypeSwitch()
/*     */     {
/*     */       public Boolean caseClass(Class clazz) {
/* 100 */         return Boolean.valueOf((!clazz.isArray()) && (!TypeUtils.isCollectionClass(clazz)));
/*     */       }
/*     */       
/*     */ 
/*     */       public Boolean caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/* 106 */         for (Type actualTypeArgument : parameterizedType.getActualTypeArguments()) {
/* 107 */           if (!TypeUtils.isSimple(actualTypeArgument)) {
/* 108 */             return Boolean.valueOf(false);
/*     */           }
/*     */         }
/* 111 */         return Boolean.valueOf(true);
/*     */       }
/*     */       
/*     */       public Boolean caseWildcardType(WildcardType wildcardType)
/*     */       {
/* 116 */         return Boolean.valueOf((TypeUtils.areSimple(wildcardType.getUpperBounds()).booleanValue()) && (TypeUtils.areSimple(wildcardType.getLowerBounds()).booleanValue()));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 121 */       public Boolean defaultCase(Type t) { return Boolean.valueOf(false); } }.doSwitch(type)).booleanValue();
/*     */   }
/*     */   
/*     */ 
/*     */   private static Boolean areSimple(Type[] types)
/*     */   {
/* 127 */     for (Type t : types) {
/* 128 */       if (!isSimple(t)) {
/* 129 */         return Boolean.valueOf(false);
/*     */       }
/*     */     }
/* 132 */     return Boolean.valueOf(true);
/*     */   }
/*     */   
/*     */   public static boolean isVoid(Type type) {
/* 136 */     return Void.TYPE.equals(type);
/*     */   }
/*     */   
/*     */   public static boolean isArray(Type t) {
/* 140 */     ((Boolean)new TypeSwitch()
/*     */     {
/*     */       public Boolean caseClass(Class clazz) {
/* 143 */         return Boolean.valueOf(clazz.isArray());
/*     */       }
/*     */       
/*     */       public Boolean caseGenericArrayType(GenericArrayType genericArrayType)
/*     */       {
/* 148 */         return Boolean.valueOf(TypeUtils.isSimple(genericArrayType.getGenericComponentType()));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 153 */       public Boolean defaultCase(Type type) { return Boolean.valueOf(false); } }.doSwitch(t)).booleanValue();
/*     */   }
/*     */   
/*     */ 
/*     */   public static boolean isCollection(Type t)
/*     */   {
/* 159 */     return getCollectionClass(t) != null;
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/reflection/java/generics/TypeUtils.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */