/*     */ package org.xerial.snappy;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.net.URL;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.nio.charset.Charset;
/*     */ import java.util.Properties;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Snappy
/*     */ {
/*     */   private static Object impl;
/*     */   
/*     */   static
/*     */   {
/*     */     try
/*     */     {
/*  44 */       impl = SnappyLoader.load();
/*     */     }
/*     */     catch (Exception e) {
/*  47 */       e.printStackTrace();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static void arrayCopy(Object src, int offset, int byteLength, Object dest, int dest_offset)
/*     */     throws IOException
/*     */   {
/*  73 */     ((SnappyNativeAPI)impl).arrayCopy(src, offset, byteLength, dest, dest_offset);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(byte[] input)
/*     */     throws IOException
/*     */   {
/*  88 */     return rawCompress(input, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int compress(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset)
/*     */     throws IOException
/*     */   {
/* 106 */     return rawCompress(input, inputOffset, inputLength, output, outputOffset);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int compress(ByteBuffer uncompressed, ByteBuffer compressed)
/*     */     throws IOException
/*     */   {
/* 125 */     if (!uncompressed.isDirect())
/* 126 */       throw new SnappyError(SnappyErrorCode.NOT_A_DIRECT_BUFFER, "input is not a direct buffer");
/* 127 */     if (!compressed.isDirect()) {
/* 128 */       throw new SnappyError(SnappyErrorCode.NOT_A_DIRECT_BUFFER, "destination is not a direct buffer");
/*     */     }
/*     */     
/*     */ 
/* 132 */     int uPos = uncompressed.position();
/* 133 */     int uLen = uncompressed.remaining();
/* 134 */     int compressedSize = ((SnappyNativeAPI)impl).rawCompress(uncompressed, uPos, uLen, compressed, compressed.position());
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 139 */     compressed.limit(compressed.position() + compressedSize);
/*     */     
/* 141 */     return compressedSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(char[] input)
/*     */   {
/* 151 */     return rawCompress(input, input.length * 2);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(double[] input)
/*     */   {
/* 161 */     return rawCompress(input, input.length * 8);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(float[] input)
/*     */   {
/* 171 */     return rawCompress(input, input.length * 4);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(int[] input)
/*     */   {
/* 181 */     return rawCompress(input, input.length * 4);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(long[] input)
/*     */   {
/* 191 */     return rawCompress(input, input.length * 8);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(short[] input)
/*     */   {
/* 201 */     return rawCompress(input, input.length * 2);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(String s)
/*     */     throws IOException
/*     */   {
/*     */     try
/*     */     {
/* 213 */       return compress(s, "UTF-8");
/*     */     }
/*     */     catch (UnsupportedEncodingException e) {
/* 216 */       throw new IllegalStateException("UTF-8 encoder is not found");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(String s, String encoding)
/*     */     throws UnsupportedEncodingException, IOException
/*     */   {
/* 230 */     byte[] data = s.getBytes(encoding);
/* 231 */     return compress(data);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] compress(String s, Charset encoding)
/*     */     throws IOException
/*     */   {
/* 244 */     byte[] data = s.getBytes(encoding);
/* 245 */     return compress(data);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String getNativeLibraryVersion()
/*     */   {
/* 255 */     URL versionFile = SnappyLoader.class.getResource("/org/xerial/snappy/VERSION");
/*     */     
/* 257 */     String version = "unknown";
/*     */     try {
/* 259 */       if (versionFile != null) {
/* 260 */         Properties versionData = new Properties();
/* 261 */         versionData.load(versionFile.openStream());
/* 262 */         version = versionData.getProperty("version", version);
/* 263 */         if (version.equals("unknown"))
/* 264 */           version = versionData.getProperty("VERSION", version);
/* 265 */         version = version.trim().replaceAll("[^0-9\\.]", "");
/*     */       }
/*     */     }
/*     */     catch (IOException e) {
/* 269 */       e.printStackTrace();
/*     */     }
/* 271 */     return version;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static boolean isValidCompressedBuffer(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 281 */     if (input == null)
/* 282 */       throw new NullPointerException("input is null");
/* 283 */     return ((SnappyNativeAPI)impl).isValidCompressedBuffer(input, offset, length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static boolean isValidCompressedBuffer(byte[] input)
/*     */     throws IOException
/*     */   {
/* 293 */     return isValidCompressedBuffer(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static boolean isValidCompressedBuffer(ByteBuffer compressed)
/*     */     throws IOException
/*     */   {
/* 303 */     return ((SnappyNativeAPI)impl).isValidCompressedBuffer(compressed, compressed.position(), compressed.remaining());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int maxCompressedLength(int byteSize)
/*     */   {
/* 316 */     return ((SnappyNativeAPI)impl).maxCompressedLength(byteSize);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] rawCompress(Object data, int byteSize)
/*     */   {
/* 329 */     byte[] buf = new byte[maxCompressedLength(byteSize)];
/* 330 */     int compressedByteSize = ((SnappyNativeAPI)impl).rawCompress(data, 0, byteSize, buf, 0);
/* 331 */     byte[] result = new byte[compressedByteSize];
/* 332 */     System.arraycopy(buf, 0, result, 0, compressedByteSize);
/* 333 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int rawCompress(Object input, int inputOffset, int inputLength, byte[] output, int outputOffset)
/*     */     throws IOException
/*     */   {
/* 355 */     if ((input == null) || (output == null)) {
/* 356 */       throw new NullPointerException("input or output is null");
/*     */     }
/* 358 */     int compressedSize = ((SnappyNativeAPI)impl).rawCompress(input, inputOffset, inputLength, output, outputOffset);
/*     */     
/* 360 */     return compressedSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int rawUncompress(byte[] input, int inputOffset, int inputLength, Object output, int outputOffset)
/*     */     throws IOException
/*     */   {
/* 389 */     if ((input == null) || (output == null))
/* 390 */       throw new NullPointerException("input or output is null");
/* 391 */     return ((SnappyNativeAPI)impl).rawUncompress(input, inputOffset, inputLength, output, outputOffset);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] uncompress(byte[] input)
/*     */     throws IOException
/*     */   {
/* 402 */     byte[] result = new byte[uncompressedLength(input)];
/* 403 */     int byteSize = uncompress(input, 0, input.length, result, 0);
/* 404 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int uncompress(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset)
/*     */     throws IOException
/*     */   {
/* 427 */     return rawUncompress(input, inputOffset, inputLength, output, outputOffset);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int uncompress(ByteBuffer compressed, ByteBuffer uncompressed)
/*     */     throws IOException
/*     */   {
/* 453 */     if (!compressed.isDirect())
/* 454 */       throw new SnappyError(SnappyErrorCode.NOT_A_DIRECT_BUFFER, "input is not a direct buffer");
/* 455 */     if (!uncompressed.isDirect()) {
/* 456 */       throw new SnappyError(SnappyErrorCode.NOT_A_DIRECT_BUFFER, "destination is not a direct buffer");
/*     */     }
/* 458 */     int cPos = compressed.position();
/* 459 */     int cLen = compressed.remaining();
/*     */     
/*     */ 
/*     */ 
/* 463 */     int decompressedSize = ((SnappyNativeAPI)impl).rawUncompress(compressed, cPos, cLen, uncompressed, uncompressed.position());
/*     */     
/* 465 */     uncompressed.limit(uncompressed.position() + decompressedSize);
/*     */     
/* 467 */     return decompressedSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static char[] uncompressCharArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 478 */     return uncompressCharArray(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static char[] uncompressCharArray(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 491 */     int uncompressedLength = uncompressedLength(input, offset, length);
/* 492 */     char[] result = new char[uncompressedLength / 2];
/* 493 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, offset, length, result, 0);
/* 494 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static double[] uncompressDoubleArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 505 */     int uncompressedLength = uncompressedLength(input, 0, input.length);
/* 506 */     double[] result = new double[uncompressedLength / 8];
/* 507 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, 0, input.length, result, 0);
/* 508 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int uncompressedLength(byte[] input)
/*     */     throws IOException
/*     */   {
/* 522 */     return ((SnappyNativeAPI)impl).uncompressedLength(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int uncompressedLength(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 538 */     if (input == null) {
/* 539 */       throw new NullPointerException("input is null");
/*     */     }
/* 541 */     return ((SnappyNativeAPI)impl).uncompressedLength(input, offset, length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int uncompressedLength(ByteBuffer compressed)
/*     */     throws IOException
/*     */   {
/* 558 */     if (!compressed.isDirect()) {
/* 559 */       throw new SnappyError(SnappyErrorCode.NOT_A_DIRECT_BUFFER, "input is not a direct buffer");
/*     */     }
/* 561 */     return ((SnappyNativeAPI)impl).uncompressedLength(compressed, compressed.position(), compressed.remaining());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static float[] uncompressFloatArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 572 */     return uncompressFloatArray(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static float[] uncompressFloatArray(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 585 */     int uncompressedLength = uncompressedLength(input, offset, length);
/* 586 */     float[] result = new float[uncompressedLength / 4];
/* 587 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, offset, length, result, 0);
/* 588 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int[] uncompressIntArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 599 */     return uncompressIntArray(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int[] uncompressIntArray(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 612 */     int uncompressedLength = uncompressedLength(input, offset, length);
/* 613 */     int[] result = new int[uncompressedLength / 4];
/* 614 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, offset, length, result, 0);
/* 615 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static long[] uncompressLongArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 626 */     return uncompressLongArray(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static long[] uncompressLongArray(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 639 */     int uncompressedLength = uncompressedLength(input, offset, length);
/* 640 */     long[] result = new long[uncompressedLength / 8];
/* 641 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, offset, length, result, 0);
/* 642 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static short[] uncompressShortArray(byte[] input)
/*     */     throws IOException
/*     */   {
/* 653 */     return uncompressShortArray(input, 0, input.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static short[] uncompressShortArray(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/* 666 */     int uncompressedLength = uncompressedLength(input, offset, length);
/* 667 */     short[] result = new short[uncompressedLength / 2];
/* 668 */     int byteSize = ((SnappyNativeAPI)impl).rawUncompress(input, offset, length, result, 0);
/* 669 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input)
/*     */     throws IOException
/*     */   {
/*     */     try
/*     */     {
/* 681 */       return uncompressString(input, "UTF-8");
/*     */     }
/*     */     catch (UnsupportedEncodingException e) {
/* 684 */       throw new IllegalStateException("UTF-8 decoder is not found");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input, int offset, int length)
/*     */     throws IOException
/*     */   {
/*     */     try
/*     */     {
/* 699 */       return uncompressString(input, offset, length, "UTF-8");
/*     */     }
/*     */     catch (UnsupportedEncodingException e) {
/* 702 */       throw new IllegalStateException("UTF-8 decoder is not found");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input, int offset, int length, String encoding)
/*     */     throws IOException, UnsupportedEncodingException
/*     */   {
/* 719 */     byte[] uncompressed = new byte[uncompressedLength(input, offset, length)];
/* 720 */     int compressedSize = uncompress(input, offset, length, uncompressed, 0);
/* 721 */     return new String(uncompressed, encoding);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input, int offset, int length, Charset encoding)
/*     */     throws IOException, UnsupportedEncodingException
/*     */   {
/* 737 */     byte[] uncompressed = new byte[uncompressedLength(input, offset, length)];
/* 738 */     int compressedSize = uncompress(input, offset, length, uncompressed, 0);
/* 739 */     return new String(uncompressed, encoding);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input, String encoding)
/*     */     throws IOException, UnsupportedEncodingException
/*     */   {
/* 753 */     byte[] uncompressed = uncompress(input);
/* 754 */     return new String(uncompressed, encoding);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String uncompressString(byte[] input, Charset encoding)
/*     */     throws IOException, UnsupportedEncodingException
/*     */   {
/* 767 */     byte[] uncompressed = uncompress(input);
/* 768 */     return new String(uncompressed, encoding);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\xerial\snappy\Snappy.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */