package prefuse.util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.awt.geom.RoundRectangle2D;
import prefuse.visual.VisualItem;

public class GraphicsLib
{
  public static final int NO_INTERSECTION = 0;
  public static final int COINCIDENT = -1;
  public static final int PARALLEL = -2;
  
  public static int intersectLineLine(Line2D paramLine2D1, Line2D paramLine2D2, Point2D paramPoint2D)
  {
    double d1 = paramLine2D1.getX1();
    double d2 = paramLine2D1.getY1();
    double d3 = paramLine2D1.getX2();
    double d4 = paramLine2D1.getY2();
    double d5 = paramLine2D2.getX1();
    double d6 = paramLine2D2.getY1();
    double d7 = paramLine2D2.getX2();
    double d8 = paramLine2D2.getY2();
    return intersectLineLine(d1, d2, d3, d4, d5, d6, d7, d8, paramPoint2D);
  }
  
  public static int intersectLineLine(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4, double paramDouble5, double paramDouble6, double paramDouble7, double paramDouble8, Point2D paramPoint2D)
  {
    double d1 = (paramDouble7 - paramDouble5) * (paramDouble2 - paramDouble6) - (paramDouble8 - paramDouble6) * (paramDouble1 - paramDouble5);
    double d2 = (paramDouble3 - paramDouble1) * (paramDouble2 - paramDouble6) - (paramDouble4 - paramDouble2) * (paramDouble1 - paramDouble5);
    double d3 = (paramDouble8 - paramDouble6) * (paramDouble3 - paramDouble1) - (paramDouble7 - paramDouble5) * (paramDouble4 - paramDouble2);
    if (d3 != 0.0D)
    {
      double d4 = d1 / d3;
      double d5 = d2 / d3;
      if ((0.0D <= d4) && (d4 <= 1.0D) && (0.0D <= d5) && (d5 <= 1.0D))
      {
        paramPoint2D.setLocation(paramDouble1 + d4 * (paramDouble3 - paramDouble1), paramDouble2 + d4 * (paramDouble4 - paramDouble2));
        return 1;
      }
      return 0;
    }
    return (d1 == 0.0D) || (d2 == 0.0D) ? -1 : -2;
  }
  
  public static int intersectLineRectangle(Point2D paramPoint2D1, Point2D paramPoint2D2, Rectangle2D paramRectangle2D, Point2D[] paramArrayOfPoint2D)
  {
    double d1 = paramPoint2D1.getX();
    double d2 = paramPoint2D1.getY();
    double d3 = paramPoint2D2.getX();
    double d4 = paramPoint2D2.getY();
    double d5 = paramRectangle2D.getMaxX();
    double d6 = paramRectangle2D.getMaxY();
    double d7 = paramRectangle2D.getMinX();
    double d8 = paramRectangle2D.getMinY();
    if (paramArrayOfPoint2D[0] == null) {
      paramArrayOfPoint2D[0] = new Point2D.Double();
    }
    if (paramArrayOfPoint2D[1] == null) {
      paramArrayOfPoint2D[1] = new Point2D.Double();
    }
    int i = 0;
    if (intersectLineLine(d7, d8, d5, d8, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (intersectLineLine(d5, d8, d5, d6, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (i == 2) {
      return i;
    }
    if (intersectLineLine(d5, d6, d7, d6, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (i == 2) {
      return i;
    }
    if (intersectLineLine(d7, d6, d7, d8, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    return i;
  }
  
  public static int intersectLineRectangle(Line2D paramLine2D, Rectangle2D paramRectangle2D, Point2D[] paramArrayOfPoint2D)
  {
    double d1 = paramLine2D.getX1();
    double d2 = paramLine2D.getY1();
    double d3 = paramLine2D.getX2();
    double d4 = paramLine2D.getY2();
    double d5 = paramRectangle2D.getMaxX();
    double d6 = paramRectangle2D.getMaxY();
    double d7 = paramRectangle2D.getMinX();
    double d8 = paramRectangle2D.getMinY();
    if (paramArrayOfPoint2D[0] == null) {
      paramArrayOfPoint2D[0] = new Point2D.Double();
    }
    if (paramArrayOfPoint2D[1] == null) {
      paramArrayOfPoint2D[1] = new Point2D.Double();
    }
    int i = 0;
    if (intersectLineLine(d7, d8, d5, d8, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (intersectLineLine(d5, d8, d5, d6, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (i == 2) {
      return i;
    }
    if (intersectLineLine(d5, d6, d7, d6, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    if (i == 2) {
      return i;
    }
    if (intersectLineLine(d7, d6, d7, d8, d1, d2, d3, d4, paramArrayOfPoint2D[i]) > 0) {
      i++;
    }
    return i;
  }
  
  public static double[] convexHull(double[] paramArrayOfDouble, int paramInt)
  {
    if (paramInt < 6) {
      throw new IllegalArgumentException("Input must have at least 3 points");
    }
    int i = paramInt / 2 - 1;
    float[] arrayOfFloat = new float[i];
    int[] arrayOfInt1 = new int[i];
    int[] arrayOfInt2 = new int[paramInt / 2];
    return convexHull(paramArrayOfDouble, paramInt, arrayOfFloat, arrayOfInt1, arrayOfInt2);
  }
  
  public static double[] convexHull(double[] paramArrayOfDouble, int paramInt, float[] paramArrayOfFloat, int[] paramArrayOfInt1, int[] paramArrayOfInt2)
  {
    int i = paramInt / 2 - 1;
    if (paramInt < 6) {
      throw new IllegalArgumentException("Input must have at least 3 points");
    }
    if ((paramArrayOfFloat.length < i) || (paramArrayOfInt1.length < i) || (paramArrayOfInt2.length < paramInt / 2)) {
      throw new IllegalArgumentException("Pre-allocated data structure too small");
    }
    int j = 0;
    for (int k = 2; k < paramInt; k += 2) {
      if (paramArrayOfDouble[(k + 1)] < paramArrayOfDouble[(j + 1)]) {
        j = k;
      } else if (paramArrayOfDouble[(k + 1)] == paramArrayOfDouble[(j + 1)]) {
        j = paramArrayOfDouble[k] < paramArrayOfDouble[j] ? k : j;
      }
    }
    k = 0;
    int m = 0;
    while (k < paramInt)
    {
      if (k != j)
      {
        paramArrayOfFloat[m] = ((float)Math.atan2(paramArrayOfDouble[(k + 1)] - paramArrayOfDouble[(j + 1)], paramArrayOfDouble[k] - paramArrayOfDouble[j]));
        paramArrayOfInt1[m] = k;
        m++;
      }
      k += 2;
    }
    ArrayLib.sort(paramArrayOfFloat, paramArrayOfInt1, i);
    float f = paramArrayOfFloat[0];
    m = 0;
    for (int n = 1; n < i; n++) {
      if (f == paramArrayOfFloat[n])
      {
        double d1 = Math.sqrt(paramArrayOfDouble[n] * paramArrayOfDouble[n] + paramArrayOfDouble[(n + 1)] * paramArrayOfDouble[(n + 1)]);
        double d2 = Math.sqrt(paramArrayOfDouble[m] * paramArrayOfDouble[m] + paramArrayOfDouble[(m + 1)] * paramArrayOfDouble[(m + 1)]);
        if (d1 >= d2)
        {
          paramArrayOfInt1[n] = -1;
        }
        else
        {
          paramArrayOfInt1[m] = -1;
          f = paramArrayOfFloat[n];
          m = n;
        }
      }
      else
      {
        f = paramArrayOfFloat[n];
        m = n;
      }
    }
    n = 0;
    paramArrayOfInt2[(n++)] = j;
    int i1 = 0;
    int i2 = 0;
    while (i2 < 2)
    {
      if (paramArrayOfInt1[i1] != -1)
      {
        paramArrayOfInt2[(n++)] = paramArrayOfInt1[i1];
        i2++;
      }
      i1++;
    }
    while (i1 < i)
    {
      if (paramArrayOfInt1[i1] != -1)
      {
        while (isNonLeft(j, paramArrayOfInt2[(n - 2)], paramArrayOfInt2[(n - 1)], paramArrayOfInt1[i1], paramArrayOfDouble)) {
          n--;
        }
        paramArrayOfInt2[(n++)] = paramArrayOfInt1[i1];
      }
      i1++;
    }
    double[] arrayOfDouble = new double[2 * n];
    for (int i3 = 0; i3 < n; i3++)
    {
      arrayOfDouble[(2 * i3)] = paramArrayOfDouble[paramArrayOfInt2[i3]];
      arrayOfDouble[(2 * i3 + 1)] = paramArrayOfDouble[(paramArrayOfInt2[i3] + 1)];
    }
    return arrayOfDouble;
  }
  
  private static boolean isNonLeft(int paramInt1, int paramInt2, int paramInt3, int paramInt4, double[] paramArrayOfDouble)
  {
    double d1 = Math.sqrt(Math.pow(paramArrayOfDouble[(paramInt3 + 1)] - paramArrayOfDouble[(paramInt2 + 1)], 2.0D) + Math.pow(paramArrayOfDouble[paramInt3] - paramArrayOfDouble[paramInt2], 2.0D));
    double d2 = Math.sqrt(Math.pow(paramArrayOfDouble[(paramInt4 + 1)] - paramArrayOfDouble[(paramInt3 + 1)], 2.0D) + Math.pow(paramArrayOfDouble[paramInt4] - paramArrayOfDouble[paramInt3], 2.0D));
    double d3 = Math.sqrt(Math.pow(paramArrayOfDouble[(paramInt4 + 1)] - paramArrayOfDouble[(paramInt1 + 1)], 2.0D) + Math.pow(paramArrayOfDouble[paramInt4] - paramArrayOfDouble[paramInt1], 2.0D));
    double d4 = Math.sqrt(Math.pow(paramArrayOfDouble[(paramInt2 + 1)] - paramArrayOfDouble[(paramInt1 + 1)], 2.0D) + Math.pow(paramArrayOfDouble[paramInt2] - paramArrayOfDouble[paramInt1], 2.0D));
    double d5 = Math.sqrt(Math.pow(paramArrayOfDouble[(paramInt3 + 1)] - paramArrayOfDouble[(paramInt1 + 1)], 2.0D) + Math.pow(paramArrayOfDouble[paramInt3] - paramArrayOfDouble[paramInt1], 2.0D));
    double d6 = Math.acos((d2 * d2 + d5 * d5 - d3 * d3) / (2.0D * d2 * d5));
    double d7 = Math.acos((d5 * d5 + d1 * d1 - d4 * d4) / (2.0D * d5 * d1));
    double d8 = 3.141592653589793D - d6 - d7;
    return d8 <= 0.0D;
  }
  
  public static float[] centroid(float[] paramArrayOfFloat, int paramInt)
  {
    float[] arrayOfFloat = { 0.0F, 0.0F };
    for (int i = 0; i < paramInt; i += 2)
    {
      arrayOfFloat[0] += paramArrayOfFloat[i];
      arrayOfFloat[1] += paramArrayOfFloat[(i + 1)];
    }
    arrayOfFloat[0] /= paramInt / 2;
    arrayOfFloat[1] /= paramInt / 2;
    return arrayOfFloat;
  }
  
  public static void growPolygon(float[] paramArrayOfFloat, int paramInt, float paramFloat)
  {
    float[] arrayOfFloat = centroid(paramArrayOfFloat, paramInt);
    for (int i = 0; i < paramInt; i += 2)
    {
      float f1 = paramArrayOfFloat[i] - arrayOfFloat[0];
      float f2 = paramArrayOfFloat[(i + 1)] - arrayOfFloat[1];
      float f3 = (float)Math.sqrt(f1 * f1 + f2 * f2);
      paramArrayOfFloat[i] += paramFloat * f1 / f3;
      paramArrayOfFloat[(i + 1)] += paramFloat * f2 / f3;
    }
  }
  
  public static GeneralPath cardinalSpline(float[] paramArrayOfFloat, float paramFloat, boolean paramBoolean)
  {
    GeneralPath localGeneralPath = new GeneralPath();
    localGeneralPath.moveTo(paramArrayOfFloat[0], paramArrayOfFloat[1]);
    return cardinalSpline(localGeneralPath, paramArrayOfFloat, paramFloat, paramBoolean, 0.0F, 0.0F);
  }
  
  public static GeneralPath cardinalSpline(float[] paramArrayOfFloat, int paramInt1, int paramInt2, float paramFloat, boolean paramBoolean)
  {
    GeneralPath localGeneralPath = new GeneralPath();
    localGeneralPath.moveTo(paramArrayOfFloat[paramInt1], paramArrayOfFloat[(paramInt1 + 1)]);
    return cardinalSpline(localGeneralPath, paramArrayOfFloat, paramInt1, paramInt2, paramFloat, paramBoolean, 0.0F, 0.0F);
  }
  
  public static GeneralPath cardinalSpline(GeneralPath paramGeneralPath, float[] paramArrayOfFloat, float paramFloat1, boolean paramBoolean, float paramFloat2, float paramFloat3)
  {
    for (int i = 0; (i < paramArrayOfFloat.length) && (!Float.isNaN(paramArrayOfFloat[i])); i++) {}
    return cardinalSpline(paramGeneralPath, paramArrayOfFloat, 0, i / 2, paramFloat1, paramBoolean, paramFloat2, paramFloat3);
  }
  
  public static GeneralPath cardinalSpline(GeneralPath paramGeneralPath, float[] paramArrayOfFloat, int paramInt1, int paramInt2, float paramFloat1, boolean paramBoolean, float paramFloat2, float paramFloat3)
  {
    int i = 2 * paramInt2;
    int j = paramInt1 + i;
    if (i < 6) {
      throw new IllegalArgumentException("To create spline requires at least 3 points");
    }
    float f3;
    float f4;
    if (paramBoolean)
    {
      f3 = paramArrayOfFloat[(paramInt1 + 2)] - paramArrayOfFloat[(j - 2)];
      f4 = paramArrayOfFloat[(paramInt1 + 3)] - paramArrayOfFloat[(j - 1)];
    }
    else
    {
      f3 = paramArrayOfFloat[(paramInt1 + 4)] - paramArrayOfFloat[paramInt1];
      f4 = paramArrayOfFloat[(paramInt1 + 5)] - paramArrayOfFloat[(paramInt1 + 1)];
    }
    float f1;
    float f2;
    for (int k = paramInt1 + 2; k < j - 2; k += 2)
    {
      f1 = f3;
      f2 = f4;
      f3 = paramArrayOfFloat[(k + 2)] - paramArrayOfFloat[(k - 2)];
      f4 = paramArrayOfFloat[(k + 3)] - paramArrayOfFloat[(k - 1)];
      paramGeneralPath.curveTo(paramFloat2 + paramArrayOfFloat[(k - 2)] + paramFloat1 * f1, paramFloat3 + paramArrayOfFloat[(k - 1)] + paramFloat1 * f2, paramFloat2 + paramArrayOfFloat[k] - paramFloat1 * f3, paramFloat3 + paramArrayOfFloat[(k + 1)] - paramFloat1 * f4, paramFloat2 + paramArrayOfFloat[k], paramFloat3 + paramArrayOfFloat[(k + 1)]);
    }
    if (paramBoolean)
    {
      f1 = f3;
      f2 = f4;
      f3 = paramArrayOfFloat[paramInt1] - paramArrayOfFloat[(k - 2)];
      f4 = paramArrayOfFloat[(paramInt1 + 1)] - paramArrayOfFloat[(k - 1)];
      paramGeneralPath.curveTo(paramFloat2 + paramArrayOfFloat[(k - 2)] + paramFloat1 * f1, paramFloat3 + paramArrayOfFloat[(k - 1)] + paramFloat1 * f2, paramFloat2 + paramArrayOfFloat[k] - paramFloat1 * f3, paramFloat3 + paramArrayOfFloat[(k + 1)] - paramFloat1 * f4, paramFloat2 + paramArrayOfFloat[k], paramFloat3 + paramArrayOfFloat[(k + 1)]);
      f1 = f3;
      f2 = f4;
      f3 = paramArrayOfFloat[(paramInt1 + 2)] - paramArrayOfFloat[(j - 2)];
      f4 = paramArrayOfFloat[(paramInt1 + 3)] - paramArrayOfFloat[(j - 1)];
      paramGeneralPath.curveTo(paramFloat2 + paramArrayOfFloat[(j - 2)] + paramFloat1 * f1, paramFloat3 + paramArrayOfFloat[(j - 1)] + paramFloat1 * f2, paramFloat2 + paramArrayOfFloat[0] - paramFloat1 * f3, paramFloat3 + paramArrayOfFloat[1] - paramFloat1 * f4, paramFloat2 + paramArrayOfFloat[0], paramFloat3 + paramArrayOfFloat[1]);
      paramGeneralPath.closePath();
    }
    else
    {
      paramGeneralPath.curveTo(paramFloat2 + paramArrayOfFloat[(k - 2)] + paramFloat1 * f3, paramFloat3 + paramArrayOfFloat[(k - 1)] + paramFloat1 * f4, paramFloat2 + paramArrayOfFloat[k] - paramFloat1 * f3, paramFloat3 + paramArrayOfFloat[(k + 1)] - paramFloat1 * f4, paramFloat2 + paramArrayOfFloat[k], paramFloat3 + paramArrayOfFloat[(k + 1)]);
    }
    return paramGeneralPath;
  }
  
  public static GeneralPath stackSpline(GeneralPath paramGeneralPath, float[] paramArrayOfFloat, float paramFloat1, float paramFloat2, boolean paramBoolean, float paramFloat3, float paramFloat4)
  {
    for (int i = 0; (i < paramArrayOfFloat.length) && (!Float.isNaN(paramArrayOfFloat[i])); i++) {}
    return stackSpline(paramGeneralPath, paramArrayOfFloat, 0, i / 2, paramFloat1, paramFloat2, paramBoolean, paramFloat3, paramFloat4);
  }
  
  public static GeneralPath stackSpline(GeneralPath paramGeneralPath, float[] paramArrayOfFloat, int paramInt1, int paramInt2, float paramFloat1, float paramFloat2, boolean paramBoolean, float paramFloat3, float paramFloat4)
  {
    int i = 2 * paramInt2;
    int j = paramInt1 + i;
    if (i < 6) {
      throw new IllegalArgumentException("To create spline requires at least 3 points");
    }
    if (paramBoolean)
    {
      f3 = paramArrayOfFloat[(paramInt1 + 2)] - paramArrayOfFloat[(j - 2)];
      f4 = paramArrayOfFloat[(paramInt1 + 3)] - paramArrayOfFloat[(j - 1)];
    }
    else
    {
      f3 = paramArrayOfFloat[(paramInt1 + 4)] - paramArrayOfFloat[paramInt1];
      f4 = paramArrayOfFloat[(paramInt1 + 5)] - paramArrayOfFloat[(paramInt1 + 1)];
    }
    for (int k = paramInt1 + 2; k < j - 2; k += 2)
    {
      f1 = f3;
      f2 = f4;
      f3 = paramArrayOfFloat[(k + 2)] - paramArrayOfFloat[(k - 2)];
      f4 = paramArrayOfFloat[(k + 3)] - paramArrayOfFloat[(k - 1)];
      if ((Math.abs(paramArrayOfFloat[k] - paramArrayOfFloat[(k - 2)]) < paramFloat1) || (Math.abs(paramArrayOfFloat[(k + 1)] - paramArrayOfFloat[(k - 1)]) < paramFloat1)) {
        paramGeneralPath.lineTo(paramFloat3 + paramArrayOfFloat[k], paramFloat4 + paramArrayOfFloat[(k + 1)]);
      } else {
        paramGeneralPath.curveTo(paramFloat3 + paramArrayOfFloat[(k - 2)] + paramFloat2 * f1, paramFloat4 + paramArrayOfFloat[(k - 1)] + paramFloat2 * f2, paramFloat3 + paramArrayOfFloat[k] - paramFloat2 * f3, paramFloat4 + paramArrayOfFloat[(k + 1)] - paramFloat2 * f4, paramFloat3 + paramArrayOfFloat[k], paramFloat4 + paramArrayOfFloat[(k + 1)]);
      }
    }
    float f1 = f3;
    float f2 = f4;
    float f3 = paramArrayOfFloat[paramInt1] - paramArrayOfFloat[(k - 2)];
    float f4 = paramArrayOfFloat[(paramInt1 + 1)] - paramArrayOfFloat[(k - 1)];
    if ((Math.abs(paramArrayOfFloat[k] - paramArrayOfFloat[(k - 2)]) < paramFloat1) || (Math.abs(paramArrayOfFloat[(k + 1)] - paramArrayOfFloat[(k - 1)]) < paramFloat1)) {
      paramGeneralPath.lineTo(paramFloat3 + paramArrayOfFloat[k], paramFloat4 + paramArrayOfFloat[(k + 1)]);
    } else {
      paramGeneralPath.curveTo(paramFloat3 + paramArrayOfFloat[(k - 2)] + paramFloat2 * f1, paramFloat4 + paramArrayOfFloat[(k - 1)] + paramFloat2 * f2, paramFloat3 + paramArrayOfFloat[k] - paramFloat2 * f3, paramFloat4 + paramArrayOfFloat[(k + 1)] - paramFloat2 * f4, paramFloat3 + paramArrayOfFloat[k], paramFloat4 + paramArrayOfFloat[(k + 1)]);
    }
    if (paramBoolean)
    {
      if ((Math.abs(paramArrayOfFloat[(j - 2)] - paramArrayOfFloat[0]) < paramFloat1) || (Math.abs(paramArrayOfFloat[(j - 1)] - paramArrayOfFloat[1]) < paramFloat1))
      {
        paramGeneralPath.lineTo(paramFloat3 + paramArrayOfFloat[0], paramFloat4 + paramArrayOfFloat[1]);
      }
      else
      {
        f1 = f3;
        f2 = f4;
        f3 = paramArrayOfFloat[(paramInt1 + 2)] - paramArrayOfFloat[(j - 2)];
        f4 = paramArrayOfFloat[(paramInt1 + 3)] - paramArrayOfFloat[(j - 1)];
        paramGeneralPath.curveTo(paramFloat3 + paramArrayOfFloat[(j - 2)] + paramFloat2 * f1, paramFloat4 + paramArrayOfFloat[(j - 1)] + paramFloat2 * f2, paramFloat3 + paramArrayOfFloat[0] - paramFloat2 * f3, paramFloat4 + paramArrayOfFloat[1] - paramFloat2 * f4, paramFloat3 + paramArrayOfFloat[0], paramFloat4 + paramArrayOfFloat[1]);
      }
      paramGeneralPath.closePath();
    }
    return paramGeneralPath;
  }
  
  public static void expand(Rectangle2D paramRectangle2D, double paramDouble)
  {
    paramRectangle2D.setRect(paramRectangle2D.getX() - paramDouble, paramRectangle2D.getY() - paramDouble, paramRectangle2D.getWidth() + 2.0D * paramDouble, paramRectangle2D.getHeight() + 2.0D * paramDouble);
  }
  
  public static void setBounds(VisualItem paramVisualItem, Shape paramShape, BasicStroke paramBasicStroke)
  {
    Object localObject;
    double d1;
    double d2;
    double d3;
    double d4;
    double d5;
    if ((paramShape instanceof RectangularShape))
    {
      localObject = (RectangularShape)paramShape;
      d1 = ((RectangularShape)localObject).getX();
      d2 = ((RectangularShape)localObject).getY();
      d3 = ((RectangularShape)localObject).getWidth();
      d4 = ((RectangularShape)localObject).getHeight();
    }
    else if ((paramShape instanceof Line2D))
    {
      localObject = (Line2D)paramShape;
      d1 = ((Line2D)localObject).getX1();
      d2 = ((Line2D)localObject).getY1();
      d3 = ((Line2D)localObject).getX2();
      d4 = ((Line2D)localObject).getY2();
      if (d3 < d1)
      {
        d5 = d1;
        d1 = d3;
        d3 = d5 - d1;
      }
      else
      {
        d3 -= d1;
      }
      if (d4 < d2)
      {
        d5 = d2;
        d2 = d4;
        d4 = d5 - d2;
      }
      else
      {
        d4 -= d2;
      }
    }
    else
    {
      localObject = paramShape.getBounds2D();
      d1 = ((Rectangle2D)localObject).getX();
      d2 = ((Rectangle2D)localObject).getY();
      d3 = ((Rectangle2D)localObject).getWidth();
      d4 = ((Rectangle2D)localObject).getHeight();
    }
    if ((paramBasicStroke != null) && ((d5 = paramBasicStroke.getLineWidth()) > 1.0D))
    {
      double d6 = d5 / 2.0D;
      d1 -= d6;
      d2 -= d6;
      d3 += d5;
      d4 += d5;
    }
    paramVisualItem.setBounds(d1, d2, d3, d4);
  }
  
  public static void paint(Graphics2D paramGraphics2D, VisualItem paramVisualItem, Shape paramShape, BasicStroke paramBasicStroke, int paramInt)
  {
    if (paramInt == 0) {
      return;
    }
    Color localColor1 = ColorLib.getColor(paramVisualItem.getStrokeColor());
    Color localColor2 = ColorLib.getColor(paramVisualItem.getFillColor());
    int i = ((paramInt == 1) || (paramInt == 3)) && (localColor1.getAlpha() != 0) ? 1 : 0;
    int j = ((paramInt == 2) || (paramInt == 3)) && (localColor2.getAlpha() != 0) ? 1 : 0;
    if ((i == 0) && (j == 0)) {
      return;
    }
    Stroke localStroke = null;
    if (i != 0)
    {
      localStroke = paramGraphics2D.getStroke();
      paramGraphics2D.setStroke(paramBasicStroke);
    }
    AffineTransform localAffineTransform = paramGraphics2D.getTransform();
    double d5 = Math.max(localAffineTransform.getScaleX(), localAffineTransform.getScaleY());
    if (d5 > 1.5D)
    {
      if (j != 0)
      {
        paramGraphics2D.setPaint(localColor2);
        paramGraphics2D.fill(paramShape);
      }
      if (i != 0)
      {
        paramGraphics2D.setPaint(localColor1);
        paramGraphics2D.draw(paramShape);
      }
    }
    else
    {
      Object localObject;
      int k;
      int m;
      int n;
      int i1;
      if ((paramShape instanceof RectangularShape))
      {
        localObject = (RectangularShape)paramShape;
        double d1 = ((RectangularShape)localObject).getX();
        double d3 = ((RectangularShape)localObject).getWidth();
        double d2 = ((RectangularShape)localObject).getY();
        double d4 = ((RectangularShape)localObject).getHeight();
        k = (int)d1;
        m = (int)d2;
        n = (int)(d3 + d1 - k);
        i1 = (int)(d4 + d2 - m);
        if ((paramShape instanceof Rectangle2D))
        {
          if (j != 0)
          {
            paramGraphics2D.setPaint(localColor2);
            paramGraphics2D.fillRect(k, m, n, i1);
          }
          if (i != 0)
          {
            paramGraphics2D.setPaint(localColor1);
            paramGraphics2D.drawRect(k, m, n, i1);
          }
        }
        else if ((paramShape instanceof RoundRectangle2D))
        {
          RoundRectangle2D localRoundRectangle2D = (RoundRectangle2D)paramShape;
          int i2 = (int)localRoundRectangle2D.getArcWidth();
          int i3 = (int)localRoundRectangle2D.getArcHeight();
          if (j != 0)
          {
            paramGraphics2D.setPaint(localColor2);
            paramGraphics2D.fillRoundRect(k, m, n, i1, i2, i3);
          }
          if (i != 0)
          {
            paramGraphics2D.setPaint(localColor1);
            paramGraphics2D.drawRoundRect(k, m, n, i1, i2, i3);
          }
        }
        else if ((paramShape instanceof Ellipse2D))
        {
          if (j != 0)
          {
            paramGraphics2D.setPaint(localColor2);
            paramGraphics2D.fillOval(k, m, n, i1);
          }
          if (i != 0)
          {
            paramGraphics2D.setPaint(localColor1);
            paramGraphics2D.drawOval(k, m, n, i1);
          }
        }
        else
        {
          if (j != 0)
          {
            paramGraphics2D.setPaint(localColor2);
            paramGraphics2D.fill(paramShape);
          }
          if (i != 0)
          {
            paramGraphics2D.setPaint(localColor1);
            paramGraphics2D.draw(paramShape);
          }
        }
      }
      else if ((paramShape instanceof Line2D))
      {
        if (i != 0)
        {
          localObject = (Line2D)paramShape;
          k = (int)(((Line2D)localObject).getX1() + 0.5D);
          m = (int)(((Line2D)localObject).getY1() + 0.5D);
          n = (int)(((Line2D)localObject).getX2() + 0.5D);
          i1 = (int)(((Line2D)localObject).getY2() + 0.5D);
          paramGraphics2D.setPaint(localColor1);
          paramGraphics2D.drawLine(k, m, n, i1);
        }
      }
      else
      {
        if (j != 0)
        {
          paramGraphics2D.setPaint(localColor2);
          paramGraphics2D.fill(paramShape);
        }
        if (i != 0)
        {
          paramGraphics2D.setPaint(localColor1);
          paramGraphics2D.draw(paramShape);
        }
      }
    }
    if (i != 0) {
      paramGraphics2D.setStroke(localStroke);
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\GraphicsLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */