package prefuse.util;

import java.io.IOException;
import java.util.Properties;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import prefuse.util.io.IOLib;

public class PrefuseConfig
  extends Properties
{
  private static final Logger s_logger = Logger.getLogger(PrefuseConfig.class.getName());
  private static final PrefuseConfig s_config = new PrefuseConfig();
  
  public static PrefuseConfig getConfig()
  {
    return s_config;
  }
  
  private PrefuseConfig()
  {
    setDefaults();
    String str1;
    try
    {
      str1 = System.getProperty("prefuse.config");
    }
    catch (Exception localException1)
    {
      str1 = null;
    }
    if (str1 == null) {
      str1 = "prefuse.conf";
    }
    try
    {
      load(IOLib.streamFromString(str1));
      s_logger.info("Loaded config file: " + str1);
    }
    catch (Exception localException2) {}
    String str2 = getProperty("util.logdir");
    String str3 = getProperty("util.logfile");
    if (str2 != null) {
      try
      {
        Logger localLogger = Logger.getLogger("prefuse");
        localLogger.setUseParentHandlers(false);
        FileHandler localFileHandler = new FileHandler(str2 + "/" + str3);
        localFileHandler.setFormatter(new SimpleFormatter());
        localLogger.addHandler(localFileHandler);
      }
      catch (IOException localIOException)
      {
        localIOException.printStackTrace();
      }
    }
  }
  
  public static String get(String paramString)
  {
    return s_config.getProperty(paramString);
  }
  
  public static int getInt(String paramString)
  {
    String str = s_config.getProperty(paramString);
    try
    {
      return Integer.parseInt(str);
    }
    catch (NumberFormatException localNumberFormatException) {}
    return Integer.MIN_VALUE;
  }
  
  public static long getLong(String paramString)
  {
    String str = s_config.getProperty(paramString);
    try
    {
      return Long.parseLong(str);
    }
    catch (NumberFormatException localNumberFormatException) {}
    return Long.MIN_VALUE;
  }
  
  public static float getFloat(String paramString)
  {
    String str = s_config.getProperty(paramString);
    try
    {
      return Float.parseFloat(str);
    }
    catch (NumberFormatException localNumberFormatException) {}
    return NaN.0F;
  }
  
  public static double getDouble(String paramString)
  {
    String str = s_config.getProperty(paramString);
    try
    {
      return Double.parseDouble(str);
    }
    catch (NumberFormatException localNumberFormatException) {}
    return NaN.0D;
  }
  
  public static boolean getBoolean(String paramString)
  {
    String str = s_config.getProperty(paramString);
    return "true".equalsIgnoreCase(str);
  }
  
  private void setDefaults()
  {
    setProperty("size.scale2D", "0.5");
    setProperty("activity.threadPriority", "6");
    setProperty("data.delimiter", ".");
    setProperty("data.graph.nodeGroup", "nodes");
    setProperty("data.graph.edgeGroup", "edges");
    setProperty("data.visual.fieldPrefix", "_");
    setProperty("data.io.worker.threadPriority", String.valueOf(5));
    setProperty("data.filter.optimizeThreshold", "300");
    setProperty("data.graph.sourceKey", "source");
    setProperty("data.graph.targetKey", "target");
    setProperty("data.tree.sourceKey", "parent");
    setProperty("data.tree.targetKey", "child");
    setProperty("visualization.allItems", "_all_");
    setProperty("visualization.focusItems", "_focus_");
    setProperty("visualization.selectedItems", "_selected_");
    setProperty("visualization.searchItems", "_search_");
    setProperty("util.logfile", "prefuse_log_%g.txt");
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\PrefuseConfig.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */