package prefuse.util.ui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class JRangeSlider
  extends JComponent
  implements MouseListener, MouseMotionListener, KeyListener
{
  public static final int VERTICAL = 0;
  public static final int HORIZONTAL = 1;
  public static final int LEFTRIGHT_TOPBOTTOM = 0;
  public static final int RIGHTLEFT_BOTTOMTOP = 1;
  public static final int PREFERRED_BREADTH = 16;
  public static final int PREFERRED_LENGTH = 300;
  protected static final int ARROW_SZ = 16;
  protected static final int ARROW_WIDTH = 8;
  protected static final int ARROW_HEIGHT = 4;
  protected BoundedRangeModel model;
  protected int orientation;
  protected int direction;
  protected boolean empty;
  protected int increment = 1;
  protected int minExtent = 0;
  protected ArrayList listeners = new ArrayList();
  protected ChangeEvent changeEvent = null;
  protected ChangeListener lstnr;
  protected Color thumbColor = new Color(150, 180, 220);
  static final int PICK_NONE = 0;
  static final int PICK_LEFT_OR_TOP = 1;
  static final int PICK_THUMB = 2;
  static final int PICK_RIGHT_OR_BOTTOM = 3;
  int pick;
  int pickOffsetLow;
  int pickOffsetHigh;
  int mouse;
  
  public JRangeSlider(int paramInt1, int paramInt2, int paramInt3, int paramInt4, int paramInt5)
  {
    this(new DefaultBoundedRangeModel(paramInt3, paramInt4, paramInt1, paramInt2), paramInt5, 0);
  }
  
  public JRangeSlider(int paramInt1, int paramInt2, int paramInt3, int paramInt4, int paramInt5, int paramInt6)
  {
    this(new DefaultBoundedRangeModel(paramInt3, paramInt4, paramInt1, paramInt2), paramInt5, paramInt6);
  }
  
  public JRangeSlider(BoundedRangeModel paramBoundedRangeModel, int paramInt1, int paramInt2)
  {
    super.setFocusable(true);
    this.model = paramBoundedRangeModel;
    this.orientation = paramInt1;
    this.direction = paramInt2;
    setForeground(Color.LIGHT_GRAY);
    this.lstnr = createListener();
    paramBoundedRangeModel.addChangeListener(this.lstnr);
    addMouseListener(this);
    addMouseMotionListener(this);
    addKeyListener(this);
  }
  
  protected ChangeListener createListener()
  {
    return new RangeSliderChangeListener();
  }
  
  public int getLowValue()
  {
    return this.model.getValue();
  }
  
  public void setLowValue(int paramInt)
  {
    int i = this.model.getValue() - paramInt + this.model.getExtent();
    this.model.setRangeProperties(paramInt, i, this.model.getMinimum(), this.model.getMaximum(), false);
    this.model.setValue(paramInt);
  }
  
  public int getHighValue()
  {
    return this.model.getValue() + this.model.getExtent();
  }
  
  public void setHighValue(int paramInt)
  {
    this.model.setExtent(paramInt - this.model.getValue());
  }
  
  public void setRange(int paramInt1, int paramInt2)
  {
    this.model.setRangeProperties(paramInt1, paramInt2 - paramInt1, this.model.getMinimum(), this.model.getMaximum(), false);
  }
  
  public int getMinimum()
  {
    return this.model.getMinimum();
  }
  
  public void setMinimum(int paramInt)
  {
    this.model.setMinimum(paramInt);
  }
  
  public int getMaximum()
  {
    return this.model.getMaximum();
  }
  
  public void setMaximum(int paramInt)
  {
    this.model.setMaximum(paramInt);
  }
  
  public void setMinExtent(int paramInt)
  {
    this.minExtent = paramInt;
  }
  
  public void setEmpty(boolean paramBoolean)
  {
    this.empty = paramBoolean;
    repaint();
  }
  
  public Color getThumbColor()
  {
    return this.thumbColor;
  }
  
  public void setThumbColor(Color paramColor)
  {
    this.thumbColor = paramColor;
  }
  
  public BoundedRangeModel getModel()
  {
    return this.model;
  }
  
  public void setModel(BoundedRangeModel paramBoundedRangeModel)
  {
    this.model.removeChangeListener(this.lstnr);
    this.model = paramBoundedRangeModel;
    this.model.addChangeListener(this.lstnr);
    repaint();
  }
  
  public void addChangeListener(ChangeListener paramChangeListener)
  {
    if (!this.listeners.contains(paramChangeListener)) {
      this.listeners.add(paramChangeListener);
    }
  }
  
  public void removeChangeListener(ChangeListener paramChangeListener)
  {
    this.listeners.remove(paramChangeListener);
  }
  
  protected void fireChangeEvent()
  {
    repaint();
    if (this.changeEvent == null) {
      this.changeEvent = new ChangeEvent(this);
    }
    Iterator localIterator = this.listeners.iterator();
    while (localIterator.hasNext()) {
      ((ChangeListener)localIterator.next()).stateChanged(this.changeEvent);
    }
  }
  
  public Dimension getPreferredSize()
  {
    if (this.orientation == 0) {
      return new Dimension(16, 300);
    }
    return new Dimension(300, 16);
  }
  
  protected void customPaint(Graphics2D paramGraphics2D, int paramInt1, int paramInt2) {}
  
  public void paintComponent(Graphics paramGraphics)
  {
    Rectangle localRectangle = getBounds();
    int i = (int)localRectangle.getWidth() - 1;
    int j = (int)localRectangle.getHeight() - 1;
    int k = toScreen(getLowValue());
    int m = toScreen(getHighValue());
    if (this.empty) {
      if (this.direction == 0)
      {
        k = 16;
        m = this.orientation == 0 ? j - 16 : i - 16;
      }
      else
      {
        k = this.orientation == 0 ? j - 16 : i - 16;
        m = 16;
      }
    }
    Graphics2D localGraphics2D = (Graphics2D)paramGraphics;
    localGraphics2D.setColor(getBackground());
    localGraphics2D.fillRect(0, 0, i, j);
    localGraphics2D.setColor(getForeground());
    localGraphics2D.drawRect(0, 0, i, j);
    customPaint(localGraphics2D, i, j);
    localGraphics2D.setStroke(new BasicStroke(1.0F));
    if (this.orientation == 0)
    {
      if (this.direction == 0)
      {
        localGraphics2D.setColor(getForeground());
        localGraphics2D.fillRect(0, k - 16, i, 15);
        paint3DRectLighting(localGraphics2D, 0, k - 16, i, 15);
        if (this.thumbColor != null)
        {
          localGraphics2D.setColor(this.thumbColor);
          localGraphics2D.fillRect(0, k, i, m - k - 1);
          paint3DRectLighting(localGraphics2D, 0, k, i, m - k - 1);
        }
        localGraphics2D.setColor(getForeground());
        localGraphics2D.fillRect(0, m, i, 15);
        paint3DRectLighting(localGraphics2D, 0, m, i, 15);
        localGraphics2D.setColor(Color.black);
        paintArrow(localGraphics2D, (i - 8) / 2.0D, k - 16 + 6.0D, 8, 4, true);
        paintArrow(localGraphics2D, (i - 8) / 2.0D, m + 6.0D, 8, 4, false);
      }
      else
      {
        localGraphics2D.setColor(getForeground());
        localGraphics2D.fillRect(0, k, i, 15);
        paint3DRectLighting(localGraphics2D, 0, k, i, 15);
        if (this.thumbColor != null)
        {
          localGraphics2D.setColor(this.thumbColor);
          localGraphics2D.fillRect(0, m, i, k - m - 1);
          paint3DRectLighting(localGraphics2D, 0, m, i, k - m - 1);
        }
        localGraphics2D.setColor(getForeground());
        localGraphics2D.fillRect(0, m - 16, i, 15);
        paint3DRectLighting(localGraphics2D, 0, m - 16, i, 15);
        localGraphics2D.setColor(Color.black);
        paintArrow(localGraphics2D, (i - 8) / 2.0D, k + 6.0D, 8, 4, false);
        paintArrow(localGraphics2D, (i - 8) / 2.0D, m - 16 + 6.0D, 8, 4, true);
      }
    }
    else if (this.direction == 0)
    {
      localGraphics2D.setColor(getForeground());
      localGraphics2D.fillRect(k - 16, 0, 15, j);
      paint3DRectLighting(localGraphics2D, k - 16, 0, 15, j);
      if (this.thumbColor != null)
      {
        localGraphics2D.setColor(this.thumbColor);
        localGraphics2D.fillRect(k, 0, m - k - 1, j);
        paint3DRectLighting(localGraphics2D, k, 0, m - k - 1, j);
      }
      localGraphics2D.setColor(getForeground());
      localGraphics2D.fillRect(m, 0, 15, j);
      paint3DRectLighting(localGraphics2D, m, 0, 15, j);
      localGraphics2D.setColor(Color.black);
      paintArrow(localGraphics2D, k - 16 + 6.0D, (j - 8) / 2.0D, 4, 8, true);
      paintArrow(localGraphics2D, m + 6.0D, (j - 8) / 2.0D, 4, 8, false);
    }
    else
    {
      localGraphics2D.setColor(getForeground());
      localGraphics2D.fillRect(k, 0, 15, j);
      paint3DRectLighting(localGraphics2D, k, 0, 15, j);
      if (this.thumbColor != null)
      {
        localGraphics2D.setColor(this.thumbColor);
        localGraphics2D.fillRect(m, 0, k - m - 1, j);
        paint3DRectLighting(localGraphics2D, m, 0, k - m - 1, j);
      }
      localGraphics2D.setColor(getForeground());
      localGraphics2D.fillRect(m - 16, 0, 15, j);
      paint3DRectLighting(localGraphics2D, m - 16, 0, 15, j);
      localGraphics2D.setColor(Color.black);
      paintArrow(localGraphics2D, k + 6.0D, (j - 8) / 2.0D, 4, 8, true);
      paintArrow(localGraphics2D, m - 16 + 6.0D, (j - 8) / 2.0D, 4, 8, false);
    }
  }
  
  protected void paintArrow(Graphics2D paramGraphics2D, double paramDouble1, double paramDouble2, int paramInt1, int paramInt2, boolean paramBoolean)
  {
    int i = (int)(paramDouble1 + 0.5D);
    int j = (int)(paramDouble2 + 0.5D);
    int k;
    if (this.orientation == 0)
    {
      if (paramInt1 % 2 == 0) {
        paramInt1 -= 1;
      }
      if (paramBoolean) {
        for (k = 0; k < paramInt1 / 2 + 1; k++) {
          paramGraphics2D.drawLine(i + k, j + k, i + paramInt1 - k - 1, j + k);
        }
      } else {
        for (k = 0; k < paramInt1 / 2 + 1; k++) {
          paramGraphics2D.drawLine(i + paramInt1 / 2 - k, j + k, i + paramInt1 - paramInt1 / 2 + k - 1, j + k);
        }
      }
    }
    else
    {
      if (paramInt2 % 2 == 0) {
        paramInt2 -= 1;
      }
      if (paramBoolean) {
        for (k = 0; k < paramInt2 / 2 + 1; k++) {
          paramGraphics2D.drawLine(i + k, j + k, i + k, j + paramInt2 - k - 1);
        }
      } else {
        for (k = 0; k < paramInt2 / 2 + 1; k++) {
          paramGraphics2D.drawLine(i + k, j + paramInt2 / 2 - k, i + k, j + paramInt2 - paramInt2 / 2 + k - 1);
        }
      }
    }
  }
  
  protected void paint3DRectLighting(Graphics2D paramGraphics2D, int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    paramGraphics2D.setColor(Color.white);
    paramGraphics2D.drawLine(paramInt1 + 1, paramInt2 + 1, paramInt1 + 1, paramInt2 + paramInt4 - 1);
    paramGraphics2D.drawLine(paramInt1 + 1, paramInt2 + 1, paramInt1 + paramInt3 - 1, paramInt2 + 1);
    paramGraphics2D.setColor(Color.gray);
    paramGraphics2D.drawLine(paramInt1 + 1, paramInt2 + paramInt4 - 1, paramInt1 + paramInt3 - 1, paramInt2 + paramInt4 - 1);
    paramGraphics2D.drawLine(paramInt1 + paramInt3 - 1, paramInt2 + 1, paramInt1 + paramInt3 - 1, paramInt2 + paramInt4 - 1);
    paramGraphics2D.setColor(Color.darkGray);
    paramGraphics2D.drawLine(paramInt1, paramInt2 + paramInt4, paramInt1 + paramInt3, paramInt2 + paramInt4);
    paramGraphics2D.drawLine(paramInt1 + paramInt3, paramInt2, paramInt1 + paramInt3, paramInt2 + paramInt4);
  }
  
  protected int toLocal(int paramInt)
  {
    Dimension localDimension = getSize();
    int i = getMinimum();
    double d;
    if (this.orientation == 0) {
      d = (localDimension.height - 32) / (getMaximum() - i);
    } else {
      d = (localDimension.width - 32) / (getMaximum() - i);
    }
    if (this.direction == 0) {
      return (int)((paramInt - 16) / d + i + 0.5D);
    }
    if (this.orientation == 0) {
      return (int)((localDimension.height - paramInt - 16) / d + i + 0.5D);
    }
    return (int)((localDimension.width - paramInt - 16) / d + i + 0.5D);
  }
  
  protected int toScreen(int paramInt)
  {
    Dimension localDimension = getSize();
    int i = getMinimum();
    double d;
    if (this.orientation == 0) {
      d = (localDimension.height - 32) / (getMaximum() - i);
    } else {
      d = (localDimension.width - 32) / (getMaximum() - i);
    }
    if (this.direction == 0) {
      return (int)(16.0D + (paramInt - i) * d + 0.5D);
    }
    if (this.orientation == 0) {
      return (int)(localDimension.height - (paramInt - i) * d - 16.0D + 0.5D);
    }
    return (int)(localDimension.width - (paramInt - i) * d - 16.0D + 0.5D);
  }
  
  protected double toScreenDouble(int paramInt)
  {
    Dimension localDimension = getSize();
    int i = getMinimum();
    double d;
    if (this.orientation == 0) {
      d = (localDimension.height - 32) / (getMaximum() + 1 - i);
    } else {
      d = (localDimension.width - 32) / (getMaximum() + 1 - i);
    }
    if (this.direction == 0) {
      return 16.0D + (paramInt - i) * d;
    }
    if (this.orientation == 0) {
      return localDimension.height - (paramInt - i) * d - 16.0D;
    }
    return localDimension.width - (paramInt - i) * d - 16.0D;
  }
  
  private int pickHandle(int paramInt)
  {
    int i = toScreen(getLowValue());
    int j = toScreen(getHighValue());
    int k = 0;
    if (this.direction == 0)
    {
      if ((paramInt > i - 16) && (paramInt < i)) {
        k = 1;
      } else if ((paramInt >= i) && (paramInt <= j)) {
        k = 2;
      } else if ((paramInt > j) && (paramInt < j + 16)) {
        k = 3;
      }
    }
    else if ((paramInt > i) && (paramInt < i + 16)) {
      k = 1;
    } else if ((paramInt <= i) && (paramInt >= j)) {
      k = 2;
    } else if ((paramInt > j - 16) && (paramInt < j)) {
      k = 3;
    }
    return k;
  }
  
  private void offset(int paramInt)
  {
    this.model.setValue(this.model.getValue() + paramInt);
  }
  
  public void mousePressed(MouseEvent paramMouseEvent)
  {
    if (this.orientation == 0)
    {
      this.pick = pickHandle(paramMouseEvent.getY());
      this.pickOffsetLow = (paramMouseEvent.getY() - toScreen(getLowValue()));
      this.pickOffsetHigh = (paramMouseEvent.getY() - toScreen(getHighValue()));
      this.mouse = paramMouseEvent.getY();
    }
    else
    {
      this.pick = pickHandle(paramMouseEvent.getX());
      this.pickOffsetLow = (paramMouseEvent.getX() - toScreen(getLowValue()));
      this.pickOffsetHigh = (paramMouseEvent.getX() - toScreen(getHighValue()));
      this.mouse = paramMouseEvent.getX();
    }
    repaint();
  }
  
  public void mouseDragged(MouseEvent paramMouseEvent)
  {
    requestFocus();
    int i = this.orientation == 0 ? paramMouseEvent.getY() : paramMouseEvent.getX();
    int j = getMinimum();
    int k = getMaximum();
    int m = getLowValue();
    int n = getHighValue();
    switch (this.pick)
    {
    case 1: 
      int i1 = toLocal(i - this.pickOffsetLow);
      if (i1 < j) {
        i1 = j;
      }
      if (i1 > k) {
        i1 = k;
      }
      if (i1 > n - this.minExtent) {
        i1 = n - this.minExtent;
      }
      setLowValue(i1);
      break;
    case 3: 
      int i2 = toLocal(i - this.pickOffsetHigh);
      if (i2 < j) {
        i2 = j;
      }
      if (i2 > k) {
        i2 = k;
      }
      if (i2 < m + this.minExtent) {
        i2 = m + this.minExtent;
      }
      setHighValue(i2);
      break;
    case 2: 
      int i3 = toLocal(i - this.pickOffsetLow) - m;
      if ((i3 < 0) && (m + i3 < j)) {
        i3 = j - m;
      }
      if ((i3 > 0) && (n + i3 > k)) {
        i3 = k - n;
      }
      if (i3 != 0) {
        offset(i3);
      }
      break;
    }
  }
  
  public void mouseReleased(MouseEvent paramMouseEvent)
  {
    this.pick = 0;
    repaint();
  }
  
  public void mouseMoved(MouseEvent paramMouseEvent)
  {
    if (this.orientation == 0) {
      switch (pickHandle(paramMouseEvent.getY()))
      {
      case 1: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 3: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 2: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 0: 
        setCursor(Cursor.getPredefinedCursor(0));
      }
    } else {
      switch (pickHandle(paramMouseEvent.getX()))
      {
      case 1: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 3: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 2: 
        setCursor(Cursor.getPredefinedCursor(0));
        break;
      case 0: 
        setCursor(Cursor.getPredefinedCursor(0));
      }
    }
  }
  
  public void mouseClicked(MouseEvent paramMouseEvent) {}
  
  public void mouseEntered(MouseEvent paramMouseEvent) {}
  
  public void mouseExited(MouseEvent paramMouseEvent) {}
  
  private void grow(int paramInt)
  {
    this.model.setRangeProperties(this.model.getValue() - paramInt, this.model.getExtent() + 2 * paramInt, this.model.getMinimum(), this.model.getMaximum(), false);
  }
  
  public void keyPressed(KeyEvent paramKeyEvent)
  {
    int i = paramKeyEvent.getKeyCode();
    int j = this.orientation == 0 ? 1 : 0;
    int k = i == 40 ? 1 : 0;
    int m = i == 38 ? 1 : 0;
    int n = i == 37 ? 1 : 0;
    int i1 = i == 39 ? 1 : 0;
    int i2 = getMinimum();
    int i3 = getMaximum();
    int i4 = getLowValue();
    int i5 = getHighValue();
    if (((j != 0) && (i1 != 0)) || ((j == 0) && (m != 0)))
    {
      if ((i4 - this.increment >= i2) && (i5 + this.increment <= i3)) {
        grow(this.increment);
      }
    }
    else if (((j != 0) && (n != 0)) || ((j == 0) && (k != 0)))
    {
      if (i5 - i4 >= 2 * this.increment) {
        grow(-1 * this.increment);
      }
    }
    else if (((j != 0) && (k != 0)) || ((j == 0) && (n != 0)))
    {
      if (i4 - this.increment >= i2) {
        offset(-this.increment);
      }
    }
    else if (((j != 0) && (m != 0)) || ((j == 0) && (i1 != 0) && (i5 + this.increment <= i3))) {
      offset(this.increment);
    }
  }
  
  public void keyReleased(KeyEvent paramKeyEvent) {}
  
  public void keyTyped(KeyEvent paramKeyEvent) {}
  
  protected class RangeSliderChangeListener
    implements ChangeListener
  {
    protected RangeSliderChangeListener() {}
    
    public void stateChanged(ChangeEvent paramChangeEvent)
    {
      JRangeSlider.this.fireChangeEvent();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\ui\JRangeSlider.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */